Attribute VB_Name = "TestBsplineCurve"
Option Explicit

Sub TestBsplineCurve()
    Dim oElt As BsplineCurveElement
    Dim oCurve As BsplineCurve
    Dim aPoles() As Point3d
    Dim aKnots() As Double, aUniformKnots() As Double
    Dim aTangents() As Ray3d
    Dim nKnot As Long, nPt As Long
    Dim i As Long
    
    ' Ctor
    Set oCurve = New BsplineCurve
    
    ' put_Order
    oCurve.order = 4
    
    ' get_Order, SetPoles (poles are zeroed out)
    ReDim aPoles(1 To oCurve.order)
    oCurve.SetPoles aPoles
    
    ' get_PolesCount, put_Pole, get_Pole
    For i = 0 To oCurve.PolesCount - 1
        oCurve.Pole(i) = PointOnXYCircle(i, oCurve.PolesCount, 1#)
        oCurve.Pole(i) = Point3dAdd(oCurve.Pole(i), Point3dFromXYZ(0#, 0#, i))
    Next i
    
    ' CreateBsplineCurveElement
    Set oElt = CreateBsplineCurveElement1(Nothing, oCurve)
    oElt.color = 1
    ActiveModelReference.AddElement oElt
    oElt.Redraw
    
    ' ExtractBsplineCurve
    Set oCurve = oElt.ExtractBsplineCurve
    
    ' get_Closed, set_Closed (adds uniform knots)
    oCurve.Closed = Not (oCurve.Closed)
    
    ' get_Knot, get_KnotsCount, set_Knot (makes curve nonuniform)
    oCurve.Knot(oCurve.KnotsCount - 1) = oCurve.Knot(oCurve.KnotsCount - 1) / 2 + 0.5
    
    ' UpdateFromBsplineCurve (replaces elt in file)
    oElt.color = oElt.color + 1
    oElt.UpdateFromBsplineCurve oCurve
    oElt.Redraw

    ' get_Weight, put_Weight (makes curve rational)
    oCurve.weight(0) = 0.5
    oCurve.weight(0) = oCurve.weight(0) / 2
    oElt.color = oElt.color + 1
    oElt.UpdateFromBsplineCurve oCurve
    oElt.Redraw
    
    ' Bspline.ComputeKnotsCount
    nKnot = Application.Bspline.ComputeKnotsCount(oCurve.PolesCount, oCurve.order, oCurve.Closed)
    ReDim aKnots(1 To nKnot)
    
    ' SetKnots
    For i = LBound(aKnots) To UBound(aKnots)
        aKnots(i) = CDbl(i - LBound(aKnots) + 1) / (2 * (nKnot + 1))
    Next i
    oCurve.SetKnots aKnots
    oElt.color = oElt.color + 1
    oElt.UpdateFromBsplineCurve oCurve
    oElt.Redraw
    
    ' SetKnots (empty for uniform)
    oCurve.SetKnots aUniformKnots
    oElt.color = oElt.color + 1
    oElt.UpdateFromBsplineCurve oCurve
    oElt.Redraw
End Sub

