Attribute VB_Name = "TestBsplineSurface"
Option Explicit

Sub TestBsplineSurface()
    Dim oElt As BsplineSurfaceElement
    Dim oSurface As BsplineSurface
    Dim aPoles() As Point3d, aPoints() As Point3d
    Dim aBound() As Point2d
    Dim r0 As Double, r1 As Double
    Dim i As Long, n As Long, Row As Long, Col As Long
    
    ' Ctor
    Set oSurface = New BsplineSurface
    
    ' put_UOrder, put_VOrder (sets numRules too)
    oSurface.VOrder = 5
    oSurface.UOrder = 8
    
    ' SetPoles (zero poles)
    ReDim aPoles(1 To oSurface.VOrder, 1 To oSurface.UOrder)
    oSurface.SetPoles aPoles
    
    ' get_UPolesCount, get_VPolesCount, put_Pole, get_Pole
    For Row = 0 To oSurface.VPolesCount - 1
        For Col = 0 To oSurface.UPolesCount - 1
            r0 = CDbl(Row) / CDbl(oSurface.VPolesCount - 1)
            r1 = CDbl(Col) / CDbl(oSurface.UPolesCount - 1)
            oSurface.Pole(Row, Col) = Point3dFromXYZ(r0, r1, 0#)
            oSurface.Pole(Row, Col) = Point3dAdd(oSurface.Pole(Row, Col), Point3dFromXYZ(0#, 0#, 0.125 * Sin(2 * Pi * r1)))
        Next Col
    Next Row
    
    ' CreateBsplineSurfaceElement
    Set oElt = CreateBsplineSurfaceElement1(Nothing, oSurface)
    oElt.color = 1
    ActiveModelReference.AddElement oElt
    oElt.Redraw
    
    ' AddBoundary (closed holes)
    ReDim aBound(0 To 3)
    aBound(0) = Point2dFromXY(0.1, 0.1)
    aBound(1) = Point2dFromXY(0.1, 0.3)
    aBound(2) = Point2dFromXY(0.3, 0.3)
    aBound(3) = Point2dFromXY(0.1, 0.1)
    oSurface.AddBound aBound
    ReDim aBound(0 To 4)
    aBound(0) = Point2dFromXY(0.6, 0.6)
    aBound(1) = Point2dFromXY(0.9, 0.6)
    aBound(2) = Point2dFromXY(0.9, 0.9)
    aBound(3) = Point2dFromXY(0.6, 0.9)
    aBound(4) = Point2dFromXY(0.6, 0.6)
    oSurface.AddBound aBound
    ReDim aBound(0 To 3)
    aBound(0) = Point2dFromXY(0.3, 0.9)
    aBound(1) = Point2dFromXY(0.1, 0.6)
    aBound(2) = Point2dFromXY(0.5, 0.6)
    aBound(3) = Point2dFromXY(0.3, 0.9)
    oSurface.AddBound aBound
    oElt.UpdateFromBsplineSurface oSurface
    oElt.Redraw
    
    ' set_BoundariesTrim
    oSurface.BoundsTrim = True
    oElt.UpdateFromBsplineSurface oSurface
    oElt.Redraw
    
    ' get/set_Boundary, get_BoundariesCount, get_BoundaryCount
    n = oSurface.BoundsCount
    aBound = oSurface.Bound(n - 1)
    For i = 0 To oSurface.BoundCount(n - 1) - 1
        aBound(i).x = aBound(i).x + 0.3
        aBound(i).y = aBound(i).y - 0.4
    Next i
    oSurface.Bound(n - 1) = aBound
    oElt.UpdateFromBsplineSurface oSurface
    oElt.Redraw
    
    ' RemoveBoundaryAt
    oSurface.RemoveBoundAt 1
    oElt.UpdateFromBsplineSurface oSurface
    oElt.Redraw
    oSurface.RemoveBoundAt 1
    oElt.UpdateFromBsplineSurface oSurface
    oElt.Redraw
    oSurface.RemoveBoundAt 0
    oElt.UpdateFromBsplineSurface oSurface
    oElt.Redraw
    
    ' Test order of 2D array in SetPoles
    Set oSurface = New BsplineSurface
    oSurface.UOrder = 4
    oSurface.VOrder = 3
    
    ReDim aPoles(0 To oSurface.VOrder - 1, 0 To oSurface.UOrder - 1)
    aPoles(0, 0) = Point3dFromXYZ(5#, 10#, 0#)
    aPoles(0, 1) = Point3dFromXYZ(5#, 15#, 0.5)
    aPoles(0, 2) = Point3dFromXYZ(5#, 20#, 0.5)
    aPoles(0, 3) = Point3dFromXYZ(5#, 25#, 0#)
    aPoles(1, 0) = Point3dFromXYZ(10#, 10#, 0#)
    aPoles(1, 1) = Point3dFromXYZ(10#, 15#, -0.5)
    aPoles(1, 2) = Point3dFromXYZ(10#, 20#, -0.5)
    aPoles(1, 3) = Point3dFromXYZ(10#, 25#, 0#)
    aPoles(2, 0) = Point3dFromXYZ(15#, 10#, 0#)
    aPoles(2, 1) = Point3dFromXYZ(15#, 15#, 0.5)
    aPoles(2, 2) = Point3dFromXYZ(15#, 20#, 0.5)
    aPoles(2, 3) = Point3dFromXYZ(15#, 25#, 0#)
    
    oSurface.SetPoles aPoles
    oElt.color = oElt.color + 1
    oElt.UpdateFromBsplineSurface oSurface
    oElt.Redraw

    ' Test construction of closed surface in u without knots
    Set oSurface = New BsplineSurface
    oSurface.UOrder = 4
    oSurface.VOrder = 2
    
    ReDim aPoles(0 To oSurface.VOrder - 1, 0 To oSurface.UOrder - 1)    'numRow x numCol
    For Row = LBound(aPoles) To UBound(aPoles)
        For Col = LBound(aPoles, 2) To UBound(aPoles, 2)
            aPoles(Row, Col) = PointOnXYCircle(Col, UBound(aPoles, 2) + 1, 5#)
            aPoles(Row, Col).Z = CDbl(10 * Row)
        Next Col
    Next Row
    
    oSurface.SetPoles aPoles
    oSurface.UClosed = True
    
    oSurface.weight(0, 0) = 0.5
    
    Set oElt = CreateBsplineSurfaceElement1(Nothing, oSurface)
    oElt.color = 5
    ActiveModelReference.AddElement oElt
    oElt.Redraw
    
    ' Test construction of closed surface in v without knots
    Set oSurface = New BsplineSurface
    oSurface.VOrder = 4
    oSurface.UOrder = 2
    
    ReDim aPoles(0 To oSurface.VOrder - 1, 0 To oSurface.UOrder - 1)    'numRow x numCol
    For Row = LBound(aPoles) To UBound(aPoles)
        For Col = LBound(aPoles, 2) To UBound(aPoles, 2)
            aPoles(Row, Col) = PointOnXYCircle(Row, UBound(aPoles) + 1, 5#)
            aPoles(Row, Col).x = aPoles(Row, Col).x + 10#
            aPoles(Row, Col).Z = CDbl(10 * Col)
        Next Col
    Next Row

    oSurface.SetPoles aPoles
    oSurface.VClosed = True
    
    oSurface.weight(0, 0) = 0.5
    
    Set oElt = CreateBsplineSurfaceElement1(Nothing, oSurface)
    oElt.color = 6
    ActiveModelReference.AddElement oElt
    oElt.Redraw

    ' Test construction of closed surface in u and v without knots
    Set oSurface = New BsplineSurface
    oSurface.VOrder = 4
    oSurface.UOrder = 4
    
    ReDim aPoles(0 To oSurface.VOrder - 1, 0 To oSurface.UOrder - 1)    'numRow x numCol
    ' Row 0: radius 5 ccw circle in xy plane at (0,-10,  0)
    ' Row 1: radius 5 ccw circle in xz plane at (0,  0,-10)
    ' Row 2: radius 5 cw  circle in xy plane at (0, 10,  0)
    ' Row 3: radius 5 cw  circle in xz plane at (0,  0, 10)
    For Col = LBound(aPoles, 2) To UBound(aPoles, 2)
        aPoles(0, Col) = PointOnXYCircle(Col, UBound(aPoles, 2) + 1, 5#)
        aPoles(0, Col).y = aPoles(0, Col).y - 10#
        aPoles(1, Col) = PointOnXZCircle(Col, UBound(aPoles, 2) + 1, 5#)
        aPoles(1, Col).Z = aPoles(1, Col).Z - 10#
        aPoles(2, Col) = PointOnXYCircleCW(Col, UBound(aPoles, 2) + 1, 5#)
        aPoles(2, Col).y = aPoles(2, Col).y + 10#
        aPoles(3, Col) = PointOnXZCircleCW(Col, UBound(aPoles, 2) + 1, 5#)
        aPoles(3, Col).Z = aPoles(3, Col).Z + 10#
    Next Col
    
    'translate out of way of other surfaces
    For Row = LBound(aPoles) To UBound(aPoles)
        For Col = LBound(aPoles, 2) To UBound(aPoles, 2)
            aPoles(Row, Col).x = aPoles(Row, Col).x + 20#
        Next Col
    Next Row

    oSurface.SetPoles aPoles
    oSurface.VClosed = True
    oSurface.UClosed = True
    
    oSurface.weight(0, 0) = 0.5
    
    Set oElt = CreateBsplineSurfaceElement1(Nothing, oSurface)
    oElt.color = 7
    ActiveModelReference.AddElement oElt
    oElt.Redraw

End Sub
