Attribute VB_Name = "TestInterpolationCurve"
Option Explicit

Sub TestInterpolationCurve()
    Dim oElt As BsplineCurveElement
    Dim oCurve As InterpolationCurve
    Dim oPoleCurve As BsplineCurve
    Dim aPoints() As Point3d
    Dim aParams() As Double
    Dim nFitPoint As Long
    Dim i As Long
    
    ' Ctor
    Set oCurve = New InterpolationCurve
    
    ' SetFitPoints
    nFitPoint = 4
    ReDim aPoints(1 To nFitPoint)
    ' oCurve.SetFitPoints aPoints   ' illegal: only one pole survives compression (they are all zero)
    For i = LBound(aPoints) To UBound(aPoints)
        aPoints(i) = PointOnXYCircle(i - 1, nFitPoint, 1#)
    Next i
    oCurve.SetFitPoints aPoints
    
    ' get_FitPointsCount, get_FitPoint, set_FitPoint
    For i = 0 To oCurve.FitPointsCount - 1
        oCurve.FitPoint(i) = Point3dAdd(oCurve.FitPoint(i), Point3dFromXYZ(0#, 0#, i))
    Next i
    
    ' CreateInterpolationCurveElement
    Set oElt = CreateBsplineCurveElement2(Nothing, oCurve)
    oElt.color = 1
    ActiveModelReference.AddElement oElt
    oElt.Redraw
    
    ' ExtractInterpolationCurve
    Set oCurve = oElt.ExtractInterpolationCurve
    
    ' Test computed Bessel tans (get_BesselTangents, set_BesselTangents, UpdateFromInterpolationCurve)
    oCurve.BesselTangents = Not (oCurve.BesselTangents)
    oElt.color = oElt.color + 1
    oElt.UpdateFromInterpolationCurve oCurve
    oElt.Redraw
    
    ' Test non-computed non-scaled tans (set_StartTangent, set_EndTangent)
    oCurve.BesselTangents = False
    oCurve.StartTangent = Point3dFromXYZ(1#, 2#, 3#)
    oCurve.EndTangent = Point3dFromXYZ(2#, 3#, 4#)
    oElt.color = oElt.color + 1
    oElt.UpdateFromInterpolationCurve oCurve
    oElt.Redraw
    
    ' Test non-computed Bessel-scaled tans
    oCurve.BesselTangents = True
    oElt.color = oElt.color + 1
    oElt.UpdateFromInterpolationCurve oCurve
    oElt.Redraw
    
    ' Test periodic (get_Periodic, set_Periodic)
    oCurve.Periodic = Not (oCurve.Periodic)
    oElt.color = oElt.color + 1
    oElt.UpdateFromInterpolationCurve oCurve
    oElt.Redraw
    
    ' GetParameters
    aParams = oCurve.GetParameters
    
    ' BsplineCurve::FromInterpolationCurve
    Set oPoleCurve = New BsplineCurve
    oPoleCurve.FromInterpolationCurve oCurve
    oPoleCurve.polygonDisplay = True
    oElt.color = oElt.color + 1
    oElt.UpdateFromBsplineCurve oPoleCurve
    oElt.Redraw
    
End Sub
