Attribute VB_Name = "dgnXMLReport"
'----------------------------------------------------------------------+
' Module:   dgnReprt.bas
'
' Status: WORKING
'
' Date:  03/12/04
'
' This example snippets illustrates using Microsoft's XML
' 4.0 Object model to scan the desgin file and create an XML document.
'
' Author: BSI-DEVSPT
'
' Copyright: (c) 2004 Bentley Systems, Incorporated. All rights reserved.
'
'-----------------------------------------------------------------------+
Public Enum fileNameParseReturnType
    FileNameOnlyType
    FilePathOnlyType
    FileExtOnlyType
End Enum
'-----------------------------------------------------------------------+
'
'   name        elemNameFromNumber
'
'   author      BSI                                             06/2001
'
'-----------------------------------------------------------------------+
Public Function elemNameFromNumber(ByVal eType As Integer) As String
     
    Select Case eType
    Case 1: elemNameFromNumber = "Cell Library Header         "
    Case 2: elemNameFromNumber = "Cell Header                 "
    Case 3: elemNameFromNumber = "Line                        "
    Case 4: elemNameFromNumber = "Line String                 "
    Case 5: elemNameFromNumber = "Group Data                  "
    Case 6: elemNameFromNumber = "Shape                       "
    Case 7: elemNameFromNumber = "Text Node                   "
    Case 8: elemNameFromNumber = "Digitizer Setup Data        "
    Case 9: elemNameFromNumber = "Design File Header          "
    Case 10: elemNameFromNumber = "Level Symbology             "
    Case 11: elemNameFromNumber = "Curve                       "
    Case 12: elemNameFromNumber = "Complex String              "
    Case 13: elemNameFromNumber = "Conic                       "
    Case 14: elemNameFromNumber = "Complex Shape               "
    Case 15: elemNameFromNumber = "Ellipse                     "
    Case 16: elemNameFromNumber = "Arc                         "
    Case 17: elemNameFromNumber = "Text                        "
    Case 18: elemNameFromNumber = "Surface                     "
    Case 19: elemNameFromNumber = "Solid                       "
    Case 21: elemNameFromNumber = "BSpline Pole                "
    Case 22: elemNameFromNumber = "Point String                "
    Case 23: elemNameFromNumber = "Cone                        "
    Case 24: elemNameFromNumber = "BSpline Surface             "
    Case 25: elemNameFromNumber = "BSpline Boundary            "
    Case 26: elemNameFromNumber = "BSpline Knot                "
    Case 27: elemNameFromNumber = "BSplinr Curve               "
    Case 28: elemNameFromNumber = "BSpline Weight              "
    Case 33: elemNameFromNumber = "Dimension                   "
    Case 34: elemNameFromNumber = "Shared Cell Definition      "
    Case 35: elemNameFromNumber = "Shared Cell                 "
    Case 36: elemNameFromNumber = "MultiLine                   "
    Case 37: elemNameFromNumber = "Tag                         "
    Case 38: elemNameFromNumber = "DgnStore Component          "
    Case 39: elemNameFromNumber = "DgnStore Header             "
    Case 51: elemNameFromNumber = " ARCH (Intergraph)          "
    Case 54: elemNameFromNumber = "SDS (Intergraph)            "
    Case 56: elemNameFromNumber = "LEDS, WMS                   "
    Case 60: elemNameFromNumber = "SIS                         "
    Case 61: elemNameFromNumber = "EWS SCREMANC                "
    Case 62: elemNameFromNumber = "EWS PANEL                   "
    Case 63: elemNameFromNumber = "EWS PDS                     "
    Case 66: elemNameFromNumber = "MicroStation Application    "
    Case 87: elemNameFromNumber = "Raster Header               "
    Case 88: elemNameFromNumber = "Raster Component            "
    Case 89: elemNameFromNumber = "Raster Data                 "
    Case 90: elemNameFromNumber = "Raster Reference Attachment "
    Case 91: elemNameFromNumber = "Raster Reference Component  "
    Case 96: elemNameFromNumber = "Level Table                 "
    Case 97: elemNameFromNumber = "View Group Element          "
    Case 98: elemNameFromNumber = "View Element                "
    Case 99: elemNameFromNumber = "Level Mask Element          "
    Case 100: elemNameFromNumber = "Reference Attach Element    "
    Case 101: elemNameFromNumber = "Matrix Header               "
    Case 102: elemNameFromNumber = "Matrix Int Data             "
    Case 103: elemNameFromNumber = "Matrix Double Data          "
    Case 105: elemNameFromNumber = "Mesh Header                 "
    Case Else: elemNameFromNumber = "Unknown Element Type:       " + CStr(eType)
    
    End Select
    
End Function
'-----------------------------------------------------------------------+
'
'   name        fileNameParse
'
'   Author      BSI                                         09/2002
'
'-----------------------------------------------------------------------+
Public Function fileNameParse(fileName As String, returnType As _
                                fileNameParseReturnType) As String
    Dim backSlashPos As Long
    Dim dotPos As Long
    
    backSlashPos = InStrRev(fileName, "\") + 1
    dotPos = InStrRev(fileName, ".")
    
    Select Case returnType
        Case FileNameOnlyType: fileNameParse = Mid$(fileName, backSlashPos, _
                                                    dotPos - backSlashPos)
        Case FilePathOnlyType: fileNameParse = Mid$(fileName, 1, backSlashPos - 1)
        Case FileExtOnlyType: fileNameParse = Mid$(fileName, dotPos + 1)
    End Select
End Function

Sub printData(xmlTree As DOMDocument)
    Dim nodesLeaves As IXMLDOMNodeList
    Dim nodeLeaf As IXMLDOMNode
    
    Set nodesLeaves = xmlTree.selectNodes(ActiveDesignFile.Name + "/DGN_Elements/MSElem_Type")
    
    For Each nodeLeaf In nodesLeaves    'Loop All nodes called leaf
      If nodeLeaf.hasChildNodes = True Then 'If it has XML
        
        
        'List all FilePos
        Debug.Print XMLSnippets.XMLNodeValueGet(nodeLeaf, "FilePos")
                     
      End If
Next

End Sub
'-----------------------------------------------------------------------+
'
'   name        main
'
'   Author      BSI                                         03/2004
'
'-----------------------------------------------------------------------+
Sub main()
    Dim oScanCriteria   As New ElementScanCriteria
    Dim oEnumerator     As ElementEnumerator
    Dim xmlDocument     As DOMDocument
    Dim xmlRootElem     As IXMLDOMElement
    Dim xmlNodeValue    As IXMLDOMNode
    Dim strXMLFileName  As String

    Set xmlDocument = New DOMDocument
    Set xmlRootElem = xmlDocument.CreateElement(ActiveDesignFile.Name)
    
    'Add the root node
    xmlDocument.appendChild xmlRootElem
              
    'Clear out any previous scans
    oScanCriteria.Reset
    'oScanCriteria.ExcludeAllTypes
    oScanCriteria.IncludeOnlyVisible
    
    'Set enumerator
    Set oEnumerator = ActiveModelReference.Scan(oScanCriteria)
    Set xmlNodeValue = xmlDocument.CreateElement("DGN_Elements")

    ' Add the Values section node to the root element.
    xmlRootElem.appendChild xmlNodeValue
        
    'Step through every element in the file
    Do While oEnumerator.MoveNext
        With oEnumerator.Current
            
            If (.Type <> 0) Then
                Dim xmlOriginElem     As IXMLDOMElement
                Dim xmlOriginValue    As IXMLDOMNode
                
                'Create nodes that will hold the element info
                XMLSnippets.XMLNodeCreate xmlNodeValue, "MSElem_Type", elemNameFromNumber(.Type)
                XMLSnippets.XMLNodeCreate xmlNodeValue.lastChild, "Color", CStr(.Color)
                XMLSnippets.XMLNodeCreate xmlNodeValue.lastChild, "LineWeight", CStr(.LineWeight)
                XMLSnippets.XMLNodeCreate xmlNodeValue.lastChild, "FilePos", CStr(.FilePosition)
                
                'Add Cell specific only data
                If .Type = msdElementTypeCellHeader Then
                    XMLSnippets.XMLNodeCreate xmlNodeValue.lastChild, "CellName", CStr(.AsCellElement.Name)
                    Set xmlOriginValue = xmlDocument.CreateElement("Origin")
                    xmlNodeValue.lastChild.appendChild xmlOriginValue
                    XMLSnippets.XMLNodeCreate xmlNodeValue.lastChild.lastChild, "X", CStr(.AsCellElement.Origin.X)
                    XMLSnippets.XMLNodeCreate xmlNodeValue.lastChild.lastChild, "Y", CStr(.AsCellElement.Origin.Y)
                End If
            End If
        End With
    Loop

    'Build file name and save XML file
    strXMLFileName = (fileNameParse(ActiveDesignFile.FullName, FilePathOnlyType)) + fileNameParse(ActiveDesignFile.Name, FileNameOnlyType) + ".xml"
    XMLSnippets.XMLSavedocument xmlDocument, strXMLFileName

printData xmlDocument

End Sub
