Attribute VB_Name = "WinFileDialogs"
Option Explicit

Private Declare Function GetOpenfilename Lib "comdlg32.dll" Alias "GetOpenFileNameA" (pOpenfilename As OpenFilename) As Long

Private Declare Function GetSaveFileName Lib "comdlg32.dll" Alias "GetSaveFileNameA" (pOpenfilename As OpenFilename) As Long

Private Declare Function GetFileTitle Lib "comdlg32.dll" Alias "GetFileTitleA" (ByVal lpszFile As String, ByVal lpszTitle As String, ByVal cbBuf As Integer) As Integer

Private Const OFN_READONLY As Long = &H1&
Private Const OFN_OVERWRITEPROMPT As Long = &H2&
Private Const OFN_HIDEREADONLY As Long = &H4&
Private Const OFN_NOCHANGEDIR As Long = &H8&
Private Const OFN_SHOWHELP As Long = &H10&
Private Const OFN_ENABLEHOOK As Long = &H20&
Private Const OFN_ENABLETEMPLATE As Long = &H40&
Private Const OFN_ENABLETEMPLATEHANDLE As Long = &H80&
Private Const OFN_NOVALIDATE As Long = &H100&
Private Const OFN_ALLOWMULTISELECT As Long = &H200&
Private Const OFN_EXTENSIONDIFFERENT As Long = &H400&
Private Const OFN_PATHMUSTEXIST As Long = &H800&
Private Const OFN_FILEMUSTEXIST As Long = &H1000&
Private Const OFN_CREATEPROMPT As Long = &H2000&
Private Const OFN_SHAREAWARE As Long = &H4000&
Private Const OFN_NOREADONLYRETURN As Long = &H8000&
Private Const OFN_NOTESTFILECREATE As Long = &H10000
Private Const OFN_NONETWORKBUTTON As Long = &H20000
Private Const OFN_NOLONGNAMES As Long = &H40000                       '  force no long names for 4.x modules
Private Const OFN_EXPLORER As Long = &H80000                          '  new look commdlg
Private Const OFN_NODEREFERENCELINKS As Long = &H100000
Private Const OFN_LONGNAMES As Long = &H200000                        '  force long names for 3.x modules

Private Type OpenFilename
    lStructSize As Long
    hwndOwner As Long
    hInstance As Long
    lpstrFilter As String
    lpstrCustomFilter As String
    nMaxCustFilter As Long
    nFilterIndex As Long
    lpstrFile As String
    nMaxFile As Long
    lpstrFileTitle As String
    nMaxFileTitle As Long
    lpstrInitialDir As String
    lpstrTitle As String
    flags As Long
    nFileOffset As Integer
    nFileExtension As Integer
    lpstrDefExt As String
    lCustData As Long
    lpfnHook As Long
    lpTemplateName As String
End Type

Public Function ChooseOpenFile(ByVal frmOwner As UserForm, _
                                ByVal sDialogTitle As String, _
                                ByVal sFilter As String, _
                                ByVal sDefaultExtension As String, _
                                ByVal sInitialFileName As String) As String

    ' frmOwner
    '     The form that calls this function.
    '     Example: Me.
    ' sDialogTitle
    '     The title of the file open dialog that will appear.
    '     Example: "Chose a Design File".
    ' sFilter
    '     A list of zero or more pairs of strings, adjacent strings separated by a vertical bar character ("|").
    '     The first string of each pair is a human-friendly description.
    '     The second string of each pair is a file pattern. To specify multiple patterns, separate them with
    '     semicolons.
    '     The filter "All files (*.*)" will be appended automatically.
    '     Example: "Text files (*.txt)|*.txt|Graphics files (*.bmp;*.gif; *.jpg)|*.bmp;*.gif;*.jpg"
    ' sDefaultExtension
    '     If the user manually types in a file name without an extension, this extension will be appended
    '     automatically. This should NOT include a period ("."), and should be no longer than three characters.
    ' sInitialFileName
    '     The initial file name that will be displayed to the user. May contain wildcard characters.
    '     Additionally, the directory and path (if any) defines the initial location that will be shown.
    ' Returns
    '     If the user chooses a valid file, the return value is the fully qualified file name.
    '     If the user cancels, the return value is the empty string.

    Dim tOpenFile As OpenFilename
    Dim nFlags As Long
    Dim nReturn As Long
    nFlags = OFN_FILEMUSTEXIST Or OFN_HIDEREADONLY
    PopulateOpenFileStructure tOpenFile, frmOwner, sDialogTitle, sFilter, sDefaultExtension, nFlags, sInitialFileName
    nReturn = GetOpenfilename(tOpenFile)
    If nReturn <> 0 Then
        ChooseOpenFile = ZTrim(tOpenFile.lpstrFile)
    End If
End Function

Public Function ChooseSaveAsFile(ByVal frmOwner As UserForm, _
                                 ByVal sDialogTitle As String, _
                                 ByVal sFilter As String, _
                                 ByVal sDefaultExtension As String, _
                                 ByVal sInitialFileName As String) As String

    ' This function takes the same arguments, and returns the same value, as ChooseOpenFile (above).

    Dim tOpenFile As OpenFilename
    Dim nFlags As Long
    Dim nReturn As Long
    nFlags = OFN_OVERWRITEPROMPT
    PopulateOpenFileStructure tOpenFile, frmOwner, sDialogTitle, sFilter, sDefaultExtension, nFlags, sInitialFileName
    nReturn = GetSaveFileName(tOpenFile)
    If nReturn <> 0 Then
        ChooseSaveAsFile = ZTrim(tOpenFile.lpstrFile)
    End If
End Function


Private Sub PopulateOpenFileStructure(ByRef tOpenFile As OpenFilename, _
                                    ByVal frmOwner As UserForm, _
                                    ByRef sDialogTitle As String, _
                                    ByRef sFilter As String, _
                                    ByRef sDefaultExtension As String, _
                                    ByVal nFlags As Long, _
                                    ByRef sInitialFileName As String)
    ' Assume that, initially, all of the fields of tOpenFile are empty (0 or NULL).
    Dim nPos As Long
    Do
        nPos = InStr(nPos + 1, sFilter, "|")
        If nPos = 0 Then Exit Do
        Mid(sFilter, nPos, 1) = Chr(0)
    Loop
    If sFilter <> "" Then sFilter = sFilter & Chr(0)
    sFilter = sFilter & "All files (*.*)" & Chr(0) & "*.*" & Chr(0)
    sInitialFileName = sInitialFileName & String(257 - Len(sInitialFileName), 0)
    With tOpenFile
        .lStructSize = Len(tOpenFile)
'        .hwndOwner = frmOwner.hWnd
'        .hInstance = App.hInstance
        .lpstrFilter = sFilter
        .nFilterIndex = 1
        .lpstrFile = sInitialFileName
        .nMaxFile = Len(sInitialFileName) - 1
        .lpstrTitle = sDialogTitle
        .flags = nFlags
        .lpstrDefExt = sDefaultExtension
    End With
End Sub

Private Function ZTrim(ByVal s As String) As String
    Dim nPos As Long
    nPos = InStr(s, Chr(0))
    If nPos = 0 Then
        ZTrim = s
    Else
        ZTrim = VBA.Left(s, nPos - 1)
    End If
End Function
        
Sub main()
    Dim x As String
                                
    x = ChooseOpenFile(Nothing, "Open File", "Batch Files (*.bat)|*.bat", "", "")
    
    x = ChooseSaveAsFile(Nothing, "Save File", "Batch Files (*.bat)|*.bat", "", "")
                                

End Sub
