Attribute VB_Name = "modDimesionTextChange"
Option Explicit

'**************************
'   CONSTANT Declarations
'**************************

'iPartType:
'        There are different pieces of text that you can extract from a dimension segment.
'        The available options are
Public Enum iPartType
    ADTYPE_TEXT_UPPER = 3 '(when you have primary and secondary text),
    ADTYPE_TEXT_LOWER = 4 '(when you have primary and secondary text),
    ADTYPE_TEXT_SINGLE = 5         '(when you have only the primary text),
    ADTYPE_TEXT_SYMBOLS = 10 '(the various symbols like prefix, etc.)
End Enum

'
'iSubType:
'        The subtype is a second level of classification. For example,within the UPPER text,
'        you can access the tolerance text. The available options are
'
Public Enum iSubType
    ADSUB_NONE = 0
    ADSUB_TOL_UPPER = 1
    ADSUB_TOL_LOWER = 2
    ADSUB_TOL_SINGLE = 3
    ADSUB_LIM_UPPER = 4
    ADSUB_LIM_LOWER = 5
    ADSUB_LIM_SINGLE = 6
    ADSUB_PREFIX = 7
    ADSUB_SUFFIX = 8
    ADSUB_TERMSYMBOL = 9
    ADSUB_LEADER = 10
End Enum

' Properties functions
Private Declare Function mdlDim_getTextDescr Lib "stdmdlbltin.dll" (ByRef ppTextDescr As Long, ByVal pElementIn As Long, ByVal iSegmentNo As Long, ByVal iPartType As Long, ByVal iSubType As Long, ByVal modelRef As Long) As Long
Private Declare Function mdlDim_setTextFromDescr Lib "stdmdlbltin.dll" (ByVal pElementIn As Long, ByVal pTextDescrIn As Long, ByVal bHasValuePlaceHolder As Long, ByVal iSegmentNo As Long, ByVal iPartType As Long, ByVal iSubType As Long) As Long
Private Declare Function ElmdscrAccessor_getMSElement Lib "stdmdlaccessor.dll" (ByVal ElementDescr As Long) As Long
Private Declare Sub CopyMemoryToVBA Lib "kernel32" Alias "RtlMoveMemory" _
            (ByRef VBALocation As Any, ByVal SourceLoc As Long, ByVal length As Long)
Private Declare Function mdlElement_size Lib "stdmdlbltin.dll" (ByVal el As Long) As Long
Private Declare Function mdlElmdscr_replaceElement Lib "stdmdlbltin.dll" (ByRef existingDescrPP As Long, ByVal element As Long) As Long

'*---------------------------------------------------------------------------------**''**
' @description      Change dimension Text
' @param
' @return           Return True if Dimension Text was changed successfuly, else return False
' @version          1.0
' @example      Dim ele As element
'               ChangeDimText ele, " New Text is >> *"
' @remarks
' @alinkjoin
' @group
' @bsimethod                                     BSI-DEVSPT      04/04
'---------------+---------------+---------------+---------------+---------------+------''


'iSegmentNo:
'        The particular segment in the dimension chain whose text you would like to access
'
'iPartType:
'        There are different pieces of text that you can extract from a dimension segment. The available options are
'        ADTYPE_TEXT_UPPER (when you have primary and secondary text),
'        ADTYPE_TEXT_LOWER (when you have primary and secondary text),
'        ADTYPE_TEXT_SINGLE (when you have only the primary text),
'        ADTYPE_TEXT_SYMBOLS (the various symbols like prefix, etc.)
'
'iSubType:
'        The subtype is a second level of classification. For example, within the UPPER text, you can access the tolerance text. The available options are
'
'        ADSUB_TOL_UPPER,
'        ADSUB_TOL_LOWER,
'        ADSUB_TOL_SINGLE,
'        ADSUB_LIM_UPPER,
'        ADSUB_LIM_LOWER,
'        ADSUB_LIM_SINGLE,
'        ADSUB_PREFIX,
'        ADSUB_SUFFIX,
'        ADSUB_TERMSYMBOL,
'        ADSUB_LEADER
'
'bHasValuePlaceHolder:
'        When setting a dimension text, indicate whether you have an asterisk in the text string to indicate that the actual dimension value (length, angle, etc) needs to go there. For example, you can compose a string like, "My room should be * units in length".

Function ChangeDimText(ele As element, text As String, Optional iSegmentNo As Long = 0, _
        Optional iPartType As iPartType = ADTYPE_TEXT_SINGLE, Optional iSubType As iSubType = ADSUB_NONE) As Boolean
    Dim pEle As Long
    Dim eleBuffer(0 To 65000) As Byte
    Dim oText As TextElement
    Dim pTextElmDescr As Long
    Dim pOldDescr As Long
    Dim StatusInt As Long
    
    
    ele.Redraw msdDrawingModeErase
    
    pOldDescr = ele.MdlElementDescrP(True)
    
    pEle = ElmdscrAccessor_getMSElement(pOldDescr)
    
    mdlDim_getTextDescr pTextElmDescr, pEle, iSegmentNo, iPartType, iSubType, ele.ModelReference.MdlModelRefP
    Set oText = MdlCreateElementFromElementDescrP(pTextElmDescr)
    
    oText.text = text
    
    CopyMemoryToVBA eleBuffer(0), pEle, mdlElement_size(pEle)
    
    pEle = VarPtr(eleBuffer(0))
    
    StatusInt = mdlDim_setTextFromDescr(pEle, oText.MdlElementDescrP, 0, iSegmentNo, iPartType, iSubType)
    
    mdlElmdscr_replaceElement pOldDescr, pEle
    ele.MdlSetElementDescrP pOldDescr
    ele.Redraw
    ele.Rewrite
    Debug.Print StatusInt
    If (StatusInt = 0) Then
        ChangeDimText = True
    Else
        ChangeDimText = False
    End If
End Function


'*---------------------------------------------------------------------------------**''**
' @description      ChangeDimText
' @param
' @return           No return Value
' @version          1.0
' @example
'
' @remarks
' @alinkjoin
' @group
' @bsimethod           BSI-DEVSPT      04/04
'---------------+---------------+---------------+---------------+---------------+------''
Sub ChangeDimTextTest()
    Dim ele As element
    
    ' Hardcoded dimension elementID, user can change it as needed.
    ' or useing other way to get dimension element such as
    ' scan (File, fence, selection set)
    ' implement ILocateCommandEvents interface
    
    Set ele = ActiveDesignFile.GetElementByID(DLongFromLong(977))
    
    ' To change dimension Text
    ChangeDimText ele, " New Text is *"
        
    ' To change tolerance
    ' ChangeDimText ele, " .11", 0, ADTYPE_TEXT_SINGLE, ADSUB_TOL_LOWER
    
End Sub



