VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "clsConstructCallout"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'-----------------------------------------------------------------------------
'Class Module for ConstructCallout
'Description:   This command assists the user in constructing a Detail
'               callout bubble.
'
'
'---------------------------------------------------------------------
Option Explicit

Implements IPrimitiveCommandEvents

'----------------------------------------------------------------------
'
'           Private Data Members
'
'----------------------------------------------------------------------
Private m_nPoints As Integer
Private m_lpoints As Integer

Private m_atPoints(0 To 1) As Point3d
Private LinePoints(0 To 1) As Point3d
Private endPoints As Point3d
Private oEllipseEl As EllipseElement

Private Sub Class_Initialize()

End Sub

Private Sub IPrimitiveCommandEvents_Cleanup()

End Sub

Private Sub IPrimitiveCommandEvents_DataPoint(point As Point3d, ByVal View As View)

Dim Rot As Matrix3d

' Start point for ellipse
If m_nPoints = 0 Then
    m_atPoints(0) = point
    m_nPoints = 1
    
    'Prompt user for second point in ellipse
    ShowPrompt "Place edge point"
    CommandState.StartDynamics
    
' End point for ellipse and start point for line
ElseIf m_nPoints = 1 Then
    Dim oEllipseEl0 As EllipseElement
    Dim radius As Double
    
    m_atPoints(1) = point
    m_nPoints = 2

    ' Compute the ellipes radius, add and draw
    radius = Point3dDistance(m_atPoints(0), point)
    Set oEllipseEl0 = CreateEllipseElement2(Nothing, m_atPoints(0), radius, radius, Matrix3dIdentity, msdFillModeNotFilled)
    ActiveModelReference.AddElement oEllipseEl0
    oEllipseEl0.Redraw
    
    ' Start point of line
    LinePoints(0) = m_atPoints(0)
    m_lpoints = 1
     
     'Prompt user for second point for line.
     ShowPrompt "Place end point"
        
ElseIf m_nPoints = 2 Then
        
    Dim oCell As CellElement
    Dim oel As LineElement
    
    m_nPoints = 3
        
    ' Add and display line
         
    Set oel = CreateLineElement2(Nothing, endPoints, LinePoints(1))
    ActiveModelReference.AddElement oel
    oel.Redraw

    ' Place cell
    Set oCell = CreateCellElement3("test", point, True)
    ActiveModelReference.AddElement oCell
    oCell.Redraw
       
    'Stop the dynamics event and start default command
    CommandState.StopDynamics
    CommandState.StartDefaultCommand
    


End If
    

End Sub

Private Sub IPrimitiveCommandEvents_Dynamics(point As Point3d, ByVal View As View, ByVal DrawMode As MsdDrawingMode)

Dim oLineEl As LineElement
Dim radius As Double
Dim oCell As CellElement
Dim cPoint() As Point3d
Dim dPoint() As Point3d

' method called to show dynamics
If m_nPoints = 1 Then
    m_atPoints(1) = point
    LinePoints(1) = point
    
    ' Compute the ellipes radius and draw
    radius = Sqr((m_atPoints(1).X - m_atPoints(0).X) ^ 2 + (m_atPoints(1).Y - m_atPoints(0).Y) ^ 2)
    Set oEllipseEl = CreateEllipseElement2(Nothing, m_atPoints(0), radius, radius, Matrix3dIdentity, msdFillModeNotFilled)
    oEllipseEl.Redraw DrawMode
   
ElseIf m_nPoints = 2 Then
    
    Set oLineEl = CreateLineElement2(Nothing, LinePoints(0), point)
    Set oCell = CreateCellElement3("test", point, True)
   
    'Get the intersection points for cell and ellipse
    cPoint() = oLineEl.GetIntersectionPointsOnIntersector(oCell, Matrix3dIdentity)
    dPoint() = oLineEl.GetIntersectionPointsOnIntersector(oEllipseEl, Matrix3dIdentity)
        
        
    ' Set the points
    LinePoints(1) = cPoint(0)
    endPoints = dPoint(0)
    
    Set oLineEl = CreateLineElement2(Nothing, endPoints, LinePoints(1))
    
    oLineEl.Redraw DrawMode
    oCell.Redraw DrawMode
    

    End If
End Sub

Private Sub IPrimitiveCommandEvents_Keyin(ByVal KeyIn As String)

End Sub

Private Sub IPrimitiveCommandEvents_Reset()
' method called when the reset event occurs

' On reset, start the command again.
CommandState.StartPrimitive Me
m_nPoints = 0

End Sub

Private Sub IPrimitiveCommandEvents_Start()
'method called at the start of the command

'Attach library for terminator
CadInputQueue.SendCommand "attach library D:\test.cel"

'Prompt user
ShowCommand "Construct Detail Callout Bubble"
ShowPrompt "Select center of circle"
End Sub



