Attribute VB_Name = "MoveElements"
Option Explicit
'*----------------------------------------------------------------------+
'
' Copyright 1993-2002, Bentley Systems, Incorporated
' All rights reserved.
'
' Limited permission is hereby granted to reproduce and modify this
' copyrighted material provided that the resulting code is used only in
' conjunction with Bentley Systems products under the terms of the
' license agreement provided therein, and that this notice is retained
' in its entirety in any such reproduction or modification.
'
'----------------------------------------------------------------------*/
'-----------------------------------------------------------------------+
' ChangeElmOrder
'
' Status: WORKING
'
' This example illustrates how change the display order of elements. The
' elements are being copied by using the Clone method then the original
' element is being deleted.
'
' Author: BSI                                               07/08/02
'
'----------------------------------------------------------------------*/
'*----------------------------------------------------------------------+
'
'   Public Variables
'
'----------------------------------------------------------------------*/
Public processSwitch As String
Public firstElm As element
Public secondElm As element
Public CloneElem As element
Public elmCounter As Integer

'*----------------------------------------------------------------------+
'
'   name        reCreateElement
'
'   author      BSI                                         07/08/02
'
'----------------------------------------------------------------------*/
Sub reCreateElement(oElement As element)
    Dim cloneColor As Integer
    
    ' Copies the current element
    Set CloneElem = oElement.Clone

    ' Creates the new element
    ActiveModelReference.AddElement CloneElem

    ' Removes the selected element
    ActiveModelReference.RemoveElement oElement
    
    ' Redraw the element
    CloneElem.Redraw msdDrawingModeNormal
    
    cloneColor = CloneElem.Color
End Sub

'*----------------------------------------------------------------------+
'
'   name        BringElmToFrontByFence
'
'   author      BSI                                         07/08/02
'
'----------------------------------------------------------------------*/
Sub BringElmToFrontByFence()

    Dim oFence As Fence
    Dim oElement As element
    Dim oElEnum As ElementEnumerator
    
    ShowCommand "Bring Element to Front Using a Fence"
    ShowPrompt "Accept Fence or Reset to Quit"
    
    Set oFence = ActiveDesignFile.Fence
    
    ' Checks to if a fence is active
    If oFence.IsDefined = False Then
        CommandState.StartDefaultCommand
        ShowPrompt "Place a Fence First!!!"
    Else
        ' Sets the enumerator to return elements within a fence
        Set oElEnum = oFence.GetContents
        
        ' This enumerator returns all element in the fence
        While oElEnum.MoveNext
            Set oElement = oElEnum.Current
            Call reCreateElement(oElement)
        Wend
        ShowStatus "Fence Process Completed"
    End If
End Sub

'*----------------------------------------------------------------------+
'
'   name        BringElmFoward
'
'   author      BSI                                         07/08/02
'
'----------------------------------------------------------------------*/
Sub BringElmFoward()

    Dim oElement As element
    Dim newFirstElm As element
    Dim oElEnum As ElementEnumerator
    Dim oScanCriteria As New ElementScanCriteria
        
    Call reCreateElement(firstElm)
    Set newFirstElm = CloneElem
    
    ' Sets the scan criteria
    oScanCriteria.ExcludeNonGraphical
    Set oElEnum = ActiveModelReference.Scan(oScanCriteria)
            
    ' This enumerator returns all graphical elements
    Do While oElEnum.MoveNext
    
        Set oElement = oElEnum.Current
        
        If oElement.ID.Low > secondElm.ID.Low And _
            oElement.ID.Low <> newFirstElm.ID.Low Then
            
            Call reCreateElement(oElement)
       End If
    Loop
End Sub

'*----------------------------------------------------------------------+
'
'   name        SendElmToBack
'
'   author      BSI                                         07/08/02
'
'----------------------------------------------------------------------*/
Sub SendElmToBack(element As element)

    Dim oElement As element
    Dim oElEnum As ElementEnumerator
    Dim oScanCriteria As New ElementScanCriteria
    
    ' Sets the scan criteria
    oScanCriteria.ExcludeNonGraphical
    Set oElEnum = ActiveModelReference.Scan(oScanCriteria)
            
    ' This enumerator returns all graphical elements
    Do While oElEnum.MoveNext
    
        Set oElement = oElEnum.Current
        
        If oElement.ID.Low <> element.ID.Low Then
            Call reCreateElement(oElement)
       End If
    Loop
End Sub

'*----------------------------------------------------------------------+
'
'   name        SendElmToBackByFence
'
'   author      BSI                                         07/08/02
'
'----------------------------------------------------------------------*/
Sub SendElmToBackByFence()

    Dim oElement As element
    Dim oElEnum As ElementEnumerator
    Dim oScanCriteria As New ElementScanCriteria
    
    Dim oFence As Fence
    Dim IdMatch As Boolean
    Dim FenceElem As element
    Dim oFenceEnum As ElementEnumerator
    
    ShowCommand "Send Elements to Back Using a Fence"
    ShowPrompt "Accept Fence or Reset to Quit"
    
    Set oFence = ActiveDesignFile.Fence
    
    ' Checks to if a fence is active
    If oFence.IsDefined = False Then
        CommandState.StartDefaultCommand
        ShowPrompt "Place a Fence First!!!"
    Else
        ' Sets the scan criteria
        oScanCriteria.ExcludeNonGraphical
        Set oElEnum = ActiveModelReference.Scan(oScanCriteria)
                
        ' This enumerator returns all graphical elements
        Do While oElEnum.MoveNext
        
            Set oElement = oElEnum.Current
            
            ' Sets the enumerator to return elements within a fence
            Set oFenceEnum = oFence.GetContents
            
            ' This enumerator returns all element in the fence
            Do While oFenceEnum.MoveNext
                Set FenceElem = oFenceEnum.Current
                
                ' Determines if an element is in the fence by comparing
                ' the element IDs
                If oElement.ID.Low = FenceElem.ID.Low Then
                    IdMatch = True
                    Exit Do
                End If
            Loop
            
            ' If the element is not in the fence, its recreated so it displays
            ' on top of the elements that ARE in the fence
            If IdMatch = False Then
                Call reCreateElement(oElement)
            Else
                IdMatch = False
            End If
        Loop
        ShowStatus "Fence Process Completed"
    End If
End Sub

'*----------------------------------------------------------------------+
'
'   name        SendElmBackward
'
'   author      BSI                                         07/08/02
'
'----------------------------------------------------------------------*/
Sub SendElmBackward()

    Dim oElement As element
    Dim newSecondElm As element
    Dim oElEnum As ElementEnumerator
    Dim oScanCriteria As New ElementScanCriteria
        
    Call reCreateElement(secondElm)
    Set newSecondElm = CloneElem
    
    ' Sets the scan criteria
    oScanCriteria.ExcludeNonGraphical
    Set oElEnum = ActiveModelReference.Scan(oScanCriteria)
            
    ' This enumerator returns all graphical elements
    Do While oElEnum.MoveNext
    
        Set oElement = oElEnum.Current
      
        If oElement.ID.Low > secondElm.ID.Low And _
            oElement.ID.Low <> newSecondElm.ID.Low And _
            oElement.ID.Low <> firstElm.ID.Low Then
            
            Call reCreateElement(oElement)
       End If
    Loop
    
End Sub

'*----------------------------------------------------------------------+
'
'   name        selectElement
'
'   author      BSI                                         07/08/02
'
'----------------------------------------------------------------------*/
Sub selectElement()

    CommandState.StartLocate New selectElementCommand
    
End Sub

'*----------------------------------------------------------------------+
'
'   name        chgOrder
'
'   author      BSI                                         07/08/02
'
'----------------------------------------------------------------------*/
Sub chgOrder()

    ShowStatus ""
    MainDialog.Show

End Sub
