VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "CellCounterC1"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False

Option Explicit
' This function will return the difference between the high and low ranges
' This value is the X width and Y height plus the tolerance
Private Function GetRangeDiff(oRange As Range3d, tolerance As Double) As Point3d
    Dim tmpPoint As Point3d
    
    tmpPoint.X = oRange.High.X - oRange.Low.X + tolerance
    tmpPoint.Y = oRange.High.Y - oRange.Low.Y + tolerance
    tmpPoint.Z = oRange.High.Z - oRange.Low.Z + tolerance
    GetRangeDiff = tmpPoint
End Function

' This function will get the center of a Range.
' This is good for finding the center of an element and not
' having to depend on the origin point.
Private Function GetRangeCenter(oRange As Range3d) As Point3d
    Dim tmpPoint As Point3d     ' Holds the temperary valuse
    
    tmpPoint = Point3dAdd(oRange.High, oRange.Low)
    tmpPoint.X = (oRange.High.X + oRange.Low.X) / 2
    tmpPoint.Y = (oRange.High.Y + oRange.Low.Y) / 2
    tmpPoint.Z = (oRange.High.Z + oRange.Low.Z) / 2
    GetRangeCenter = tmpPoint
End Function

' This is an example of zooming in on a specific element.
' In this case I am searching for Ellipses
Public Sub ZoomElement(oElement As Element, tolerance As Double)
    Dim views As views
    Dim view As view
    Dim oRange As Range3d
    
    Set view = ActiveDesignFile.views(1)
    oRange = oElement.Range
    view.SetArea oRange.Low, GetRangeDiff(oRange, tolerance), Matrix3dIdentity(), 0
    view.Center = GetRangeCenter(oRange)
    RedrawAllViews
End Sub
' This property will return the Element number from the element type name
Public Property Get TypeName2Num(ByVal eType As String) As String
    Select Case eType
    Case "Cell Library Header": TypeName2Num = 1
    Case "Cell Header": TypeName2Num = 2
    Case "Line": TypeName2Num = 3
    Case "Line String": TypeName2Num = 4
    Case "Shape": TypeName2Num = 6
    Case "Text Node": TypeName2Num = 7
    Case "Curve": TypeName2Num = 11
    Case "Complex String": TypeName2Num = 12
    Case "Complex Shape": TypeName2Num = 14
    Case "Ellipse": TypeName2Num = 15
    Case "Arc": TypeName2Num = 16
    Case "Text": TypeName2Num = 17
    Case "Surface": TypeName2Num = 18
    Case "Solid": TypeName2Num = 19
    Case "BSpline Pole": TypeName2Num = 21
    Case "Point String": TypeName2Num = 22
    Case "Cone": TypeName2Num = 23
    Case "BSpline Surface": TypeName2Num = 24
    Case "BSpline Boundary": TypeName2Num = 25
    Case "BSpline Knot": TypeName2Num = 26
    Case "BSplinr Curve": TypeName2Num = 27
    Case "BSpline Weight": TypeName2Num = 28
    Case "Dimension": TypeName2Num = 33
    Case "Shared Cell Definition": TypeName2Num = 34
    Case "Shared Cell": TypeName2Num = 35
    Case "MultiLine": TypeName2Num = 36
    Case "Tag": TypeName2Num = 37
    Case "Raster Header": TypeName2Num = 87
    Case "Raster Comonent": TypeName2Num = 88
    Case Else: TypeName2Num = "NOT FOUND: " & eType
    End Select
End Property
' This property will return the Element name from the element type integer
Public Property Get TypeNum2Name(ByVal eType As Integer) As String
    Select Case eType
    Case 1: TypeNum2Name = "Cell Library Header"
    Case 2: TypeNum2Name = "Cell Header"
    Case 3: TypeNum2Name = "Line"
    Case 4: TypeNum2Name = "Line String"
    Case 6: TypeNum2Name = "Shape"
    Case 7: TypeNum2Name = "Text Node"
    Case 11: TypeNum2Name = "Curve"
    Case 12: TypeNum2Name = "Complex String"
    Case 14: TypeNum2Name = "Complex Shape"
    Case 15: TypeNum2Name = "Ellipse"
    Case 16: TypeNum2Name = "Arc"
    Case 17: TypeNum2Name = "Text"
    Case 18: TypeNum2Name = "Surface"
    Case 19: TypeNum2Name = "Solid"
    Case 21: TypeNum2Name = "BSpline Pole"
    Case 22: TypeNum2Name = "Point String"
    Case 23: TypeNum2Name = "Cone"
    Case 24: TypeNum2Name = "BSpline Surface"
    Case 25: TypeNum2Name = "BSpline Boundary"
    Case 26: TypeNum2Name = "BSpline Knot"
    Case 27: TypeNum2Name = "BSplinr Curve"
    Case 28: TypeNum2Name = "BSpline Weight"
    Case 33: TypeNum2Name = "Dimension"
    Case 34: TypeNum2Name = "Shared Cell Definition"
    Case 35: TypeNum2Name = "Shared Cell"
    Case 36: TypeNum2Name = "MultiLine"
    Case 37: TypeNum2Name = "Tag"
    Case 87: TypeNum2Name = "Raster Header"
    Case 88: TypeNum2Name = "Raster Comonent"
    Case Else: TypeNum2Name = "NOT FOUND: " & eType
    End Select
End Property
' This Property will return the color value from the text color passed
Public Property Get ColorVal(color As String) As Integer

    Select Case color
    Case "White"
        ColorVal = 0
    Case "Blue"
        ColorVal = 1
    Case "Green"
        ColorVal = 2
    Case "Red"
        ColorVal = 3
    Case "Yellow"
        ColorVal = 4
    Case "Magenta"
        ColorVal = 5
    Case "Brown"
        ColorVal = 6
    Case "LBlue"
        ColorVal = 7
    End Select
End Property

' This function will return the area of a Shape or Ellipse
Private Function ElementArea(oElement As Element) As Double

    Select Case oElement.Type
    Case msdElementTypeShape
        ElementArea = oElement.AsShapeElement.Area
    Case msdElementTypeEllipse
        ElementArea = oElement.AsEllipseElement.Area
    End Select
End Function

' This function checks if an element of the passed type has already been added
' to the TreeView. Returns True if it is already in the tree, False otherwise.
Private Function ChkTree4Dups(elemName As String, oTreeview As Object) As Boolean
    Dim oNode As Node
    
    ChkTree4Dups = False
    For Each oNode In oTreeview.Nodes
        If oNode.key = elemName Then
            ChkTree4Dups = True
            Exit For
        End If
    Next
End Function
' This Function checks if the element passed is a Text element.
' If it is then the text string is returned. Otherwise
' a Null string is returned
Private Function TypeChk(eType As Integer, oElement As Element) As String

    If eType = msdElementTypeText Then
        TypeChk = oElement.AsTextElement.Text + " , "
    Else
        TypeChk = ""
    End If
End Function

' This Property adds the info to the TreeView. In this case it adds
' the Element type by name as the Parent. The children are the elements
' Unique Id number
Public Sub BuildTree(oElement As Element, oTreeView1 As TreeView)
    Dim parentID As String
    Dim txtName As String
    
    ' The parent Key variable will have a value of P plus the
    ' element type number.
    If oElement.Type = msdElementTypeCellHeader Then
        parentID = "P" & oElement.AsCellElement.Name
        txtName = oElement.AsCellElement.Name
    Else
        parentID = "P" & oElement.Type
        txtName = TypeNum2Name(oElement.Type)
    End If
    ' If Chktree4Dups returns a True the Else is executed. If a False
    ' is returned the code before the else is executed
    If Not ChkTree4Dups(parentID, oTreeView1) Then
        ' Adds the Element name to the TreeView as the Parent
        oTreeView1.Nodes.Add , , parentID, txtName
        ' Adds the Element Unique ID to the parent as a child
        oTreeView1.Nodes.Add parentID, tvwChild, , DLongToString(oElement.ID)
    Else
        ' If the Parent already exists then this is added as a child
        ' to the existing parent
        oTreeView1.Nodes.Add parentID, tvwChild, , DLongToString(oElement.ID)
    End If
End Sub

' This Property returns the wheight value from the weight
' name passed.
Public Property Get WeightVal(weight As String) As Integer
    Select Case weight
    Case "Line Weight 0"
        WeightVal = 0
    Case "Line Weight 1"
        WeightVal = 1
    Case "Line Weight 2"
        WeightVal = 2
    Case "Line Weight 3"
        WeightVal = 3
    Case "Line Weight 4"
        WeightVal = 4
    Case "Line Weight 5"
        WeightVal = 5
    Case "Line Weight 6"
        WeightVal = 6
    Case "Line Weight 7"
        WeightVal = 7
    Case "Line Weight 8"
        WeightVal = 8
    End Select
End Property
