VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "PlaceBlockCommand"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit
'*******************************************************************************
' Example code for Microstation VBA
'
' This example demonstrates command handling and calculations for
' accepting a sequence of 4 input points points which define a (rectilinear)
' slab, in the manner of the 3DTools..primitives..slab icon in regular microstation.
'
' (The geometry created in this example is just shapes on the sides of the slab)
'
'*******************************************************************************

Implements IPrimitiveCommandEvents

'*******************************************************************************
' Persistent points and counter for ongoing data entry
'*******************************************************************************

Private m_atPoints(0 To 3) As Point3d
Private m_nPoints As Integer


'*******************************************************************************
' Given a ray defined by start point and (non-unit) direction vector.
' Project a space point onto the ray.
' Return the VECTOR from ray origin to projected point. (Does NOT return the POINT
' of projection.   Seems a little funny, but for where it's used the caller wants the
' vector.)
'*******************************************************************************
Private Function projectPointToRayVector(rayStart As Point3d, rayDirection As Point3d, spacePoint As Point3d) As Point3d
        Dim dot1 As Double
        Dim dot2 As Double
        Dim spaceVector As Point3d
        spaceVector = Point3dSubtract(spacePoint, rayStart)
        dot1 = Point3dDotProduct(spaceVector, rayDirection)
        dot2 = Point3dDotProduct(rayDirection, rayDirection)
        If dot2 = 0 Then
            projectPointToRayVector = rayStart
        Else
            projectPointToRayVector = Point3dScale(rayDirection, dot1 / dot2)
        End If
End Function
'*******************************************************************************
' Given three points (which define a plane)
' Compute two vectors such that (a) outVector0 is from inPoint0 to inPoint1,
' (b) outVector1 is perpendicular vector from the line between inPoint0 and inPoint1 to
'       inPoint2
'*******************************************************************************
Private Sub computeSquaredVectors(inPoint0 As Point3d, inPoint1 As Point3d, inPoint2 As Point3d, outVector0 As Point3d, outVector1 As Point3d)
        Dim vector01 As Point3d
        Dim vector02 As Point3d
        vector01 = Point3dSubtract(inPoint1, inPoint0)
        vector02 = Point3dSubtract(inPoint2, inPoint0)
        outVector0 = vector01
        outVector1 = Point3dSubtract(vector02, projectPointToRayVector(inPoint0, vector01, inPoint2))
End Sub

'*******************************************************************************
' Generate 0, 2, 4, or 8 points of slab data from 1, 2, 3, 4 data points
' 2 input ---> 2 out, just line between the inputs
' 3 input ---> 4 out, rectangle with inputs 0,1 as one edge, line of opposite edge
'                   passing through input 2
'       Rectangle points are in XY order -- loop around rectangle is points 0,1,3,2
' 4 input ---> 8 out, opposite ends of slab, in XYZ order -- loop around base
'               is points 0,1,3,2.  Loop around top is 4,5,7,6.
'*******************************************************************************
Private Sub computeSlabPoints(outPoints() As Point3d, numOut As Integer, inPoints() As Point3d, numIn As Integer)
    
    numOut = 0
    If numIn = 1 Then
        outPoints(0) = inPoints(0)
        numOut = 1
    ElseIf numIn = 2 Then
        ' just a line from point 0 to point 1
        outPoints(0) = inPoints(0)
        outPoints(1) = inPoints(1)
        numOut = 2
    ElseIf numIn >= 3 Then

        ' a rectangle in plane of 0,1,2 ...
        Dim shapePoints(0 To 4) As Point3d
        Dim vectorX As Point3d
        Dim vectorY As Point3d
        Call computeSquaredVectors(inPoints(0), inPoints(1), inPoints(2), vectorX, vectorY)
        outPoints(0) = inPoints(0)
        outPoints(1) = inPoints(1)
        outPoints(2) = Point3dAdd(inPoints(0), vectorY)
        outPoints(3) = Point3dAdd(inPoints(1), vectorY)
        
        If (numIn = 3) Then
            numOut = 4
        Else
            Dim outOfPlaneVector As Point3d
            Dim vectorZ As Point3d
            outOfPlaneVector = Point3dCrossProduct(vectorX, vectorY)
            vectorZ = projectPointToRayVector(inPoints(0), outOfPlaneVector, inPoints(3))
            
            outPoints(4) = Point3dAdd(outPoints(0), vectorZ)
            outPoints(5) = Point3dAdd(outPoints(1), vectorZ)
            outPoints(6) = Point3dAdd(outPoints(2), vectorZ)
            outPoints(7) = Point3dAdd(outPoints(3), vectorZ)
            numOut = 8
        End If
    End If
End Sub

'*******************************************************************************
' construct and draw/save a line from 2 points
'*******************************************************************************
Private Sub drawAndSaveLine(point0 As Point3d, point1 As Point3d, saveElement As Boolean, _
    drawElement As Boolean, _
    drawMode As MsdDrawingMode)
    Dim oLine As LineElement
    Set oLine = CreateLineElement2(Nothing, point0, point1)
    If saveElement Then
        ActiveModelReference.AddElement oLine
    End If
    If drawElement Then
        oLine.Redraw drawMode
    End If
End Sub

'*******************************************************************************
' construct and draw/save a shape from 4 points of quadrilateral (in boundary order)
'*******************************************************************************
Private Sub drawAndSaveQuadrilateral(point0 As Point3d, point1 As Point3d, point2 As Point3d, point3 As Point3d, saveElement As Boolean, drawElement As Boolean, drawMode As MsdDrawingMode)

    Dim oShape As ShapeElement
    Dim shapePoints(0 To 4) As Point3d
    
    shapePoints(0) = point0
    shapePoints(1) = point1
    shapePoints(2) = point2
    shapePoints(3) = point3
    shapePoints(4) = point0
    
    Set oShape = CreateShapeElement1(Nothing, shapePoints, msdFillModeFilled)
    
    If saveElement Then
        ActiveModelReference.AddElement oShape
    End If
    
    If drawElement Then
        oShape.Redraw drawMode
    End If
    
    End Sub


'*******************************************************************************
' Convert 2, 4, or 8 points of slab data to appropriate element type.
' Optionally save to file
' draw with indicated draw mode
'*******************************************************************************
Private Sub doElement(points() As Point3d, numPoint As Integer, saveElement As Boolean, _
    drawElement As Boolean, _
    drawMode As MsdDrawingMode)

    If numPoint = 2 Then
        ' Just a line ...
        Call drawAndSaveLine(points(0), points(1), saveElement, drawElement, drawMode)
    ElseIf numPoint = 4 Then
        ' Shape ..
        Call drawAndSaveQuadrilateral(points(0), points(1), points(3), points(2), saveElement, drawElement, drawMode)
    
    ElseIf numPoint = 8 Then
        
        ' two shapes at end of slab ....
        Call drawAndSaveQuadrilateral(points(0), points(1), points(3), points(2), saveElement, drawElement, drawMode)
        Call drawAndSaveQuadrilateral(points(4), points(5), points(7), points(6), saveElement, drawElement, drawMode)
        
        ' and 4 sides
        Call drawAndSaveQuadrilateral(points(0), points(1), points(5), points(4), saveElement, drawElement, drawMode)
        Call drawAndSaveQuadrilateral(points(1), points(3), points(7), points(5), saveElement, drawElement, drawMode)
        Call drawAndSaveQuadrilateral(points(3), points(2), points(6), points(7), saveElement, drawElement, drawMode)
        Call drawAndSaveQuadrilateral(points(2), points(0), points(4), points(6), saveElement, drawElement, drawMode)
    
    End If
End Sub



'*******************************************************************************
' Accept a data point (saved) during slab construction.
' Prior to fourth point, just save the point.
' On fourth point, display and save in file.
'*******************************************************************************
Private Sub IPrimitiveCommandEvents_DataPoint(Point As Point3d, ByVal View As View)
    
    Dim slabPoints(0 To 7) As Point3d
    Dim numSlabPoints As Integer
    
    ' First time around enable dynamics
    If m_nPoints = 0 Then
        CommandState.StartDynamics
    End If
    
    ' save the point
    m_atPoints(m_nPoints) = Point
    m_nPoints = m_nPoints + 1
        
    Call computeSlabPoints(slabPoints, numSlabPoints, m_atPoints, m_nPoints)
    
    If m_nPoints = 4 Then
        ' final point -- draw, save, and restart command
        Call doElement(slabPoints, numSlabPoints, True, True, msdDrawingModeNormal)
        m_nPoints = 0
        CadInputQueue.SendReset
    ElseIf m_nPoints > 1 Then
        Call doElement(slabPoints, numSlabPoints, False, False, msdDrawingModeTemporary)
    End If
    
    ' Set up prompt for next pass..
    If m_nPoints = 0 Then
        ShowPrompt "place start point"
        CommandState.StartDynamics
    ElseIf m_nPoints = 1 Then
        ShowPrompt "place target point of slab X direction"
    ElseIf m_nPoints = 2 Then
        ShowPrompt "place point for slab XY rectangle"
    ElseIf m_nPoints = 3 Then
        ShowPrompt "place point defining length of slab"
    End If
        
End Sub


'*******************************************************************************
' Dynamics handler
' Display slab as indicated by m_nPoints prior points plus the current dynamics point.
'*******************************************************************************
Private Sub IPrimitiveCommandEvents_Dynamics(Point As Point3d, ByVal View As View, ByVal drawMode As MsdDrawingMode)
    Dim slabPoints(0 To 7) As Point3d
    Dim numSlabPoints As Integer
    
    ' Insert the point in the array but don't record it in the count
    m_atPoints(m_nPoints) = Point
    ' Draw the current slab state, including the temporary point.
    Call computeSlabPoints(slabPoints, numSlabPoints, m_atPoints, m_nPoints + 1)
    Call doElement(slabPoints, numSlabPoints, False, True, drawMode)

End Sub


'*******************************************************************************
' Keyin event handler -- no action
'*******************************************************************************
Private Sub IPrimitiveCommandEvents_Keyin(ByVal Keyin As String)

End Sub

'*******************************************************************************
' Reset event handler --- restart the state machine, reset point count to 0.
'*******************************************************************************
Private Sub IPrimitiveCommandEvents_Reset()

    CommandState.StartPrimitive Me
    m_nPoints = 0
End Sub

'*******************************************************************************
Private Sub IPrimitiveCommandEvents_Start()
    ShowCommand "VBA PlaceBlock Example"
    ShowPrompt "Select start of block"
End Sub

Private Sub IPrimitiveCommandEvents_Cleanup()
End Sub

