VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "CreatePerpendicularLine"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'----------------------------------------------------------------------------
' CreatePerpendicularLine
'
' Status: WORKING
'
' Date: 11/25/2002
'
' This example illustrates how to palce a line perpendicular to anoter line.
'
' Author: BSI
'
'----------------------------------------------------------------------------
Implements IPrimitiveCommandEvents

'----------------------------------------------------------------------
'
'           Private Data Members
'
'----------------------------------------------------------------------
Private m_atPoints(0 To 1) As Point3d
Private m_nPoints As Integer
Private accudrawFlag As Integer

'-----------------------------------------------------------------------+
'
'   name        IPrimitiveCommandEvents_Cleanup
'
'   author      BSI                                             11/2002
'
'-----------------------------------------------------------------------+
Private Sub IPrimitiveCommandEvents_Cleanup()

End Sub
'-----------------------------------------------------------------------+
'
'   name        IPrimitiveCommandEvents_DataPoint
'
'   author      BSI                                             11/2002
'
'-----------------------------------------------------------------------+
Private Sub IPrimitiveCommandEvents_DataPoint(Point As Point3d, ByVal View As View)

'Turn on the default cursor
CommandState.SetDefaultCursor

If m_nPoints = 0 Then 'user has entered Startpoint of the line segment
    Dim statPoint As Point3d
    Dim endpoint As Point3d
    Dim element As element
    Dim vertices() As Point3d
    
    'Start the dynamics
    CommandState.StartDynamics
    m_atPoints(0) = Point
    m_nPoints = 1
    ShowPrompt "Place end point"
    
    'Select the element
    Set element = CommandState.LocateElement(Point, View, True)
    
    'Get the component element only
    Set element = CommandState.GetLocatedElement(False)
    
    'Swicth on the element type
    Select Case element.Type
    
    Case msdElementTypeLine, msdElementTypeCurve, msdElementTypePointString, _
        msdElementTypeShape, msdElementTypeLineString
        
        'Project the point Perpendicular to ensure that we are on the element.
        With element.AsIntersectableElement
            m_atPoints(0) = .ProjectPointOnPerpendicular(Point, View.Rotation)
        End With
        
        With element.AsVertexList
            vertices = .GetVertices
    
            StartPoint = vertices(.GetClosestSegment(Point))
            endpoint = vertices(.GetClosestSegment(Point) + 1)
            
        End With
        
    Case Else
        CommandState.StartDefaultCommand
        
    End Select
    
    'Calculate the angle of the line an add 90 degrees to it.
    'Two lines are perpendicular if they meet at a 90 angle.
    zangle = Atn2((endpoint.Y - StartPoint.Y), endpoint.X - StartPoint.X)
              
    ' Set the accudraw angle lock
    CommandState.AccuDrawHints.LockAngle zangle + Radians(90)

ElseIf m_nPoints = 1 Then 'user has entered Endpoint of the line segment
    
    Dim oLineEl As LineElement
    
    m_atPoints(1) = Point
    
    'draws the line segment
    Set oLineEl = CreateLineElement1(Nothing, m_atPoints)
    ActiveModelReference.AddElement oLineEl
    oLineEl.Redraw

    'set Startpoint for next line segment to the Endpoint of the previous one
    m_atPoints(0) = m_atPoints(1)
    
    CommandState.StartDefaultCommand
    
    If accudrawFlag = 1 Then
     
        CadInputQueue.SendCommand "accudraw activate"
    
    End If
    
    'Turn off accudraw angle lock
    ActiveSettings.AngleLockEnabled = False
        
End If
End Sub

'-----------------------------------------------------------------------+
'
'   name        IPrimitiveCommandEvents_Dynamics
'
'   author      BSI                                             11/2002
'
'-----------------------------------------------------------------------+
Private Sub IPrimitiveCommandEvents_Dynamics(Point As Point3d, ByVal View As View, ByVal DrawMode As MsdDrawingMode)

If m_nPoints = 1 Then
         
    m_atPoints(1) = Point
    Dim oLineEl As LineElement
        
    ' Create the line
    Set oLineEl = CreateLineElement1(Nothing, m_atPoints)
    oLineEl.Color = 5 ' color to indicate its a temporary display
            
    ' Draw the elements
    oLineEl.Redraw DrawMode
        
    ShowPrompt "Select a datapoint"
End If

End Sub
'-----------------------------------------------------------------------+
'
'   name        IPrimitiveCommandEvents_Keyin
'
'   author      BSI                                             11/2002
'
'-----------------------------------------------------------------------+
Private Sub IPrimitiveCommandEvents_Keyin(ByVal KeyIn As String)

End Sub

'-----------------------------------------------------------------------+
'
'   name        IPrimitiveCommandEvents_Reset
'
'   author      BSI                                             11/2002
'
'-----------------------------------------------------------------------+
Private Sub IPrimitiveCommandEvents_Reset()
    
    ' method called when the reset event occurs
    ' On reset, start the command again.
    CommandState.StartPrimitive Me
    m_nPoints = 0
    
End Sub

'-----------------------------------------------------------------------+
'
'   name        IPrimitiveCommandEvents_Start
'
'   author      BSI                                             11/2002
'
'-----------------------------------------------------------------------+
Private Sub IPrimitiveCommandEvents_Start()
Dim newPoint As Point3d
   
    'Turn on the locate cursor
    CommandState.SetLocateCursor
    
    'Check to see if accudraw is enabled
    If CommandState.AccuDrawHints.IsEnabled = False Then
        
        'Enable accudraw
        CadInputQueue.SendCommand "accudraw activate"
        accudrawFlag = 1
        
    Else
        accudrawFlag = 0
    End If
    
    ShowCommand "VBA Perpendicular Line"
    ShowPrompt "Select a Line"
    
End Sub

