Attribute VB_Name = "ElementDependency"
'---------------------------------------------------------------------------------------+
'
'   Copyright: (c) Bentley Systems, Incorporated. All rights reserved.
'
'---------------------------------------------------------------------------------------+
'---------------------------------------------------------------------------------------+
' Module:   ElementDependency.bas
'
' Status:   WORKING
'
' Date:  05/04/04
'
' This example illustrates how to scan the active model for Visible
' elements that have Dependencies or is Dependant to other elements
'
' Author: BSI-DEVSPT
'---------------------------------------------------------------------------------------+
Option Explicit
'---------------------------------------------------------------------------------------+
'
'   Private declaration imports
'
'---------------------------------------------------------------------------------------+
Declare Function mdlModelRef_getElementRef Lib "stdmdlbltin.dll" (ByVal modelRef As Long, ByVal filePos As Long) As Long    ' Returns a pointer to a structure
Declare Function elementRef_hasDependents Lib "stdmdlbltin.dll" (ByVal elemRef As Long) As Long

'*---------------------------------------------------------------------------------**''**
' @description      Test whether the specified element has dependents
' @param
' @return           TRUE if the element has dependents, FALSE otherwise.
' @version          1.0
' @dependencies     stdmdlbltin.dll
' @example          Debug.Print "Element has dependencies "_ +
'                   cstr (ElementHasDependencies(oClement))
' @remarks
' @alinkjoin
' @group
' @bsimethod                                                    BSI-DEVSPT      05/04
'---------------+---------------+---------------+---------------+---------------+------''
Function ElementHasDependencies(oElement As Element) As Boolean

    ElementHasDependencies = elementRef_hasDependents(mdlModelRef_getElementRef(oElement.ModelReference.MdlModelRefP, oElement.FilePosition))

End Function

'*---------------------------------------------------------------------------------**''**
' @description      Determines if the element is Dependant(has dependency linkages)
' @param            oElement IN element to test
' @return           Returns True if the element dependency linkages else False
' @version          1.0
' @dependencies
' @example          Debug.Print "Element is dependant " + cstr (ElementIsDependant(oClement))
' @remarks
' @alinkjoin
' @group
' @bsimethod                                                    BSI-DEVSPT      05/04
'---------------+---------------+---------------+---------------+---------------+------''
Function ElementIsDependant(oElement As Element) As Boolean

    Dim oDataBlock() As DataBlock
    Const LINKAGEID_Dependency = 22224

    oDataBlock = oElement.GetUserAttributeData(LINKAGEID_Dependency)

    If UBound(oDataBlock) >= 0 Then
        ElementIsDependant = True
    Else
        ElementIsDependant = False
    End If

End Function

'*---------------------------------------------------------------------------------**''**
' @description      Main entry point for example
' @param            void
' @return           Nothing
' @version          1.0
' @dependencies
' @example
' @remarks
' @alinkjoin
' @group
' @bsimethod                                                    BSI-DEVSPT      05/04
'---------------+---------------+---------------+---------------+---------------+------''
Sub main()

    Dim olevel As Level
    Dim oEnumerator As ElementEnumerator
    Dim oScanCriteria As New ElementScanCriteria

    oScanCriteria.IncludeOnlyVisible
    Set oEnumerator = ActiveModelReference.Scan(oScanCriteria)

    While oEnumerator.MoveNext

        ' If the elements is a cell enumerate the cell components and check for dependencies or is dependant
        If oEnumerator.Current.Type = msdElementTypeCellHeader Then
            Dim ee As ElementEnumerator

            Set ee = oEnumerator.Current.AsCellElement.GetSubElements

            Debug.Print "Cell " + oEnumerator.Current.AsCellElement.Name + " has dependencies: " + CStr(ElementHasDependencies(oEnumerator.Current)) + " Is Dependant: " + CStr(ElementIsDependant(oEnumerator.Current))
            Do While ee.MoveNext
                Debug.Print vbTab + "Element Type: " + CStr(ee.Current.Type) + " has dependencies: " + CStr(ElementHasDependencies(ee.Current)) + " Is Dependant: " + CStr(ElementIsDependant(ee.Current))
            Loop
        Else
            Debug.Print "Element Type: " + CStr(oEnumerator.Current.Type) + " has dependencies: " + CStr(ElementHasDependencies(oEnumerator.Current)) + " Is Dependant: " + CStr(ElementIsDependant(oEnumerator.Current))
        End If
    Wend
End Sub
