Attribute VB_Name = "Module1"
Option Explicit

'/*----------------------------------------------------------------------+
'|                                                                       |
'| Copyright 1993-2003, Bentley Systems, Incorporated                    |
'| All rights reserved.                                                  |
'|                                                                       |
'| Limited permission is hereby granted to reproduce and modify this     |
'| copyrighted material provided that the resulting code is used only in |
'| conjunction with Bentley Systems products under the terms of the      |
'| license agreement provided therein, and that this notice is retained  |
'| in its entirety in any such reproduction or modification.             |
'|                                                                       |
'+----------------------------------------------------------------------*/
' This example illustrates how to use the same VBA app in different
' versions of V8. The example will scan the active Model for Shape elements
' then display their linestyle values if their ByLevel or ByElement in a
' message box.
'-----------------------------------------------------------------------+
'
'   name        LineStyleForV80
'
'   author      BSI                                             02/2003
'
' This code handles the linestyle property found in version 08.00.04.01
'
'-----------------------------------------------------------------------+
Function LineStyleForV80(oElement As Element) As Integer

     Select Case oElement.LineStyleSource
        ' ByLevel
        Case msdValueSourceLevel
            LineStyleForV80 = oElement.Level.ElementLineStyle
            Exit Function

        ' ByElement
        Case msdValueSourceLocal
            LineStyleForV80 = oElement.LineStyle
            Exit Function
            
        ' ByCell
        Case msdValueSourceCell
            LineStyleForV80 = 8
            Exit Function
 
        Case Else
            MsgBox "The linestyle for an element could not be found"
            Exit Function
    End Select

End Function

'-----------------------------------------------------------------------+
'
'   name        LineStyleForV81
'
'   author      BSI                                             02/2003
'
' This code handles the Linestyle object for RC1 (08.00.07.32) and above
'
'-----------------------------------------------------------------------+
Function LineStyleForV81(oElement As Element) As Integer

     Select Case oElement.LineStyleSource
        ' ByLevel
        Case msdValueSourceLevel
            LineStyleForV81 = oElement.Level.ElementLineStyle.Name
            Exit Function

        ' ByElement
        Case msdValueSourceLocal
            LineStyleForV81 = oElement.LineStyle.Number
            Exit Function
            
        ' ByCell
        Case msdValueSourceCell
            LineStyleForV81 = 8
            Exit Function
 
        Case Else
            MsgBox "The linestyle for an element could not be found"
            Exit Function
    End Select

End Function

'-----------------------------------------------------------------------+
'
'   name        LineStyleGet
'
'   author      BSI                                             02/2003
'
'-----------------------------------------------------------------------+
Function LineStyleGet(oElement As Element, version As Long) As Integer
    
    If version > 8000401 And version < 9000000 Then
        LineStyleGet = LineStyleForV81(oElement)
    
    ElseIf version > 8000000 And version <= 8000401 Then
        LineStyleGet = LineStyleForV80(oElement)
        
    Else
        ShowError "Invalid version number"
    End If
    
End Function

'-----------------------------------------------------------------------+
'
'   name        VersionGetNumber
'
'   author      BSI                                             02/2003
'
'-----------------------------------------------------------------------+
Function VersionGetNumber() As Long
    
    Dim partialString   As String
    Dim wholeString     As String
    Dim strVersion      As String
    Dim length          As Long
    Dim inStringLoc     As Long
    
    ' Version property return a string that contains the active V8s version
    ' number. For example, "Version 08.00.04.01 Windows x86"
    wholeString = MicroStationDGN.version
    
    ' Gets the number of characters in the string
    length = Len(wholeString)
    
    ' Determines how far in from the left of the string where the version
    ' value is
    inStringLoc = InStr(wholeString, "0")
    
    ' Only process if inStringLoc at least contains a 1
    If inStringLoc > 0 Then
        strVersion = Mid(wholeString, inStringLoc, 11)
        
        ' Use the Mid function to only extract the numbers of the version, not
        ' taking the "."s
        partialString = Mid(strVersion, 1, 2)
        partialString = partialString + Mid(strVersion, 4, 2)
        partialString = partialString + Mid(strVersion, 7, 2)
        partialString = partialString + Mid(strVersion, 10, 2)
        
        ' Converts the String value to a Long
        VersionGetNumber = Val(partialString)
    Else
        ShowError "Invalid version number"
    End If

End Function

'-----------------------------------------------------------------------+
'
'   name        ScanForShapes
'
'   author      BSI                                             02/2003
'
'-----------------------------------------------------------------------+
Sub ScanForShapes()

    Dim oEnumerator As ElementEnumerator
    Dim oScan       As New ElementScanCriteria
    Dim lStyle      As Integer
    Dim version     As Long
    
    ' Gets the version number
    version = VersionGetNumber
    
    ' Scan only for Shape elements
    oScan.ExcludeAllTypes
    oScan.IncludeType msdElementTypeShape
    
    Set oEnumerator = ActiveModelReference.Scan(oScan)
    
    While oEnumerator.MoveNext
    
        ' Retrieve linestyle number
        lStyle = LineStyleGet(oEnumerator.Current, version)
        
        If lStyle = 8 Then
            MsgBox "The linestyle is: ByCell"
        Else
            ' Display the linestyle number in a message box
            MsgBox "The linestyle is: " & CStr(lStyle)
        End If
    Wend
End Sub
