Attribute VB_Name = "TestComplexChainCtors"
Option Explicit
Sub TestComplexChainCtors()
    Dim oCChain As ComplexStringElement, oOriginalCChain As ComplexStringElement
    Dim oCShape As ComplexShapeElement, oOriginalCShape As ComplexShapeElement
    Dim oLine As LineElement
    Dim oArc As ArcElement
    Dim ellipse As Ellipse3d
    Dim oACurve As CurveElement
    Dim oBCurve As BsplineCurveElement
    Dim curve As BsplineCurve
    Dim aElements() As ChainableElement
    Dim aPoints() As Point3d
    Dim transform As Transform3d
    Dim nElt As Long, color As Long
    
    nElt = 4
    color = 0
    
    'Create line
    ReDim aPoints(0 To 2)
    aPoints(0) = Point3dZero
    aPoints(1) = Point3dFromXYZ(1#, 1#, 0#)
    aPoints(2) = Point3dFromXYZ(2#, 0#, 0#)
    Set oLine = CreateLineElement1(Nothing, aPoints)
    
    'Create arc (reversed)
    ellipse.Center = Point3dFromXYZ(2#, 0.5, 0#)
    ellipse.Start = 0#
    ellipse.Sweep = Pi
    ellipse.Vector0 = Point3dFromXYZ(0#, 0.5, 0#)
    ellipse.Vector90 = Point3dFromXYZ(0.5, 0#, 0#)
    Set oArc = CreateEllipticalElement1(Nothing, ellipse)
    
    'Create Akima Curve (reversed)
    ReDim aPoints(0 To 3)
    aPoints(0) = Point3dFromXYZ(4#, 1#, 0#)
    aPoints(1) = Point3dFromXYZ(3.5, 0.5, 0#)
    aPoints(2) = Point3dFromXYZ(2.5, 1.5, 0#)
    aPoints(3) = Point3dFromXYZ(2#, 1#, 0#)
    Set oACurve = CreateCurveElement1(Nothing, aPoints)
    
    'Create B-spline Curve (reversed)
    Set curve = New BsplineCurve
    ReDim aPoints(0 To 2)
    aPoints(0) = Point3dFromXYZ(5#, 1#, 0#)
    aPoints(1) = Point3dFromXYZ(4.5, 0#, 0#)
    aPoints(2) = Point3dFromXYZ(4#, 1#, 0#)
    curve.SetPoles aPoints
    Set oBCurve = CreateBsplineCurveElement1(Nothing, curve)
     
    ReDim aElements(0 To nElt - 1)
    Set aElements(0) = oLine.AsChainableElement
    Set aElements(1) = oArc.AsChainableElement
    Set aElements(2) = oACurve.AsChainableElement
    Set aElements(3) = oBCurve.AsChainableElement
    
    'create a complex string
    Set oCChain = CreateComplexStringElement2(aElements, 0.00000001)
    color = color + 1
    oCChain.color = color
    ActiveModelReference.AddElement oCChain
    oCChain.Redraw
    
    Set oOriginalCChain = oCChain
    
    transform = Transform3dFromPoint3d(Point3dFromXYZ(0#, -2#, 0#))
    oCChain.transform transform
    
    'create an arc that closes up the string (reversed)
    ellipse.Center = Point3dInterpolate(oCChain.startPoint, 0.5, oCChain.EndPoint)
    ellipse.Start = -Pi
    ellipse.Sweep = Pi
    ellipse.Vector0 = Point3dSubtract(oCChain.EndPoint, ellipse.Center)
    ellipse.Vector90 = Point3dScale(Point3dNormalize(Point3dCrossProduct(Point3dFromXYZ(0#, 0#, 1#), ellipse.Vector0)), 3#)
    Set oArc = CreateEllipticalElement1(Nothing, ellipse)
    
    ReDim aElements(0 To 1)
    Set aElements(0) = oCChain
    Set aElements(1) = oArc.AsChainableElement
    
    ' create complex shape
    Set oCShape = CreateComplexShapeElement2(aElements, msdFillModeFilled, 0.00000001)
    color = color + 1
    oCShape.color = color
    ActiveModelReference.AddElement oCShape
    oCShape.Redraw
    
    Set oOriginalCShape = oCShape
    
    ' multiple chains: only first chain (at end) is output
    ReDim aElements(0 To 3)
    Set aElements(0) = oLine.AsChainableElement
    Set aElements(1) = oArc.AsChainableElement      'arc has been translated out of the chain
    Set aElements(2) = oACurve.AsChainableElement   'first chain starts here...
    Set aElements(3) = oBCurve.AsChainableElement   '...and ends here
    
    Set oCChain = CreateComplexStringElement2(aElements, 0.00000001)
    transform = Transform3dFromPoint3d(Point3dFromXYZ(0.5, 0#, 0#))
    oCChain.transform transform
    color = color + 1
    oCChain.color = color
    ActiveModelReference.AddElement oCChain
    oCChain.Redraw
    
    ' multiple chains: only first chain (at start) is output
    Set aElements(0) = oACurve.AsChainableElement   'first chain starts here...
    Set aElements(1) = oBCurve.AsChainableElement   '...and ends here
    Set aElements(2) = oLine.AsChainableElement
    Set aElements(3) = oArc.AsChainableElement      'arc has been translated out of the chain
    
    Set oCChain = CreateComplexStringElement2(aElements, 0.00000001)
    transform = Transform3dFromTransform3dTimesTransform3d(Transform3dFromPoint3d(Point3dFromXYZ(0.5, 0#, 0#)), transform)
    oCChain.transform transform
    color = color + 1
    oCChain.color = color
    ActiveModelReference.AddElement oCChain
    oCChain.Redraw
    
    ' multiple chains: only first chain (at end) is output
    Set aElements(0) = oLine.AsChainableElement
    Set aElements(1) = oBCurve.AsChainableElement   '...and ends here
    Set aElements(2) = oACurve.AsChainableElement   'first chain starts here...
    Set aElements(3) = oArc.AsChainableElement      'arc has been translated out of the chain
    
    Set oCChain = CreateComplexStringElement2(aElements, 0.00000001)
    transform = Transform3dFromTransform3dTimesTransform3d(Transform3dFromPoint3d(Point3dFromXYZ(0.5, 0#, 0#)), transform)
    oCChain.transform transform
    color = color + 1
    oCChain.color = color
    ActiveModelReference.AddElement oCChain
    oCChain.Redraw

    'create an arc that nearly closes up the original string
    ellipse.Center = Point3dInterpolate(oOriginalCChain.startPoint, 0.5, oOriginalCChain.EndPoint)
    ellipse.Start = 0
    ellipse.Sweep = -Pi + 0.00001
    ellipse.Vector0 = Point3dSubtract(oOriginalCChain.EndPoint, ellipse.Center)
    ellipse.Vector90 = Point3dScale(Point3dNormalize(Point3dCrossProduct(Point3dFromXYZ(0#, 0#, 1#), ellipse.Vector0)), 3#)
    Set oArc = CreateEllipticalElement1(Nothing, ellipse)
    
    ReDim aElements(0 To 1)
    Set aElements(0) = oOriginalCChain
    Set aElements(1) = oArc.AsChainableElement
    
    ' test that a shape returned by chain ctor gets its closure line removed, if it is smaller than tolerance
    Set oCChain = CreateComplexStringElement2(aElements, 0.00000001)
    transform = Transform3dFromPoint3d(Point3dFromXYZ(0.5, 0#, 0#))
    oCChain.transform transform
    color = color + 1
    oCChain.color = color
    ActiveModelReference.AddElement oCChain
    oCChain.Redraw
    
    ' test that a chain returned by a shape ctor gets its closure line added (no matter how big a gap)
    ellipse.Sweep = ellipse.Sweep / 2#  'create large gap
    Set oArc = CreateEllipticalElement1(Nothing, ellipse)
    Set aElements(1) = oArc.AsChainableElement
    Set oCShape = CreateComplexShapeElement2(aElements, msdFillModeFilled, 0.00000001)
    transform = Transform3dFromTransform3dTimesTransform3d(Transform3dFromPoint3d(Point3dFromXYZ(0.5, 0#, 0#)), transform)
    oCShape.transform transform
    color = color + 1
    oCShape.color = color
    ActiveModelReference.AddElement oCShape
    oCShape.Redraw
    
End Sub
