Attribute VB_Name = "TestPlanePlaneIntersect"
Option Explicit

Sub TestPlanePlaneIntesect()
    Dim oLine As LineElement, oPlane As EllipseElement
    Dim sEllipse As Ellipse3d
    Dim sPlane0 As Plane3d, sPlane1 As Plane3d
    Dim sProjection0 As Point3d, sProjection1 As Point3d, sProjectionAverage As Point3d
    Dim sRay As Ray3d
    Dim param0 As Double, param1 As Double, radius As Double, swell As Double, dist As Double
    
    sPlane0.Origin = Point3dFromXYZ(-6#, -3#, 1#)
    sPlane0.normal = Point3dFromXYZ(4#, 8#, -2#)    'unnormalized
    
    sPlane1.Origin = Point3dFromXYZ(4#, -2#, -1#)
    sPlane1.normal = Point3dFromXYZ(1#, -3#, -2#)   'unnormalized
    
    'Visualize plane centers
    Set oLine = CreateLineElement2(Nothing, sPlane0.Origin, sPlane0.Origin)
    oLine.LineWeight = 5
    ActiveModelReference.AddElement oLine
    oLine.Redraw
    Set oLine = CreateLineElement2(oLine, sPlane1.Origin, sPlane1.Origin)
    ActiveModelReference.AddElement oLine
    oLine.Redraw

    'Plane-Plane intersect
    sRay = Plane3dIntersectPlane3d(sPlane0, sPlane1)
    
    'Visualize intersection ray
    Set oLine = CreateLineElement2(Nothing, sRay.Origin, Point3dAdd(sRay.Origin, sRay.Direction))
    oLine.LineWeight = 2
    oLine.color = 3
    ActiveModelReference.AddElement oLine
    oLine.Redraw
    Set oLine = CreateLineElement2(oLine, sRay.Origin, sRay.Origin)
    oLine.LineWeight = 5
    ActiveModelReference.AddElement oLine
    oLine.Redraw

    'Test Point3dProjectToRay3d (without View) (visualize projections of plane centers to intersection line)
    sProjection0 = Point3dProjectToRay3d(param0, sPlane0.Origin, sRay)
    sProjection1 = Point3dProjectToRay3d(param1, sPlane1.Origin, sRay)
    Set oLine = CreateLineElement2(Nothing, sPlane0.Origin, sProjection0)
    ActiveModelReference.AddElement oLine
    oLine.Redraw
    Set oLine = CreateLineElement2(Nothing, sPlane1.Origin, sProjection1)
    ActiveModelReference.AddElement oLine
    oLine.Redraw

    'Get plane visualization center
    sProjectionAverage = Point3dInterpolate(sProjection0, 0.5, sProjection1)
    
    'Get plane visualization radius large enough to contain origins of planes
    swell = 1.2
    radius = Point3dDistance(sProjectionAverage, sPlane0.Origin)
    dist = Point3dDistance(sProjectionAverage, sPlane1.Origin)
    If dist > radius Then
        radius = dist
    End If
    radius = radius * swell
    
    'Visualize Plane0
    sEllipse.Start = 0
    sEllipse.Sweep = 2# * Pi
    sEllipse.Center = sProjectionAverage
    sEllipse.Vector0 = Point3dScale(Point3dNormalize(sRay.Direction), radius)
    sEllipse.Vector90 = Point3dScale(Point3dNormalize(Point3dCrossProduct(sPlane0.normal, sEllipse.Vector0)), radius)
    Set oPlane = CreateEllipticalElement1(Nothing, sEllipse, msdFillModeFilled)
    ActiveModelReference.AddElement oPlane
    oPlane.Redraw
    
    'Visualize Plane1
    sEllipse.Vector90 = Point3dScale(Point3dNormalize(Point3dCrossProduct(sPlane1.normal, sEllipse.Vector0)), radius)
    Set oPlane = CreateEllipticalElement1(Nothing, sEllipse, msdFillModeFilled)
    ActiveModelReference.AddElement oPlane
    oPlane.Redraw

    'Visualize intersection segment
    Set oLine = CreateLineElement2(Nothing, Point3dAddScaled(sProjectionAverage, sEllipse.Vector0, -1#), Point3dAdd(sProjectionAverage, sEllipse.Vector0))
    ActiveModelReference.AddElement oLine
    oLine.Redraw
End Sub

