Attribute VB_Name = "TestPoint3dProject"
Option Explicit

Sub TestPoint3dProject()
    Dim oLine As LineElement, oLine0 As LineElement, oLine1 As LineElement, oPlane As EllipseElement
    Dim sEllipse As Ellipse3d
    Dim sRotation As Matrix3d
    Dim sPlane0 As Plane3d, sPlane1 As Plane3d
    Dim sProjection0 As Point3d, sProjection1 As Point3d, sProjectionAverage As Point3d
    Dim sRay As Ray3d
    Dim param0 As Double, param1 As Double, radius As Double, swell As Double, dist As Double
    
    sPlane0.Origin = Point3dFromXYZ(-6#, -3#, 1#)
    sPlane0.normal = Point3dFromXYZ(4#, 8#, -2#)    'unnormalized
    
    sPlane1.Origin = Point3dFromXYZ(4#, -2#, -1#)
    sPlane1.normal = Point3dFromXYZ(1#, -3#, -2#)   'unnormalized
    
    'Visualize plane centers
    Set oLine0 = CreateLineElement2(Nothing, sPlane0.Origin, sPlane0.Origin)
    oLine0.LineWeight = 5
    oLine0.color = 1
    ActiveModelReference.AddElement oLine0
    oLine0.Redraw
    Set oLine1 = CreateLineElement2(Nothing, sPlane1.Origin, sPlane1.Origin)
    oLine1.LineWeight = 5
    oLine1.color = 2
    ActiveModelReference.AddElement oLine1
    oLine1.Redraw

    'Plane-Plane intersect
    sRay = Plane3dIntersectPlane3d(sPlane0, sPlane1)
    
    'Visualize intersection ray
    Set oLine = CreateLineElement2(Nothing, sRay.Origin, Point3dAdd(sRay.Origin, sRay.Direction))
    oLine.LineWeight = 2
    oLine.color = 3
    ActiveModelReference.AddElement oLine
    oLine.Redraw
    Set oLine = CreateLineElement2(oLine, sRay.Origin, sRay.Origin)
    oLine.LineWeight = 5
    ActiveModelReference.AddElement oLine
    oLine.Redraw

    'Test Point3dProjectToRay3d (without View) (visualize projections of plane centers to intersection line)
    sProjection0 = Point3dProjectToRay3d(param0, sPlane0.Origin, sRay)
    sProjection1 = Point3dProjectToRay3d(param1, sPlane1.Origin, sRay)
    Set oLine = CreateLineElement2(oLine0, sPlane0.Origin, sProjection0)
    oLine.LineWeight = 1
    ActiveModelReference.AddElement oLine
    oLine.Redraw
    Set oLine = CreateLineElement2(oLine1, sPlane1.Origin, sProjection1)
    oLine.LineWeight = 1
    ActiveModelReference.AddElement oLine
    oLine.Redraw

    'Get plane visualization center
    sProjectionAverage = Point3dInterpolate(sProjection0, 0.5, sProjection1)
    
    'Get plane visualization radius large enough to contain origins of planes
    swell = 1.2
    radius = Point3dDistance(sProjectionAverage, sPlane0.Origin)
    dist = Point3dDistance(sProjectionAverage, sPlane1.Origin)
    If dist > radius Then
        radius = dist
    End If
    radius = radius * swell
    
    'Visualize Plane0
    sEllipse.Start = 0
    sEllipse.Sweep = 2# * Pi
    sEllipse.Center = sProjectionAverage
    sEllipse.Vector0 = Point3dScale(Point3dNormalize(sRay.Direction), radius)
    sEllipse.Vector90 = Point3dScale(Point3dNormalize(Point3dCrossProduct(sPlane0.normal, sEllipse.Vector0)), radius)
    Set oPlane = CreateEllipticalElement1(oLine0, sEllipse, msdFillModeFilled)
    oPlane.LineWeight = 1
    ActiveModelReference.AddElement oPlane
    oPlane.Redraw
    
    'Visualize Plane1
    sEllipse.Vector90 = Point3dScale(Point3dNormalize(Point3dCrossProduct(sPlane1.normal, sEllipse.Vector0)), radius)
    Set oPlane = CreateEllipticalElement1(oLine1, sEllipse, msdFillModeFilled)
    oPlane.LineWeight = 1
    ActiveModelReference.AddElement oPlane
    oPlane.Redraw

    'Visualize intersection segment
    Set oLine = CreateLineElement2(oLine, Point3dAddScaled(sProjectionAverage, sEllipse.Vector0, -1#), Point3dAdd(sProjectionAverage, sEllipse.Vector0))
    oLine.color = 3
    oLine.LineWeight = 1
    ActiveModelReference.AddElement oLine
    oLine.Redraw

    'Test Point3dProjectToRay3d (project along View normal = y-axis of plane1)
    sRotation = ActiveDesignFile.Views(1).Rotation
    ActiveDesignFile.Views(1).Rotation = Matrix3dFromPoint3dRows(Point3dNormalize(sRay.Direction), Point3dNormalize(sPlane1.normal), Point3dNormalize(Point3dCrossProduct(sPlane1.normal, sRay.Direction)))
    ActiveDesignFile.Views(1).Redraw
    sProjection0 = Point3dProjectToRay3d(param0, sPlane1.Origin, sRay, ActiveDesignFile.Views(1))
    Set oLine = CreateLineElement2(oLine0, sProjection0, sProjection0) 'should be same as true projection
    oLine.color = 4
    ActiveModelReference.AddElement oLine
    oLine.Redraw

    'If param0 <> param1 Or Not Point3dEqualTolerance(sProjection0, sProjection1, 0.0000000001) Then
    '    Err.Raise 3492
    'End If
    
    'Test Point3dProjectToPlane3d
    ActiveDesignFile.Views(1).Rotation = Matrix3dFromMatrix3dTimesMatrix3d(Matrix3dFromAxisAndRotationAngle(0, 0.2), ActiveDesignFile.Views(1).Rotation)
    ActiveDesignFile.Views(1).Redraw
    sProjection0 = Point3dProjectToPlane3d(sPlane1.Origin, sPlane0)
    Set oLine = CreateLineElement2(oLine0, sProjection0, sProjection0)
    ActiveModelReference.AddElement oLine
    oLine.Redraw
    
    'Test Point3dProjectToPlane3d (project along View normal)
    sProjection0 = Point3dProjectToPlane3d(sPlane1.Origin, sPlane0, 1)
    Set oLine = CreateLineElement2(oLine0, sProjection0, sProjection0)
    oLine.color = 4
    ActiveModelReference.AddElement oLine
    oLine.Redraw

    'Test Point3dProjectToPlane3d (project along Accudraw plane normal)
    If CommandState.AccuDrawHints.IsEnabled Then
        'Get Accudraw active
        CadInputQueue.SendKeyin "Place Line"
        
        'Change the Accudraw plane
        CommandState.AccuDrawHints.SetOrientationByZAxis Point3dFromXYZ(-1, 2, -3)
        
        'Issue data point to make the rotation stick
        CadInputQueue.SendDataPoint Point3dZero, 1
        
        'Project while the rotation is still there
        sProjection0 = Point3dProjectToPlane3d(sPlane1.Origin, sPlane0, 1, True)
        
        'Cancel the command
        CadInputQueue.SendReset
        CommandState.StartDefaultCommand
        
        Set oLine = CreateLineElement2(oLine0, sProjection0, sProjection0)
        oLine.color = 5
        ActiveModelReference.AddElement oLine
        oLine.Redraw
    End If
    
    'Test Point3dProjectToPlane3d (project along ACS plane normal)
    CadInputQueue.SendKeyin "Define ACS Points"                 'define ACS
    CadInputQueue.SendDataPoint Point3dZero                     'origin at world origin
    CadInputQueue.SendDataPoint Point3dFromXYZ(1, 1, 1)         'x-axis point
    CadInputQueue.SendDataPoint Point3dFromXYZ(0, 1, -1)        'y-axis direction
    CadInputQueue.SendKeyin "Lock ACS On"                       'set ACS plane lock
    sProjection0 = Point3dProjectToPlane3d(sPlane1.Origin, sPlane0, 1, True)
    Set oLine = CreateLineElement2(oLine0, sProjection0, sProjection0)
    oLine.color = 6
    ActiveModelReference.AddElement oLine
    oLine.Redraw
    
    'restore
    CadInputQueue.SendKeyin "Rotate ACS Absolute 0,0,0"         'Restore ACS to world
    CadInputQueue.SendKeyin "Lock ACS Off"                      'unset ACS plane lock
    ActiveDesignFile.Views(1).Rotation = sRotation
    ActiveDesignFile.Views(1).Redraw
End Sub


