Attribute VB_Name = "TestRayPlaneIntersect"
Option Explicit

Sub TestRayPlaneIntersect()
    Dim oLine As LineElement, oPlane As EllipseElement, oPoint As LineElement
    Dim sEllipse As Ellipse3d
    Dim sRotation As Matrix3d
    Dim sPlane As Plane3d
    Dim sRay As Ray3d
    Dim sIntersectPt As Point3d, sProjection0 As Point3d, sProjection1 As Point3d
    Dim param As Double, radius As Double, dist As Double, swell As Double
    
    sRay.Origin = Point3dZero
    sRay.Direction = Point3dFromXYZ(3#, -1#, 5#)
    
    sPlane.Origin = Point3dFromXYZ(-6#, -3#, 1#)
    sPlane.normal = Point3dFromXYZ(4#, 8#, -2#)     'unnormalized
    
    'Ray-Plane intersect
    sIntersectPt = Plane3dIntersectRay3d(param, sPlane, sRay)
    
    'Check parameter for validity
    If Abs(Point3dDistance(sIntersectPt, sRay.Origin) - Abs(param) * Point3dMagnitude(sRay.Direction)) > 0.0000000001 Then
        Err.Raise 3492
    End If
    
    'Check Point3dProjectToPlane3d (without View)
    sProjection0 = Point3dProjectToPlane3d(sRay.Origin, sPlane)
    Set oLine = CreateLineElement2(Nothing, sProjection0, sProjection0)
    oLine.LineWeight = 5
    ActiveModelReference.AddElement oLine
    oLine.Redraw
    
    
    'Get plane visualization radius large enough to contain plane origin and intersection
    swell = 1.2
    radius = Point3dDistance(sProjection0, sPlane.Origin)
    dist = Point3dDistance(sIntersectPt, sPlane.Origin)
    If dist > radius Then
        radius = dist
    End If
    radius = radius * swell

    'Visualize the plane
    sEllipse.Start = 0#
    sEllipse.Sweep = 2# * Pi
    sEllipse.Center = sProjection0
    sEllipse.Vector0 = Point3dScale(Point3dNormalize(Point3dCrossProduct(sPlane.normal, Point3dFromXYZ(1#, 0#, 0#))), radius)
    sEllipse.Vector90 = Point3dScale(Point3dNormalize(Point3dCrossProduct(sPlane.normal, sEllipse.Vector0)), radius)
    Set oPlane = CreateEllipticalElement1(Nothing, sEllipse, msdFillModeFilled)
    ActiveModelReference.AddElement oPlane
    oPlane.Redraw
    
    'Visualize the line
    Set oLine = CreateLineElement2(Nothing, sRay.Origin, Point3dAddScaled(sRay.Origin, sRay.Direction, swell * param))
    ActiveModelReference.AddElement oLine
    oLine.Redraw
    
    'Visualize the intersection point
    Set oPoint = CreateLineElement2(Nothing, sIntersectPt, sIntersectPt)
    oPoint.LineWeight = 5
    ActiveModelReference.AddElement oPoint
    oPoint.Redraw
    
    'Test Point3dProjectToPlane3d (with View)
    sRotation = ActiveDesignFile.Views(1).Rotation
    ActiveDesignFile.Views(1).Rotation = Matrix3dFromPoint3dRows(Point3dNormalize(sEllipse.Vector0), Point3dNormalize(sEllipse.Vector90), Point3dNormalize(sPlane.normal))
    sProjection1 = Point3dProjectToPlane3d(sRay.Origin, sPlane, 1)
    If Not Point3dEqualTolerance(sProjection0, sProjection1, 0.0000000001) Then
        Err.Raise 3492
    End If
    ActiveDesignFile.Views(1).Rotation = sRotation

End Sub
