Attribute VB_Name = "TestRegionBooleanFromClosedElt"
Option Explicit

Sub TestRegionBoolean()
    Dim aRegion() As Element, oEllipse As EllipseElement, oElement As Element
    Dim sEllipse As Ellipse3d
    Dim sVec0 As Point3d, sVec90 As Point3d, sCross As Point3d
    Dim oEnumerator As ElementEnumerator
    Dim axis0 As Double, axis1 As Double
    Dim i As Long, color As Long, weight As Long, saveFillColor As Long, fillColor As Long
    
    color = 2
    weight = 3
    saveFillColor = ActiveSettings.fillColor
    fillColor = 0
    
    'Make arbitrary plane for geometry
    sVec0 = Point3dFromXYZ(1, 1, 0)
    sVec90 = Point3dFromXYZ(-1, 1, 1)
    sCross = Point3dCrossProduct(sVec0, sVec90)
    sVec90 = Point3dCrossProduct(sCross, sVec0)
    sVec0 = Point3dNormalize(sVec0)
    sVec90 = Point3dNormalize(sVec90)
    
    'Make closed element at origin
    sEllipse.Vector0 = sVec0
    sEllipse.Vector90 = sVec90
    sEllipse.Center = Point3dZero()
    sEllipse.Start = 0#
    sEllipse.Sweep = 2 * Pi
    ActiveSettings.fillColor = fillColor
    Set oEllipse = CreateEllipticalElement1(Nothing, sEllipse, msdFillModeNotFilled)
    ActiveModelReference.AddElement oEllipse
    oEllipse.Redraw
    
    'Make collection of coplanar elements
    ReDim aRegion(0 To 1)

    sEllipse.Center = Point3dAddScaled(Point3dZero(), sVec90, 3#)
    sEllipse.Vector90 = Point3dScale(sVec90, 2.5)
    Set aRegion(0) = CreateEllipticalElement1(Nothing, sEllipse, msdFillModeNotFilled)
    
    sEllipse.Center = Point3dAddScaled(Point3dZero(), sVec0, 5#)
    'sEllipse.Vector0 = Point3dScale(sVec0, 4#) ' just touching origin circle is a bug---won't get this ellipse in the union!
    sEllipse.Vector0 = Point3dScale(sVec0, 5#)
    sEllipse.Vector90 = Point3dScale(sVec90, 3#)
    Set aRegion(1) = CreateEllipticalElement1(Nothing, sEllipse, msdFillModeNotFilled)
        
    For i = LBound(aRegion) To UBound(aRegion)
        ActiveModelReference.AddElement aRegion(i)
        aRegion(i).Redraw
    Next
    
    'Union closed element with collection union
    fillColor = fillColor + 1
    ActiveSettings.fillColor = fillColor
    Set oEnumerator = oEllipse.GetUnionShapesFromRegion(aRegion, oEllipse, msdFillModeFilled)
    Do While oEnumerator.MoveNext
        Set oElement = oEnumerator.Current
        ActiveModelReference.AddElement oElement
        oElement.Redraw
    Loop
    
    'Intersect closed element with collection union
    fillColor = fillColor + 1
    ActiveSettings.fillColor = fillColor
    Set oEnumerator = oEllipse.GetIntersectionShapesFromRegion(aRegion, oEllipse, msdFillModeFilled)
    Do While oEnumerator.MoveNext
        Set oElement = oEnumerator.Current
        ActiveModelReference.AddElement oElement
        oElement.Redraw
    Loop

    'Closed element minus collection union
    fillColor = fillColor + 1
    ActiveSettings.fillColor = fillColor
    Set oEnumerator = oEllipse.GetDifferenceShapesFromRegion(aRegion, oEllipse, msdFillModeFilled)
    Do While oEnumerator.MoveNext
        Set oElement = oEnumerator.Current
        ActiveModelReference.AddElement oElement
        oElement.Redraw
    Loop
    
    'Make ellipses inside oEllipse to subtract them off
    sEllipse.Vector0 = Point3dScale(sVec0, 1# / 3#)
    sEllipse.Vector90 = Point3dScale(sVec90, 1# / 3#)
    sEllipse.Center = Point3dAddScaled(Point3dZero, sVec0, -1# / 2#)
    sEllipse.Start = 0#
    sEllipse.Sweep = 2 * Pi
    Set aRegion(0) = CreateEllipticalElement1(Nothing, sEllipse, msdFillModeNotFilled)
    ActiveModelReference.AddElement aRegion(0)
    aRegion(0).Redraw
    sEllipse.Center = Point3dAddScaled(Point3dZero, sVec0, 1# / 2#)
    Set aRegion(1) = CreateEllipticalElement1(Nothing, sEllipse, msdFillModeNotFilled)
    ActiveModelReference.AddElement aRegion(1)
    aRegion(1).Redraw
    
    'Closed element minus collection union contained within
    fillColor = fillColor + 1
    ActiveSettings.fillColor = fillColor
    Set oEnumerator = oEllipse.GetDifferenceShapesFromRegion(aRegion, oEllipse, msdFillModeFilled)
    Do While oEnumerator.MoveNext
        Set oElement = oEnumerator.Current
        ActiveModelReference.AddElement oElement
        oElement.Redraw
    Loop

    'Create non-closed element collection
    Set oEllipse = aRegion(1).AsEllipseElement
    ReDim aRegion(0 To 2)
    Set aRegion(0) = CreateLineElement2(Nothing, sEllipse.Center, Point3dAdd(sEllipse.Center, Point3dScale(sEllipse.Vector0, 2#)))
    ActiveModelReference.AddElement aRegion(0)
    aRegion(0).Redraw
    Set aRegion(1) = CreateLineElement2(Nothing, sEllipse.Center, Point3dAdd(sEllipse.Center, Point3dScale(sEllipse.Vector90, 2#)))
    ActiveModelReference.AddElement aRegion(1)
    aRegion(1).Redraw
    Set aRegion(2) = CreateLineElement2(Nothing, aRegion(0).AsLineElement.EndPoint, aRegion(1).AsLineElement.EndPoint)
    ActiveModelReference.AddElement aRegion(2)
    aRegion(2).Redraw
    
    'Test with non-closed element collection
    fillColor = fillColor + 1
    ActiveSettings.fillColor = fillColor
    Set oEnumerator = oEllipse.GetDifferenceShapesFromRegion(aRegion, oEllipse, msdFillModeFilled)
    Do While oEnumerator.MoveNext
        Set oElement = oEnumerator.Current
        ActiveModelReference.AddElement oElement
        oElement.Redraw
    Loop
        
    'Create element collection disjoint from ellipse
    Set aRegion(0) = CreateLineElement2(Nothing, Point3dAdd(sEllipse.Center, Point3dScale(sEllipse.Vector90, 5#)), Point3dAdd(sEllipse.Center, Point3dScale(sEllipse.Vector0, 2#)))
    ActiveModelReference.AddElement aRegion(0)
    aRegion(0).Redraw
    Set aRegion(1) = CreateLineElement2(Nothing, Point3dAdd(sEllipse.Center, Point3dScale(sEllipse.Vector90, 5#)), Point3dAdd(sEllipse.Center, Point3dScale(sEllipse.Vector90, 2#)))
    ActiveModelReference.AddElement aRegion(1)
    aRegion(1).Redraw
    
    'Test empty return
    fillColor = fillColor + 1
    ActiveSettings.fillColor = fillColor
    Set oEnumerator = oEllipse.GetIntersectionShapesFromRegion(aRegion, oEllipse, msdFillModeFilled)
    Do While oEnumerator.MoveNext
        Set oElement = oEnumerator.Current
        ActiveModelReference.AddElement oElement
        oElement.Redraw
    Loop
    
    ActiveSettings.fillColor = saveFillColor
End Sub
