Attribute VB_Name = "Module1"
Option Explicit

'-----------------------------------------------------------------------+
'                                                                       |
' Copyright 1993-2003, Bentley Systems, Incorporated                    |
' All rights reserved.                                                  |
'                                                                       |
' Limited permission is hereby granted to reproduce and modify this     |
' copyrighted material provided that the resulting code is used only in |
' conjunction with Bentley Systems products under the terms of the      |
' license agreement provided therein, and that this notice is retained  |
' in its entirety in any such reproduction or modification.             |
'                                                                       |
'----------------------------------------------------------------------*/
'-----------------------------------------------------------------------+
' formPosition
'
' Status: WORKING
'
' This example illustrates how to change the display order of elements
' by what level their on. It reads the level names from a text file, if
' the level exists in the current model, it places all the elements on
' that level into a selection set. It then uses the working set (wsetadd
' and wsetdrop) commands to change the display order. The first level in
' the text file will end up being "below" all the rest, and so on...
'
' Author: BSI                                                   02/13/03
'
'-----------------------------------------------------------------------+
'-----------------------------------------------------------------------+
'
'   name        LevelScanForElems
'
'   author      BSI                                             02/2003
'
'-----------------------------------------------------------------------+
Sub LevelScanForElems(lvlName As String)

    Dim oEnumerator As ElementEnumerator
    Dim oScan As New ElementScanCriteria
    Dim levelName As String
    Dim oElement As Element
    Dim oLevel As Level
    
    ' Sets the level to search for
    Set oLevel = ActiveDesignFile.Levels.Find(lvlName)
    
    oScan.ExcludeAllLevels
    oScan.IncludeLevel oLevel
    
    Set oEnumerator = ActiveModelReference.Scan(oScan)
    While oEnumerator.MoveNext
        
        ' Determines if the level from the text file matched the level found in the
        ' enumerator
        If StrComp(lvlName, oEnumerator.Current.Level.Name, vbTextCompare) = 0 Then
            If oEnumerator.Current.IsGraphical = True Then
                Set oElement = oEnumerator.Current
                
                ' This statement places the element into a selection set
                ActiveModelReference.SelectElement oElement, True
            End If
        End If
    Wend
    
    ' Determines if any elements are in a current selection set, if so, manipulate
    ' the working set
    If ActiveModelReference.AnyElementsSelected = True Then
        CadInputQueue.SendCommand "Wset add"
        CadInputQueue.SendCommand "Wset Drop"
        
        ' Removes all elements from the current selection set
        ActiveModelReference.UnselectAllElements
    End If
    
End Sub

'-----------------------------------------------------------------------+
'
'   name        LevelExist
'
'   author      BSI                                             05/2002
'
' This function will determine if the level is actually in the DGN.
'
'-----------------------------------------------------------------------+
Function LevelExist(levelName As String) As Boolean

    Dim oLevel As Level
    Set oLevel = ActiveDesignFile.Levels.Find(levelName)

    If oLevel Is Nothing Then
        LevelExist = False
    Else
        LevelExist = True
    End If
End Function

'-----------------------------------------------------------------------+
'
'   name        ChgDispOrder
'
'   author      BSI                                             02/2003
'
'-----------------------------------------------------------------------+
Sub ChgDispOrder()

    Dim textLine    As String
    Dim textFile    As String
    Dim lvlName     As String
    Dim Length      As String
    Dim inStringLoc As String
    
    ' Specifies the text file to open
    textFile = "D:\temp9\displayOrder\lvlNames.txt"
    
    ' Opens the text file
    Open textFile For Input As #1
    
    ' Loops through each line of text
    Do While Not EOF(1)
        Line Input #1, textLine
      
        ' Gets how many characters are in the string
        Length = Len(textLine)
        
        ' If a Tab character is at the end of the string (Excel
        ' will add this), this will locate it
        inStringLoc = InStr(textLine, vbTab)
        
        ' Gets the level name to be processed
        If inStringLoc = "0" Then
            lvlName = Mid(textLine, 1, Length)
        Else
            lvlName = Mid(textLine, 1, inStringLoc - 1)
        End If
        
        ' Determines if the level exists in the active model
        If LevelExist(lvlName) = True Then
            LevelScanForElems lvlName
        End If
    Loop
    
    ' Closes the text file
    Close #1
    
    ' Updates all the view windows
    RedrawAllViews
End Sub
