Attribute VB_Name = "FF"
Option Explicit

'----------------------------------------------------------------------+
'|                                                                       |
'| Copyright 1993-2001, Bentley Systems, Incorporated                    |
'| All rights reserved.                                                  |
'|                                                                       |
'| Limited permission is hereby granted to reproduce and modify this     |
'| copyrighted material provided that the resulting code is used only in |
'| conjunction with Bentley Systems products under the terms of the      |
'| license agreement provided therein, and that this notice is retained  |
'| in its entirety in any such reproduction or modification.             |
'|                                                                       |
'+----------------------------------------------------------------------*/
' This example VBA app wil use the Fence File command (FF=) to merge elements
' from the master DGN and References into a new DGN with a *.rcd extension.
' This was created to simulate the RefMerge MDL app that comes delivered with
' previous versions of MicroStation (v7.1 and lower). It does not have the
' ability to process as a initapp like RefMerge can.

' The elements that are copied will depend on if there level is turned ON.
' in the View Window that is used. There are 2 methods to determine which
' View Window is used:

' 1. The View Window number can be passed as an argument if the VBA app is run
'    from the Key-in tool. The command would be:

'    vba run ff.main 1

'    The above "1" being the desired view number.

' 2. If the app is run from the VBA interface or if it's run from the Key-in
'    tool without an argument (i.e. vba run ff.main), the last View Window
'    that a datapoint was placed in will be used.

'-----------------------------------------------------------------------+
'
'   name        Public Variables
'
'-----------------------------------------------------------------------+
Public filePath         As String
Public fileName         As String
Public fileNameAndExt   As String

'-----------------------------------------------------------------------+
'
'   name        fileGetFileName
'
'   Author      BSI                                         11/2001
'
'-----------------------------------------------------------------------+
Function fileGetFileName(filespec As String) As String
    Dim fs As Object
    Dim tmpstr As String
    Dim x As Integer
    
    Set fs = CreateObject("Scripting.FileSystemObject")
    tmpstr = fs.GetFileName(filespec)
    x = Len(tmpstr)
    'whatever.abc ".abc" = 4 characters
    fileGetFileName = Left(tmpstr, x - 4)
    
End Function

'-----------------------------------------------------------------------+
'
'   name        Main
'
'   Author      BSI                                         11/2001
'
'-----------------------------------------------------------------------+
Sub Main()
    
    Dim point           As Point3d
    Dim fenceType       As Integer
    Dim ViewWindow      As Integer
    Dim getArguments    As String
    
'   Initializes the variable with a null value
    getArguments = ""
            
'   Gets full path and file name
    filePath = ActiveDesignFile.Path
    
'   Gets the file name and extension
    fileNameAndExt = ActiveDesignFile.Name
    
'   Uses the fileGetFileName function to extract JUST the file name
    fileName = fileGetFileName(fileNameAndExt)
    
'   Gets the argument from the key-in tool
    getArguments = KeyinArguments
    
    '   If this app is run from the key-in tool AND an argument is passed,
    '   it will be used to define which View Window the Fence File command
    '   will use. The only valid argument that can be passed is a number of
    '   1 through 8, otherwise the last View Window the user clicked into
    '   will be used
    If getArguments = "1" Or getArguments = "2" Or getArguments = "3" Or getArguments = "4" Or getArguments = "5" Or getArguments = "6" Or getArguments = "7" Or getArguments = "8" Then
    
        ViewWindow = Val(getArguments)
        
    ElseIf getArguments = "" Then
    
        '   TCB variable that gets the last active View Window
        ViewWindow = GetCExpressionValue("tcb->lstvw") + 1
    Else
        MsgBox ("The argument can only be a number from 1 - 8")
        Exit Sub
        
    End If
    
    '   Uses a key-in to fit the desired View Window
    CadInputQueue.SendCommand "FIT VIEW EXTENDED " + CStr(ViewWindow)
    CadInputQueue.SendDataPoint point, ViewWindow
    
    '   Uses a key-in to Save Settings. This is being done now so that the
    '   elements in the new *.rcd will be fitted in the View Window
    CadInputQueue.SendCommand "FILEDESIGN "
    
'   Gets the current Fence Type
    fenceType = GetCExpressionValue("tcb->msToolSettings.fence.placeMode")

'   Sets the Fence Type to From View, this will make the fence be
'   around all the visible elements
    SetCExpressionValue "tcb->msToolSettings.fence.placeMode", 4&, ""
    
'   Uses a key-in to set the Fence Mode to Overlap (to be sure all
'   elements are being selected)
    CadInputQueue.SendCommand "LOCK FENCE OVERLAP "

'   Uses a key-in to start the Place Fence command
    CadInputQueue.SendCommand "PLACE FENCE ICON "

'   Send a data point to place the fence
    point.x = 0
    point.Y = 0
    point.Z = 0
    CadInputQueue.SendDataPoint point, ViewWindow

'   Uses the class module Macro1ModalHandler to process the image
    Dim modalHandler As New Macro1ModalHandler
    AddModalDialogEventsHandler modalHandler

'   Uses a key-in to initiate the fence file command
    CadInputQueue.SendKeyin "ff="
    
'   Send a data point to select the View Window and accept the Fence
'   File command
    CadInputQueue.SendDataPoint point, ViewWindow
    
'   Changes the Fence Type back to the original setting
    SetCExpressionValue "tcb->msToolSettings.fence.placeMode", fenceType, ""

'   Uses a key-in to set the Fence Mode to Inside (the most common setting)
    CadInputQueue.SendCommand "LOCK FENCE INSIDE "
    
'   Uses a key-in to set the View Window to what it was before the FF=
    CadInputQueue.SendCommand "VIEW PREVIOUS " + CStr(ViewWindow)
    CadInputQueue.SendDataPoint point, ViewWindow
    
'   Uses a key-in to start the Place Fence command (were doing this to remove
'   the fence from the View Window)
    CadInputQueue.SendCommand "PLACE FENCE ICON "
    
'   Unregisters a handler for modal dialog events
    RemoveModalDialogEventsHandler modalHandler
    
'   Redraws or updates all the View Windows
    RedrawAllViews
    
'   Uses a key-in to Save Settings
    CadInputQueue.SendCommand "FILEDESIGN "
    
'   Displaying message in the status window and clearing the prompt window
    ShowStatus "Processing Complete"
    ShowPrompt ""
    
'   Starts MicroStation's default command, sets MicroStation back to a normal state
    CommandState.StartDefaultCommand
    
End Sub
