Attribute VB_Name = "files"
Option Explicit
'-----------------------------------------------------------------------+
'                                                                       |
' Copyright 1993-2003, Bentley Systems, Incorporated                    |
' All rights reserved.                                                  |
'                                                                       |
' Limited permission is hereby granted to reproduce and modify this     |
' copyrighted material provided that the resulting code is used only in |
' conjunction with Bentley Systems products under the terms of the      |
' license agreement provided therein, and that this notice is retained  |
' in its entirety in any such reproduction or modification.             |
'                                                                       |
'----------------------------------------------------------------------*/
'----------------------------------------------------------------------------
' FileMultiSelect
'
' Status: WORKING
'
' Date: 07.17.03
'
' This example illustrates how to select multiple files.
'
' Author: BSI
'
'----------------------------------------------------------------------------

'-----------------------------------------------------------------------+
'
'   Declarations
'
'-----------------------------------------------------------------------+
Public Declare Function GetOpenFileName Lib "comdlg32.dll" Alias _
                        "GetOpenFileNameA" (pOpenFileName As OPENFILENAME) As Long

'-----------------------------------------------------------------------+
'
' Types
'
'-----------------------------------------------------------------------+
Public Type OPENFILENAME
    lStructSize         As Long
    hwndOwner           As Long
    hInstance           As Long
    lpstrFilter         As String
    lpstrCustomFilter   As String
    nMaxCustFilter      As Long
    nFilterIndex        As Long
    lpstrFile           As String
    nMaxFile            As Long
    lpstrFileTitle      As String
    nMaxFileTitle       As Long
    lpstrInitialDir     As String
    lpstrTitle          As String
    flags               As Long
    nFileOffset         As Integer
    nFileExtension      As Integer
    lpstrDefExt         As String
    lCustData           As Long
    lpfnHook            As Long
    lpTemplateName      As String
End Type

'-----------------------------------------------------------------------+
'
'   Constants
'
'-----------------------------------------------------------------------+
Public Const OFN_EXPLORER = &H80000
Public Const OFN_FILEMUSTEXIST = &H1000&
Public Const OFN_HIDEREADONLY = &H4&
Public Const OFN_PATHMUSTEXIST = &H800&
Public Const OFN_ALLOWMULTISELECT = &H200&
Public Const OFN_LONGNAMES = &H200000

'-----------------------------------------------------------------------+
'
'   name        FileMultiOpenDialog
'
'   author      BSI-DEVSPT                                  7/2003
'
'-----------------------------------------------------------------------+
Public Function FileMultiOpenDialog(strTitle As String, strDir As String, strFileExt As String, _
                               strFileExtDescription As String) As String

    ' SAMPLE USAGE:
    '
    ' Dim filename As String
    ' ...
    ' filename = FileMultiOpenDialog("Import Excel Spreadsheet data", "c:\", "*.xls", "Excel Spreadsheet")
    ' MsgBox (filename)
    
    Dim OpenFile    As OPENFILENAME
    Dim status      As Long
    Dim pos         As Long     ' Used for trimming C style string.
    Dim strTmp      As String   ' Used for trimming C style string.
    Dim strFilter   As String
         
    ' Populate the OPENFILENAME data type
    OpenFile.lStructSize = Len(OpenFile)
    OpenFile.hwndOwner = 0
    OpenFile.hInstance = 0 'App.hInstance
    strFilter = strFileExtDescription & " (" & strFileExt & ")" & Chr(0) & strFileExt & Chr(0)
    OpenFile.lpstrFilter = strFilter
    OpenFile.nFilterIndex = 1
    OpenFile.lpstrInitialDir = strDir
    OpenFile.lpstrTitle = strTitle
    OpenFile.lpstrFile = String(257, 0)
    OpenFile.nMaxFile = Len(OpenFile.lpstrFile) - 1
    OpenFile.lpstrFileTitle = OpenFile.lpstrFile
    OpenFile.nMaxFileTitle = OpenFile.nMaxFile
    OpenFile.flags = OFN_ALLOWMULTISELECT
    
    ' Open the file open dialog box
    status = GetOpenFileName(OpenFile)
         
    ' Return the file selected
    If status = 0 Then
        FileMultiOpenDialog = ""
    Else
        ' Trim C style string for compatibility with VB strings.
        strTmp = Chr$(0)
        pos = InStr(OpenFile.lpstrFile, strTmp)
        FileMultiOpenDialog = Left(OpenFile.lpstrFile, pos - 1)
    End If
    
End Function

'-----------------------------------------------------------------------+
'
'   name        main
'
'   author      BSI-DEVSPT                                  7/2003
'
'-----------------------------------------------------------------------+
Sub main()

Dim i As Integer
Dim Selectedfiles As String
Dim myFiles() As String
Dim myPath As String

    'Select File /s
    Selectedfiles = FileMultiOpenDialog("File Muti Select", "c:\", "*.*", "files")

    'Split on the spaces
    myFiles = Split(Selectedfiles, " ")
    
    Select Case UBound(myFiles)
        Dim msg
        
        'if only one was selected we are done
        Case 0
            msg = myFiles(0)
        
        'if more than one, we need to loop through it
        'and append the root directory
        Case Is > 0
          
            For i = 1 To UBound(myFiles)
                myPath = myFiles(0) & IIf(Right(myFiles(0), 1) <> "\", "\", "") & myFiles(i)
                msg = msg + myPath + vbCrLf
            Next i
        
        ' Display the files selected
        MsgBox msg
        
    End Select
    
End Sub
