Attribute VB_Name = "Module1"
Option Explicit

'-----------------------------------------------------------------------+
'                                                                       |
' Copyright 1993-2003, Bentley Systems, Incorporated                    |
' All rights reserved.                                                  |
'                                                                       |
' Limited permission is hereby granted to reproduce and modify this     |
' copyrighted material provided that the resulting code is used only in |
' conjunction with Bentley Systems products under the terms of the      |
' license agreement provided therein, and that this notice is retained  |
' in its entirety in any such reproduction or modification.             |
'                                                                       |
'----------------------------------------------------------------------*/
'-----------------------------------------------------------------------+
' SymbologySet
'
' Status: WORKING
'
' This example will delete all unused levels.
'
' Author: BSI                                                   09/10/03
'
'-----------------------------------------------------------------------+
'-----------------------------------------------------------------------+
'
'   name        LevelHasNoElements
'
'   author      BSI                                             09/2003
'
'-----------------------------------------------------------------------+
Function LevelHasNoElements(oLevel As Level) As Boolean

    Dim oElementEnumerator  As ElementEnumerator
    Dim oScanCriteria       As New ElementScanCriteria
    
    ' Exclude these parameters
    oScanCriteria.ExcludeAllLevels
    oScanCriteria.ExcludeNonGraphical
    
    ' Include only the level that we need to determine if any elements
    ' are on it
    oScanCriteria.IncludeLevel oLevel
    
    Set oElementEnumerator = ActiveModelReference.Scan(oScanCriteria)
    
    While oElementEnumerator.MoveNext
        LevelHasNoElements = False
        Exit Function
    Wend
    LevelHasNoElements = True

End Function

'-----------------------------------------------------------------------+
'
'   name        LevelDelete
'
'   author      BSI                                             09/2003
'
'-----------------------------------------------------------------------+
Sub LevelDelete(LevelArray() As String)
    
    Dim oLevel  As Level
    Dim count   As Integer
    
    ' Iterate through the levels to be processed and delete the levels
    For count = 0 To UBound(LevelArray)
        Set oLevel = ActiveDesignFile.Levels(LevelArray(count))
        ActiveDesignFile.DeleteLevel oLevel
        ActiveDesignFile.RewriteLevels
    Next

End Sub

'-----------------------------------------------------------------------+
'
'   name        LevelsDeleteUnused
'
'   author      BSI                                             09/2003
'
'-----------------------------------------------------------------------+
Sub LevelsDeleteUnused()

    Dim oLevel          As Level
    Dim oLevels         As Levels
    Dim counter         As Integer
    Dim LevelArray()    As String
    
    counter = -1
    
    Set oLevels = ActiveDesignFile.Levels
    
    ' Iterate through each level
    For Each oLevel In oLevels
    
        ' If the level is in a DgnLib it's empty so ignore it
        If oLevel.IsFromLevelLibrary = False Then
        
            ' If the level has no elements on it, it will be deleted
            If LevelHasNoElements(oLevel) = True Then
            
                ' An error will occur if the Default level is deleted
                If oLevel.Name <> "Default" Then
                    counter = counter + 1
                    
                    ' Add each level name into the array
                    ReDim Preserve LevelArray(counter)
                    LevelArray(counter) = oLevel.Name
                End If
            End If
        End If
    Next

    ' If no levels are to be proccessed don't use LevelDelete()
    If counter > -1 Then
        LevelDelete LevelArray
    End If
    
End Sub
