Attribute VB_Name = "Module1"
Option Explicit

' User defined types
Private Type RECT
   Left As Long
   Top As Long
   Right As Long
   Bottom As Long
End Type

' User defined constants
Public Const SM_CXSCREEN = 0
Public Const SM_CYSCREEN = 1

' Win32 API functions imported for this example
Private Declare Function GetCursorPos Lib "user32" (lpRect As RECT) As Long
Private Declare Function GetDesktopWindow Lib "user32" () As Long
Private Declare Function GetSystemMetrics Lib "user32" (ByVal nIndex As Long) As Long
Private Declare Function GetWindowRect Lib "user32" (ByVal hWnd As Long, lpRect As RECT) As Long
Private Declare Function SetCursorPos Lib "user32" (ByVal x As Long, ByVal y As Long) As Long

'----------------------------------------------------------------------+
'
'   name        CursorPositionSet
'
'   author      BSI - Developer Support                 8/2002
'
'----------------------------------------------------------------------+
'
'   Description:    Set the current cursor position.
'
'   Version:        1.0
'
'   Example:
'                   ' Set the current cursor position
'                   Dim rectScreen As RECT
'                   CursorPositionSet rectScreen
'
'----------------------------------------------------------------------+
Public Sub CursorPositionSet(ByRef rectScreen As RECT)

    ' Set the current cursor position
    SetCursorPos rectScreen.Left, rectScreen.Top

End Sub

'----------------------------------------------------------------------+
'
'   name        CursorPositionGet
'
'   author      BSI - Developer Support                 8/2002
'
'----------------------------------------------------------------------+
'
'   Description:    Extract the current cursor position.
'
'   Version:        1.0
'
'   Example:
'                   ' Get the current cursor position
'                   Dim rectScreen As RECT
'                   CursorPositionGet rectScreen
'
'----------------------------------------------------------------------+
Public Sub CursorPositionGet(ByRef rectScreen As RECT)
    
    ' Get the current cursor location
    GetCursorPos rectScreen

End Sub

'----------------------------------------------------------------------+
'
'   name        CursorPositionTest
'
'   author      BSI - Developer Support                 8/2002
'
'----------------------------------------------------------------------+
Public Sub CursorPositionTest()

    Dim rectScreen As RECT
    Dim rectMove As RECT
    Dim dRatio As Double
    Dim x As Long
    Dim y As Long
    Dim status As Integer

    ' Display the desktop resolution in the MicroStation message center and calculate screen proportion
    Call GetWindowRect(GetDesktopWindow(), rectScreen)
    MicroStationDGN.Application.ShowMessage "Screen Resolution: [X] = " & CStr(rectScreen.Right) & "[Y] = " & CStr(rectScreen.Bottom)
    dRatio = rectScreen.Right / rectScreen.Bottom
    
    ' Display the cursor position in the MicroStation message center
    CursorPositionGet rectScreen
    MicroStationDGN.Application.ShowMessage "Starting Cursor Postion: [X] = " & CStr(rectScreen.Left) & "[Y] = " & CStr(rectScreen.Top)
    
    ' Move cursor diagonally across the extent of the screen
    For x = 0 To rectScreen.Right Step 1
        
        ' Set the current cursor position
        rectMove.Left = x
        rectMove.Top = CInt(x / dRatio)
        CursorPositionSet rectMove

    Next x
    
    ' Center the cursor on the screen
    x = GetSystemMetrics(SM_CXSCREEN) / 2
    y = GetSystemMetrics(SM_CYSCREEN) / 2
    status = SetCursorPos(x, y)
    MicroStationDGN.Application.ShowMessage "Final Cursor Postion: [X] = " & CStr(x) & "[Y] = " & CStr(y)

End Sub
