Attribute VB_Name = "CellChgType"
Option Explicit
'-----------------------------------------------------------------------+
'
'   CellTypeChange.mvba
'   1.0
'   01.16.2004
'
'   Copyright: (c) 2004 Bentley Systems, Incorporated. All rights reserved.
'
'----------------------------------------------------------------------+
'----------------------------------------------------------------------+
'
'   STATUS: WORKING
'   DATE:   01/16/2004
'
'   This example illustrates how to change a Model's Cell Property
'   Cell Type to either 'Graphic' or 'Point'.
'
'----------------------------------------------------------------------+
Declare Function mdlModelRef_setCellType Lib "stdmdlbltin.dll" (ByVal _
                        modelRef As Long, ByRef cellType As Long) As Long
                        
Declare Function mdlModelRef_getCellType Lib "stdmdlbltin.dll" (ByVal _
                        modelRef As Long, ByRef cellType As Long) As Long
                        
'Constants
Private Const CELLLIBTYPE_GRAPHIC   As Integer = 0
Private Const CELLLIBTYPE_MENU      As Integer = 1
Private Const CELLLIBTYPE_POINT     As Integer = 7
                        
'----------------------------------------------------------------------+
'
'   Name        CellTypeNameFromNumber
'
'   Author      BSI-DEVSPT                                  01/2004
'
'----------------------------------------------------------------------+
Public Function CellTypeNameFromNumber(lngCellTypeNum As Long) As String

    ' Get the name of the Cell Type from it's number
    Select Case lngCellTypeNum
        Case 0: CellTypeNameFromNumber = "CELLLIBTYPE_GRAPHIC"
        Case 1: CellTypeNameFromNumber = "CELLLIBTYPE_MENU"
        Case 7: CellTypeNameFromNumber = "CELLLIBTYPE_POINT"
        Case Else: CellTypeNameFromNumber = "Unknown Cell Type" & CStr(lngCellTypeNum)
    End Select
    
End Function
                        
'----------------------------------------------------------------------+
'
'   Name        CellTypeGet
'
'   Author      BSI - Developer Support                 01/2004
'
'----------------------------------------------------------------------+
Function CellTypeGet(oModel As ModelReference) As Long

    Dim status      As Long
    Dim lngCellType As Long
    Dim modelRefP   As Long
    
    ' Get the modelreference pointer
    modelRefP = oModel.MdlModelRefP
    
    ' Get the model's Cell Type
    If (mdlModelRef_getCellType(modelRefP, lngCellType) = 0) Then
        CellTypeGet = lngCellType
        Exit Function
    End If
    
    CellTypeGet = status
    
End Function
                        
'----------------------------------------------------------------------+
'
'   Name        CellTypeSet
'
'   Author      BSI - Developer Support                 01/2004
'
'----------------------------------------------------------------------+
Function CellTypeSet(oModel As ModelReference, lngCellType As Long) As Long

    Dim status      As Long
    Dim modelRefP   As Long
    
    ' Get the modelreference pointer
    modelRefP = oModel.MdlModelRefP
    
    ' Change the model's Cell Type
    status = mdlModelRef_setCellType(modelRefP, lngCellType)
    
    CellTypeSet = status
    
End Function
                      
'----------------------------------------------------------------------+
'
'   Name        StartApp
'
'   Author      BSI - Developer Support                 01/2004
'
'----------------------------------------------------------------------+
Sub StartApp()
    
    Dim status              As Long
    Dim lngModelCntr        As Long
    Dim lngModelCngCntr     As Long
    Dim lngCellTypeExist    As Long
    Dim lngCellTypeNew      As Long
    Dim oModel              As ModelReference
    Dim oModels             As ModelReferences
    
    ' To change the model's cell type, Specify CELLLIBTYPE_GRAPHIC(0) or
    ' CELLLIBTYPE_POINT(7)
    lngCellTypeNew = CELLLIBTYPE_GRAPHIC
    
    ' Set the Models object to contain all the models in the active DGN
    Set oModels = ActiveDesignFile.Models
        
    ' Iterate through each model
    For Each oModel In oModels
    
        Debug.Print "Model Found: '"; oModel.Name; "'"
        
        ' Add one to the model counter
        lngModelCntr = lngModelCntr + 1
        
        ' Determine if the current model already has the Cell Type specified
        If (CellTypeGet(oModel) = lngCellTypeNew) Then
        
            Debug.Print "Cell type is already: '"; _
                        CStr(CellTypeNameFromNumber(lngCellTypeNew)); _
                        "' for Model: '"; oModel.Name; "'"
        Else
            ' Change the Cell Type
            status = CellTypeSet(oModel, lngCellTypeNew)
        
            If status <> 0 Then
                Debug.Print "The Cell Type could not be changed in the Model: '"; _
                            oModel.Name; "' The status returned is: '"; status; "'"
            Else
                lngModelCngCntr = lngModelCngCntr + 1
                Debug.Print "The Cell Type has been changed in the Model: '"; _
                            oModel.Name; "' to: '"; _
                            CStr(CellTypeNameFromNumber(lngCellTypeNew)); "'"
            End If
        End If
        Debug.Print "= = = = = = = = = = = = = = ="
    Next
    
    Debug.Print "The number of Models found:  "; lngModelCntr
    Debug.Print "The number of Models changed:"; lngModelCngCntr
    
End Sub
