Attribute VB_Name = "ShowModels"
'----------------------------------------------------------------------+
' Module:   ShowModels.bas
'
' Status:   WORKING
'
' Date:  04/27/04
'
' This example illustrates how to open another file using the
' OpenDesignFileForProgram function
'
' Author: BSI-DEVSPT
'
' Copyright: (c) Bentley Systems, Incorporated. All rights reserved.
'
'-----------------------------------------------------------------------+
'*---------------------------------------------------------------------------------**''**
' Public Declrations
'---------------+---------------+---------------+---------------+---------------+------''
Public Declare Function GetOpenFileName Lib "comdlg32.dll" Alias _
                        "GetOpenFileNameA" (pOpenFileName As OPENFILENAME) As Long

'*---------------------------------------------------------------------------------**''**
' Public Types
'---------------+---------------+---------------+---------------+---------------+------''
Public Type OPENFILENAME
    lStructSize         As Long
    hwndOwner           As Long
    hInstance           As Long
    lpstrFilter         As String
    lpstrCustomFilter   As String
    nMaxCustFilter      As Long
    nFilterIndex        As Long
    lpstrFile           As String
    nMaxFile            As Long
    lpstrFileTitle      As String
    nMaxFileTitle       As Long
    lpstrInitialDir     As String
    lpstrTitle          As String
    flags               As Long
    nFileOffset         As Integer
    nFileExtension      As Integer
    lpstrDefExt         As String
    lCustData           As Long
    lpfnHook            As Long
    lpTemplateName      As String
End Type

'*---------------------------------------------------------------------------------**''**
' Public constants
'---------------+---------------+---------------+---------------+---------------+------''
Public Const OFN_EXPLORER = &H80000
Public Const OFN_FILEMUSTEXIST = &H1000&
Public Const OFN_HIDEREADONLY = &H4&
Public Const OFN_PATHMUSTEXIST = &H800&

Public Const TXT_DIALOGTITLE = "OpenFileFromProgram"
Public Const TXT_DIR = "C:\"
Public Const TXT_Ext = "*.dgn"

'*---------------------------------------------------------------------------------**''**
' @description  Open the windows file open dialog using the windows API.
' @param        strTitle Title of the dialog
' @param        strDir Starting directory
' @param        strFileExt File extension
' @param        strFileExtDescription Description
' @return       A String of a full path and file name of the file choosen
' @version      1.0
' @dependencies
' @example      strFileName = FileOpenDialog("TEST", "c:\", "*.dgn", "File to Test")
'
' @remarks
' @alinkjoin
' @group
' @bsimethod                                                    BSI-DEVSPT      04/04
'---------------+---------------+---------------+---------------+---------------+------''
Public Function FileOpenDialog(strTitle As String, strDir As String, strFileExt As String, _
                               strFileExtDescription As String) As String

    ' SAMPLE USAGE:
    '
    ' Dim filename As String
    ' ...
    ' filename = FileOpenDialog("Import Excel Spreadsheet data", "c:\", "*.xls", "Excel Spreadsheet")
    ' MsgBox (filename)

    Dim OpenFile    As OPENFILENAME
    Dim status      As Long
    Dim pos         As Long     ' Used for trimming C style string.
    Dim strTmp      As String   ' Used for trimming C style string.
    Dim strFilter   As String

    ' Populate the OPENFILENAME data type
    OpenFile.lStructSize = Len(OpenFile)
    OpenFile.hwndOwner = 0
    OpenFile.hInstance = 0 'App.hInstance
    strFilter = strFileExtDescription & " (" & strFileExt & ")" & Chr(0) & strFileExt & Chr(0)
    OpenFile.lpstrFilter = strFilter
    OpenFile.nFilterIndex = 1
    OpenFile.lpstrInitialDir = strDir
    OpenFile.lpstrTitle = strTitle
    OpenFile.lpstrFile = String(257, 0)
    OpenFile.nMaxFile = Len(OpenFile.lpstrFile) - 1
    OpenFile.lpstrFileTitle = OpenFile.lpstrFile
    OpenFile.nMaxFileTitle = OpenFile.nMaxFile
    OpenFile.flags = 0

    ' Open the file open dialog box
    status = GetOpenFileName(OpenFile)

    ' Return the file selected
    If status = 0 Then
        FileOpenDialog = ""
    Else
        ' Trim C style string for compatibility with VB strings.
        strTmp = Chr$(0)
        pos = InStr(OpenFile.lpstrFile, strTmp)
        FileOpenDialog = Left(OpenFile.lpstrFile, pos - 1)
    End If

End Function

'*---------------------------------------------------------------------------------**''**
' @description      Starting point for example
' @param            void
' @return           void
' @version          1.0
' @dependencies
' @example
'
' @remarks
' @alinkjoin
' @group
' @bsimethod                                                    BSI-DEVSPT      04/04
'---------------+---------------+---------------+---------------+---------------+------''
Sub main()

    ' Open the form
    frmModels.Show

End Sub
