Attribute VB_Name = "ElementsProcess"
Option Explicit

Declare Function mdlModelRef_getModelID Lib "stdmdlbltin.dll" (ByVal modelRef As Long) As Long

'-----------------------------------------------------------------------+
'                                                                       |
' Copyright 1993-2003, Bentley Systems, Incorporated                    |
' All rights reserved.                                                  |
'                                                                       |
' Limited permission is hereby granted to reproduce and modify this     |
' copyrighted material provided that the resulting code is used only in |
' conjunction with Bentley Systems products under the terms of the      |
' license agreement provided therein, and that this notice is retained  |
' in its entirety in any such reproduction or modification.             |
'                                                                       |
'----------------------------------------------------------------------*/
'-----------------------------------------------------------------------+
' RefElemDelete
'
' Status: WORKING
'
' This example illustrates how to delete elements from a reference file
' using the ILocateCommandEvents oject and mdlWorkDgn_XXX functions.
'
' Author: BSI                                                   10/01/03
'
'-----------------------------------------------------------------------+
'-----------------------------------------------------------------------+
'
'   Public variables
'
'-----------------------------------------------------------------------+
Public strDGNName       As String
Public modelRef         As Long
Public oModelReference  As ModelReference

'-----------------------------------------------------------------------+
'
'   name        UnicodeStringGet
'
'   author      BSI                                             10/2003
'
'-----------------------------------------------------------------------+
Function UnicodeStringGet(strValue As String) As Long
    If strValue <> "" Then
        UnicodeStringGet = StrPtr(strValue)
    End If
End Function

'-----------------------------------------------------------------------+
'
'   name        ElementRemove
'
'   author      BSI                                             10/2003
'
'-----------------------------------------------------------------------+
Sub ElementRemove(lngElementID As Long, oModelReference As ModelReference)
    
    Dim oScanCriteria          As New ElementScanCriteria
    Dim oEnumerator            As ElementEnumerator
    Dim ElementCounter         As Integer
    
    ' Only scan for graphic elements
    oScanCriteria.ExcludeNonGraphical
    Set oEnumerator = oModelReference.Scan(oScanCriteria)
    
    ' Iterate through the model's elements to find the correct one
    While oEnumerator.MoveNext
        If oEnumerator.Current.ID.Low = lngElementID Then
        
            ' Delete the element
            oModelReference.RemoveElement oEnumerator.Current
        End If
    Wend
End Sub

'-----------------------------------------------------------------------+
'
'   name        ModelsScan
'
'   author      BSI                                             10/2003
'
'-----------------------------------------------------------------------+
Sub ModelsScan(lngElementID As Long, ModelName As String)

    Dim oModelReferences As ModelReferences
    Set oModelReferences = ActiveDesignFile.Models
    
    ' Scan the active DGN for models
    For Each oModelReference In oModelReferences
    
        ' When the correct model is found delete the element
        If oModelReference.Name = ModelName Then
            ElementRemove lngElementID, oModelReference
        End If
    Next
End Sub

'-----------------------------------------------------------------------+
'
'   name        ElementProcess
'
'   author      BSI                                             10/2003
'
'-----------------------------------------------------------------------+
Sub ElementProcess(oElement As Element)
    
    Dim lngModelName     As Long
    Dim lngReadOnly     As Long
    Dim strDGNFullPath  As String
    Dim ModelName       As String
    
    ' DGN file name from the element selected
    strDGNName = oElement.ModelReference.DesignFile.Name
    
    ' DGN full path and name from the element selected
    strDGNFullPath = oElement.ModelReference.DesignFile.FullName
    
    ' Process the element differently depending if it is in the active DGN
    ' or in a reference from another DGN
    If strDGNFullPath = ActiveDesignFile.FullName Then
        
        If oElement.ModelReference.Name = ActiveModelReference.Name Then
        
            ' If the element is in the active model, delete it
            ActiveModelReference.RemoveElement oElement
            Exit Sub
        Else
            ' Process the element if it is in the active DGN but not in the
            ' active model
            ModelsScan oElement.ID.Low, oElement.ModelReference.Name
        End If
    Else
        ' Process the element if it is not in the active DGN
        
        ' The name of the model to get from the file once it is opened
        ModelName = oElement.ModelReference.Name
        
        ' Creates the address of the data in the string
        lngModelName = UnicodeStringGet(ModelName)
         
        ' Open the DGN as read/write
        lngReadOnly = 0
         
        ' Open the DGN with the WorkDgn functions and return the DGN's
        ' ModelReference object
        Set oModelReference = WorkDgnOpenFile(strDGNFullPath, lngModelName, lngReadOnly)
        
        ' If the object is populated keep processing
        If Not oModelReference Is Nothing Then
            WorkDgnElementRemove oModelReference, oElement.ID.Low
             
            ' Close the DGN
            WorkDgnCloseFile modelRef, oModelReference
        Else
            MsgBox "The DGN specified could not be opened"
        End If
    End If
End Sub

'-----------------------------------------------------------------------+
'
'   name        StartApp
'
'   author      BSI                                             10/2003
'
'-----------------------------------------------------------------------+
Sub StartApp()

    ' Set the ModelReference object to NULL
    Set oModelReference = Nothing
    
    ' Implements an element-modification command
    CommandState.StartLocate New ReferenceElementDelete
End Sub


