Attribute VB_Name = "SavedViewOriginChange"
Option Explicit

'-----------------------------------------------------------------------+
'
'   SavedViewOriginChg.mvba
'   1.0
'   01.23.2004
'
'   Copyright: (c) 2004 Bentley Systems, Incorporated. All rights reserved.
'
'----------------------------------------------------------------------+
'----------------------------------------------------------------------+
'
'   STATUS: WORKING
'   DATE:   01/23/2004
'
'   This example illustrates how to change the origin of a Saved View
'   object.
'
'----------------------------------------------------------------------+
'----------------------------------------------------------------------+
'
'   MDL Function Declarations
'
'----------------------------------------------------------------------+
Declare Function mdlView_extractElmdscr Lib "stdmdlbltin.dll" (ByRef viewInfoP _
                As Long, ByRef ViewPortInfoP As Long, ByVal eDP As Long) As Long
                
Declare Function mdlView_extractNamedView Lib "stdmdlbltin.dll" (ByVal name As _
                Long, ByVal description As Long, ByVal nameSpace_ As Long, _
                ByRef origin As Point3d, ByRef delta As Point3d, ByRef activeZ _
                As Double, ByVal flags As Long, ByRef rotMatrixP As Matrix3d, _
                ByVal cameraInfoP As Long, ByVal eDP As Long) As Long
                
Declare Function mdlViewInfo_create Lib "stdmdlbltin.dll" (ByRef viewInfoPP As _
                Long) As Long
                
Declare Function mdlViewInfo_free Lib "stdmdlbltin.dll" (ByRef viewInfoPP As _
                Long) As Long
                
Declare Function mdlViewInfo_setOrigin Lib "stdmdlbltin.dll" (ByVal viewInfoP _
                As Long, ByRef originP As Point3d) As Long
                
Declare Function mdlView_saveNamedElmdscr Lib "stdmdlbltin.dll" (ByRef replaceP _
                As Long, ByVal eDP As Long) As Long
                
Declare Function mdlView_createElmdscr Lib "stdmdlbltin.dll" (ByRef eDPP As Long, _
                ByVal viewInfoP As Long, ByVal ViewPortInfoP As Long) As Long
                
Declare Function mdlView_getNameOfElmdscr Lib "stdmdlbltin.dll" (ByVal viewNameP _
                As Long, ByVal eDP As Long) As Long
                
Declare Function mdlView_setNameOfElmdscr Lib "stdmdlbltin.dll" (ByRef eDPP As _
                Long, ByVal viewNameP As Long) As Long
                
Declare Sub mdlElmdscr_freeAll Lib "stdmdlbltin.dll" (ByRef elemDescr As Long)

'----------------------------------------------------------------------+
'
'   Name        SavedViewGetElement
'
'   Author      BSI-DEVSPT                                  01/2004
'
'----------------------------------------------------------------------+
Function SavedViewGetElement(savedViewName As String) As SavedViewElement
    Dim oScan As New ElementScanCriteria
    Dim oEnum As ElementEnumerator
    Dim oSavedViewElement As SavedViewElement
    
    oScan.ExcludeAllTypes
    oScan.IncludeType msdElementTypeView
    Set oEnum = ActiveModelReference.Scan(oScan)
    Do While oEnum.MoveNext
        Set oSavedViewElement = oEnum.Current
        If oEnum.Current.AsSavedViewElement.name = savedViewName Then
            Set SavedViewGetElement = oEnum.Current
            Exit Function
        End If
    Loop
    Err.Raise 513, , "No such saved view as """ & savedViewName & """"
End Function

'----------------------------------------------------------------------+
'
'   Name        PntStr
'
'   Author      BSI-DEVSPT                                  01/2004
'
'----------------------------------------------------------------------+
Function PntStr(pnt As Point3d) As String
    With pnt
        PntStr = "(" & .X & ", " & .Y & ", " & .Z & ")"
    End With
End Function

'----------------------------------------------------------------------+
'
'   Name        SavedViewModify
'
'   Author      BSI-DEVSPT                                  01/2004
'
'----------------------------------------------------------------------+
Sub SavedViewModify()
    
    Dim status          As Long
    Dim viewInfo        As Long
    Dim elmDscrP        As Long
    Dim elmDscripter    As Long
    Dim ViewPortInfoP   As Long
    Dim viewNameP       As Long
    Dim activeX         As Double
    Dim origin          As Point3d
    Dim delta           As Point3d
    Dim originNew       As Point3d
    Dim rotMatrix3d     As Matrix3d
    Dim oSavedView      As SavedViewElement
        
    ' Set the saved view object with an existing saved view
    Set oSavedView = SavedViewGetElement("circle")
    
    ' Create an element descriptor of the saved view object
    elmDscrP = oSavedView.MdlElementDescrP
    
    ' Extract the origin and other saved view attributes
    status = mdlView_extractNamedView(0, 0, 0, origin, delta, activeX, 0, _
                                        rotMatrix3d, 0, elmDscrP)
                                        
    ' Multiplies a vector by a scale factor, in this case it converts master
    ' units to units of resolution
    origin = Point3dScale(origin, ActiveModelReference.UORsPerMasterUnit)
    
    ' Print the original origin values to the VBA Immediate window
    Debug.Print "Original Origin is: " & PntStr(origin)
    
    ' Double the X value for this example
    originNew.X = origin.X + origin.X
    originNew.Y = origin.Y
    originNew.Z = origin.Z
    
    ' Create a view info object
    status = mdlViewInfo_create(viewInfo)
    
    ' Extracts ViewInfo and ViewPortInfo from the view specified by its element descriptor
    status = mdlView_extractElmdscr(viewInfo, ViewPortInfoP, elmDscrP)
    
    ' Set the new origin value to the view info object
    status = mdlViewInfo_setOrigin(viewInfo, originNew)
    
    ' Create a new element descriptor for the modified saved view (view info object)
    status = mdlView_createElmdscr(elmDscripter, viewInfo, ViewPortInfoP)
    
    ' Extract the origin and other saved view attributes
    status = mdlView_extractNamedView(0, 0, 0, origin, delta, activeX, 0, _
                                        rotMatrix3d, 0, elmDscripter)
                                        
    ' Print the new origin values to the VBA Immediate window
    Debug.Print "New Origin is: " & PntStr(Point3dScale(origin, ActiveModelReference.UORsPerMasterUnit))
        
    ' Save the element descriptor
    status = mdlView_saveNamedElmdscr(1, elmDscripter)
    
    ' Free the ViewInfoP descriptor
    status = mdlViewInfo_free(viewInfo)
    
    ' Free the element descriptor
    mdlElmdscr_freeAll (elmDscrP)
    
    ' Free the element descriptor
    mdlElmdscr_freeAll (elmDscripter)

End Sub
