VERSION 5.00
Begin {C62A69F0-16DC-11CE-9E98-00AA00574A4F} frmElmTotals 
   Caption         =   "Element Totals"
   ClientHeight    =   1545
   ClientLeft      =   45
   ClientTop       =   330
   ClientWidth     =   3255
   OleObjectBlob   =   "frmElmTotals.frx":0000
   ShowModal       =   0   'False
   StartUpPosition =   1  'CenterOwner
End
Attribute VB_Name = "frmElmTotals"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
'-----------------------------------------------------------------------+
'                                                                       |
' Copyright 1993-2001, Bentley Systems, Incorporated                    |
' All rights reserved.                                                  |
'                                                                       |
' Limited permission is hereby granted to reproduce and modify this     |
' copyrighted material provided that the resulting code is used only in |
' conjunction with Bentley Systems products under the terms of the      |
' license agreement provided therein, and that this notice is retained  |
' in its entirety in any such reproduction or modification.             |
'                                                                       |
'----------------------------------------------------------------------*/
'-----------------------------------------------------------------------+
' ElementTotals
'
' Status: WORKING
'
' Date: 05/01/01
'
' This example will scan the current design file for all possible
' element types within the design file and calculate the total
' quantity of each element type.
'
' All possible element types are then populated into a combo box
' for the user to select to get the total quantity.
'
' Tools Required:
'   None
'
' Author: Scott Gabel                                     08/24/01
'
'-----------------------------------------------------------------------+

'-----------------------------------------------------------------------+
'
'   Globals
'
'-----------------------------------------------------------------------+
Option Explicit

Dim strIndElementName()     As String       'Array of Possible element names.
Dim lngIndElementCount()    As Long      'Array of totals of all individual element names.
Dim intTotElementNames      As Integer      'Total count of all possible element names.

' Program is initially loaded
Private Sub UserForm_Initialize()
    cmdScan_Click
End Sub

' A value in the Combo Box changes
Private Sub cboElementName_Change()
    Dim intPos As Integer
    
    'Loop thru the strIndElementName array to get the index position of the selected
    'item in the combo box only if combo box is not empty
    If cboElementName.ListIndex <> -1 Then
        intPos = ElementNameGetPosition(cboElementName.List(cboElementName.ListIndex))
        'Put totals of the selected element in quantity label
        lblQuantity = lngIndElementCount(intPos)
    End If

End Sub

' The "Scan" button is clicked
Private Sub cmdScan_Click()

    Dim oScanCriteria As New ElementScanCriteria
    Dim strElementName As String
    Dim intPos As Integer
    Dim intElementTotal As Long
    
    Dim enumerator As ElementEnumerator
    Set enumerator = ActiveModelReference.Scan(oScanCriteria)
        
    ' Reset arrays and number of Elements totals
    ReDim strIndElementName(0)
    ReDim lngIndElementCount(0)
    intTotElementNames = 0
        
    ' Clear quantity label, update prompt, clear combo box text if any
    lblQuantity = ""
    ShowStatus "Scanning Elements"
        
    '  Step through the scan results
    Do While enumerator.MoveNext
        
        With enumerator.Current
            
            ' Update Element totals
            intElementTotal = intElementTotal + 1
                        
            ' Get name of element
            strElementName = elemNameFromNumber(.Type)
            
            ' If this element type already exists in the design file
            If ElementNameExists(strElementName) Then
                
                ' Get position of this element type from strIndElementName array and update the total
                ' count of this element to the lngIndElementCount array according to the index
                intPos = ElementNameGetPosition(strElementName)
                lngIndElementCount(intPos) = lngIndElementCount(intPos) + 1
                       
            ' Else this is a new element type found in the design file
            Else
                
                ' Increment number of possible element types by 1
                intTotElementNames = intTotElementNames + 1
                
                ' Redimension strIndElementName array to number of possible element types
                ReDim Preserve strIndElementName(intTotElementNames)
                
                ' Redimension lngIndElementCount array to number of possible element types
                ReDim Preserve lngIndElementCount(intTotElementNames)
                
                ' Populate element name to strIndElementName array
                strIndElementName(intTotElementNames) = strElementName
                
                ' Add this element by 1 to elementcount array
                lngIndElementCount(intTotElementNames) = lngIndElementCount(intTotElementNames) + 1
            
            End If
        
        End With
    Loop

    ' If elements are found in the design file
    If intTotElementNames <> 0 Then
        
        ' Clear out items in combo box if any
        cboElementName.Clear
    
        ' Populate combo box with strIndElementName array
        ComboBoxPopulate
    
        ' Display total count of all elements
        lblTotalElements = Str(intElementTotal)
        
        ' Update prompt
        ShowStatus "Scan Complete"
        
    ' No elements were found in the design file
    Else
        ' Clear out items in combo box if any
        cboElementName.Clear
        
        ' Display no elements found message
        lblTotalElements = "0"
    
        ' Update prompt
        ShowStatus "No Elements found"
    End If

End Sub

'-----------------------------------------------------------------------+
'
'   ElementNameExists - checks the existing strIndElementName array
'                       to see if the element name passed in already
'                       exists in the array.
'
'   author      BSI                                             08/2001
'
'-----------------------------------------------------------------------+
Public Function ElementNameExists(strElementName As String) As Boolean
    Dim i As Integer
    
    ' Loop thru possible element names found so far if any are found
    If intTotElementNames <> 0 Then
        
        For i = 1 To UBound(strIndElementName)
            
            ' If element type already exists, return True
            If strIndElementName(i) = strElementName Then
                ElementNameExists = True
                Exit Function
            End If
        
        Next i
    
    End If

    'Element name does not exist, return False
    ElementNameExists = False

End Function

'-----------------------------------------------------------------------+
'
'   ElementNameGetPosition - returns the index position of the element
'                            name passed in.
'
'   author      BSI                                             08/2001
'
'-----------------------------------------------------------------------+
Public Function ElementNameGetPosition(strElementName As String) As Integer
    Dim i As Integer
    For i = 1 To UBound(strIndElementName)
        If strIndElementName(i) = strElementName Then
            Exit For
        End If
    Next i
    ElementNameGetPosition = i
End Function

'-----------------------------------------------------------------------+
'
'   ComboBoxPopulate - populates the combo box with all possible
'                      element types.
'
'   author      BSI                                             08/2001
'
'-----------------------------------------------------------------------+
Public Sub ComboBoxPopulate()
    Dim i As Integer
    For i = 1 To UBound(strIndElementName)
        cboElementName.AddItem strIndElementName(i)
    Next i
End Sub

'-----------------------------------------------------------------------+
'
'   elemNameFromNumber
'
'   author      BSI                                             03/2001
'
'-----------------------------------------------------------------------+
Public Function elemNameFromNumber(ByVal eType As Integer) As String
     
    Select Case eType
    Case 1: elemNameFromNumber = "Cell Library Header"
    Case 2: elemNameFromNumber = "Cell Header"
    Case 3: elemNameFromNumber = "Line"
    Case 4: elemNameFromNumber = "Line String"
    Case 5: elemNameFromNumber = "Group Data"
    Case 6: elemNameFromNumber = "Shape"
    Case 7: elemNameFromNumber = "Text Node"
    Case 8: elemNameFromNumber = "Digitizer Setup Data"
    Case 9: elemNameFromNumber = "Design File Header"
    Case 10: elemNameFromNumber = "Level Symbology"
    Case 11: elemNameFromNumber = "Curve"
    Case 12: elemNameFromNumber = "Complex String"
    Case 13: elemNameFromNumber = "Conic"
    Case 14: elemNameFromNumber = "Complex Shape"
    Case 15: elemNameFromNumber = "Ellipse"
    Case 16: elemNameFromNumber = "Arc"
    Case 17: elemNameFromNumber = "Text"
    Case 18: elemNameFromNumber = "Surface"
    Case 19: elemNameFromNumber = "Solid"
    Case 21: elemNameFromNumber = "BSpline Pole"
    Case 22: elemNameFromNumber = "Point String"
    Case 23: elemNameFromNumber = "Cone"
    Case 24: elemNameFromNumber = "BSpline Surface"
    Case 25: elemNameFromNumber = "BSpline Boundary"
    Case 26: elemNameFromNumber = "BSpline Knot"
    Case 27: elemNameFromNumber = "BSplin3 Curve"
    Case 28: elemNameFromNumber = "BSpline Weight"
    Case 33: elemNameFromNumber = "Dimension"
    Case 34: elemNameFromNumber = "Shared Cell Definition"
    Case 35: elemNameFromNumber = "Shared Cell"
    Case 36: elemNameFromNumber = "MultiLine"
    Case 37: elemNameFromNumber = "Tag"
    Case 38: elemNameFromNumber = "DgnStore Component"
    Case 39: elemNameFromNumber = "DgnStore Header"
    Case 51: elemNameFromNumber = "ARCH (Intergraph)"
    Case 54: elemNameFromNumber = "SDS (Intergraph)"
    Case 56: elemNameFromNumber = "LEDS, WMS"
    Case 60: elemNameFromNumber = "SIS"
    Case 61: elemNameFromNumber = "EWS SCREMANC"
    Case 62: elemNameFromNumber = "EWS PANEL"
    Case 63: elemNameFromNumber = "EWS PDS"
    Case 66: elemNameFromNumber = "MicroStation Application"
    Case 87: elemNameFromNumber = "Raster Header"
    Case 88: elemNameFromNumber = "Raster Component"
    Case 89: elemNameFromNumber = "Raster Data"
    Case 90: elemNameFromNumber = "Raster Reference Attachment"
    Case 91: elemNameFromNumber = "Raster Reference Component"
    Case 96: elemNameFromNumber = "Level Table"
    Case 97: elemNameFromNumber = "View Group Element"
    Case 98: elemNameFromNumber = "View Element"
    Case 99: elemNameFromNumber = "Level Mask Element"
    Case 100: elemNameFromNumber = "Reference Attach Element"
    Case 101: elemNameFromNumber = "Matrix Header"
    Case 102: elemNameFromNumber = "Matrix Int Data"
    Case 103: elemNameFromNumber = "Matrix Double Data"
    Case 105: elemNameFromNumber = "Mesh Header"
    Case Else: elemNameFromNumber = "Unknown Element Type"
    
    End Select
    
End Function


