Attribute VB_Name = "Module1"
Option Explicit

'/*----------------------------------------------------------------------+
'|                                                                       |
'| Copyright 1993-2003, Bentley Systems, Incorporated                    |
'| All rights reserved.                                                  |
'|                                                                       |
'| Limited permission is hereby granted to reproduce and modify this     |
'| copyrighted material provided that the resulting code is used only in |
'| conjunction with Bentley Systems products under the terms of the      |
'| license agreement provided therein, and that this notice is retained  |
'| in its entirety in any such reproduction or modification.             |
'|                                                                       |
'+----------------------------------------------------------------------*/

' This VBA example will change the Resolution field in the Settings->Design-
' File..->Working Units->Advanced..->Advanced Unit Settings dialog box.
' It takes 2 arguments, the Model name and Resolution value. To run it, type
' in the keyin window:

' vba run [ResolutionSet]Module1.StartApp <ModelName> <ResolutionValue>

' This example is not a recommended practice. In V8, the correct usage of
' Models should be used instead since each Model can have it's own
' independent working units setting.

'-----------------------------------------------------------------------+
'
'   name        ResolutionSet
'
'   Author      BSI                                         02/2003
'
'-----------------------------------------------------------------------+
Sub ResolutionSet(parseArray() As String)

    Dim oModels         As ModelReferences
    Dim oModel          As ModelReference
    Dim counter         As Integer
    Dim isModelFound    As Boolean
    
    Set oModels = ActiveDesignFile.models
    
    ' Cycles through all the Models
    For counter = 0 To oModels.count - 1
        Set oModel = oModels.Item(counter + 1)
        
        ' Determines if the Model exists in the DGN using the 1st argument
        If StrComp(oModel.Name, parseArray(0), vbTextCompare) = False Then
        
            ' If the Model is locked, its unlocked
            If oModel.IsLocked Then
                oModel.IsLocked = False
            End If
            
            isModelFound = True
            
' If the 2nd argument is not numeric, the app is exited
On Error GoTo errorHandler

            ' Sets the Resolution as per the 2nd argument
            oModel.UORsPerStorageUnit = parseArray(1)
            CadInputQueue.SendCommand "filedesign"
        End If
    Next counter
    
    ' Displays a message and exits the app if the Model is found or not using
    ' the 2nd argument
    If isModelFound = False Then
        ShowMessage "The Model: '" & parseArray(0) & _
                    "' does not exist in this DGN. App exited."
                    
        CommandState.StartDefaultCommand
        Exit Sub
    Else
        ShowMessage "The Resolution has been successfully changed to: " & _
                    parseArray(1) & " for the Model: '" & parseArray(0) & _
                    "'. App exited."
                    
        CommandState.StartDefaultCommand
        Exit Sub
    End If

errorHandler:
    ShowMessage parseArray(1) & _
                    ": is an invalid value for the Resolution. App exited."
                    
    CommandState.StartDefaultCommand
End Sub

'-----------------------------------------------------------------------+
'
'   name        ParseArguments
'
'   Author      BSI                                         02/2003
'
'-----------------------------------------------------------------------+
Public Sub ParseArguments(fileName As String)
    Dim parseArray() As String
    Dim count   As Long
    Dim i       As Long
    
    ' Places each argument into an array
    parseArray = Split(fileName, " ", -1, vbTextCompare)

    ' Counts the number of arguments
    For count = 1 To UBound(parseArray)
    Next
    
    ' Processes the arguments. If there are more then or less then 2, the
    ' app will exit
    Select Case count
        Case Is < 2
            ShowMessage "2 arguments are needed, Model Name & Resolution. App exited."
            CommandState.StartDefaultCommand
            Exit Sub
            
        Case Is > 2
            ShowMessage "Only 2 arguments are needed, Model Name & Resolution. App exited."
            CommandState.StartDefaultCommand
            Exit Sub
            
        Case Else
           ResolutionSet parseArray
    End Select
    
End Sub

'-----------------------------------------------------------------------+
'
'   name        StartApp
'
'   Author      BSI                                         02/2003
'
'-----------------------------------------------------------------------+
Sub StartApp()

    Dim getArguments As String
    
    ShowMessage ""
    
    ' Gets the argument from the key-in window
    getArguments = KeyinArguments
    
    ' Passes the arguments for processing
    ParseArguments getArguments

End Sub

