Attribute VB_Name = "Start"
'----------------------------------------------------------------------+
' Module:   Solid2Smart.bas
'
' Status: WORKING
'
' Date:  11/28/03
'
' This example illustrates how to convert solids to SmartSolids.
'
' Author: BSI-DEVSPT
'
' Copyright: (c) 2003 Bentley Systems, Incorporated. All rights reserved.
'
'-----------------------------------------------------------------------+
Option Explicit

'----------------------------------------------------------------------+
'
'   Declarations
'
'----------------------------------------------------------------------+
Declare Function mdlKISolid_elementToBody Lib "stdkisolid.dll" (ByRef bodyPP As Long, ByVal edP As Long, ByVal modelRef As Long) As Long
Declare Function mdlKISolid_bodyToElement Lib "stdkisolid.dll" (ByRef edPP As Long, ByVal bodyP As Long, ByVal wireframe As Long, ByVal nIsoparametrics As Long, ByVal templateP As Long, ByVal modelRef As Long) As Long
Declare Sub mdlKISolid_freeBody Lib "stdkisolid.dll" (ByVal bodyP As Long)
Declare Sub mdlElmdscr_freeAll Lib "stdmdlbltin.dll" (ByRef elemDescr As Long)
Declare Function mdlElmdscr_rewrite Lib "stdmdlbltin.dll" (ByVal newElemDscr As Long, ByVal oldElemDscr As Long, ByVal filePos As Long) As Long
Declare Function ElmdscrAccessor_getMSElement Lib "stdmdlaccessor.dll" (ByVal ElementDescr As Long) As Long
Declare Function mdlCurrTrans_begin Lib "stdmdlbltin.dll" () As Long
Declare Function mdlCurrTrans_end Lib "stdmdlbltin.dll" () As Long
Declare Function mdlElmdscr_read Lib "stdmdlbltin.dll" (ByRef elemDescr As Long, ByVal filePos As Long, ByVal modelRef As Long, ByVal expandSharedCells As Long, ByRef startFilePos As Long) As Long

'----------------------------------------------------------------------+
'
'   Constants
'
'----------------------------------------------------------------------+
Const SUCCESS = 0
Const ERROR = -1

'----------------------------------------------------------------------+
'
'   Name        ConvertType19ToSmartSolid
'
'   Author      BSI-DEVSPT                                  11/2003
'
'----------------------------------------------------------------------+
'
'   Description:    Converts a solid (type 19) or surface (type 18)
'                   to a smartSolid or smartsurface
'
'   Version:        1.0
'
'   Returns:        Returns the SUCCESS if converted otherwise ERROR
'
'   Remarks:        Requires the appropriate declarations for the MDL
'                   functions.
'
'   Example:        status = ConvertType19ToSmartSolid (solidElm)
'
'----------------------------------------------------------------------+
Function ConvertType19ToSmartSolid(ByRef solidType19 As Element) As Long

    Dim bodyP As Long
    Dim newSolidElmP As Long
    Dim type19ElmdscrP As Long
    Dim msElmp As Long
    Dim startFP As Long
    Const NODE_ID = 1

    'Read in the element descriptor
    If (mdlElmdscr_read(type19ElmdscrP, solidType19.FilePosition, solidType19.ModelReference.MdlModelRefP, True, startFP) > 0) Then
        ConvertType19ToSmartSolid = ERROR
    End If
    
    'Convert element to a Parasolid body
    If (mdlKISolid_elementToBody(bodyP, type19ElmdscrP, solidType19.ModelReference.MdlModelRefP)) = SUCCESS Then

            'Get the MSElement pointer
            msElmp = ElmdscrAccessor_getMSElement(type19ElmdscrP)

            'Convert body back to an Element
            If (mdlKISolid_bodyToElement(newSolidElmP, bodyP, True, -1, msElmp, solidType19.ModelReference.MdlModelRefP)) = SUCCESS Then

                mdlElmdscr_rewrite newSolidElmP, type19ElmdscrP, startFP

                ' Free memory
                mdlKISolid_freeBody bodyP
                mdlElmdscr_freeAll newSolidElmP
                ConvertType19ToSmartSolid = SUCCESS
            Else
                ConvertType19ToSmartSolid = ERROR
            End If
    Else
        ConvertType19ToSmartSolid = ERROR
    End If

End Function

'-----------------------------------------------------------------------+
'
'   name        ScanFile
'
'   author      BSI-DEVSPT                                  10/2001
'
'-----------------------------------------------------------------------+
Private Sub ScanFile()

    Dim oScanCriteria           As New ElementScanCriteria
    Dim oEnumerator             As ElementEnumerator
    Dim tdStart, tdEnd          As Date
    Dim count                   As Integer
    Dim sHelp                   As String

    ' Clear out any previous scans
    oScanCriteria.Reset
    oScanCriteria.ExcludeAllTypes
    oScanCriteria.IncludeType (msdElementTypeSolid)
    oScanCriteria.IncludeType (msdElementTypeSurface)

    ' Start timer
    tdStart = Now

    ' Set enumerator
    Set oEnumerator = ActiveModelReference.Scan(oScanCriteria)

    'Push the top-of-stack transformation on the Current Transform stack.
    mdlCurrTrans_end

    ' Step through every element in the file
    Do While oEnumerator.MoveNext
        With oEnumerator.Current
            Dim cellElm As Element

            count = count + 1
            Set cellElm = oEnumerator.Current

            'Convert the solid element to a smartSolid element
            ConvertType19ToSmartSolid oEnumerator.Current
            
        End With
    Loop

    'Remove the top-of-stack transformation from the Current Transform stack.
    mdlCurrTrans_begin

    ' Report results
    tdEnd = Now
    sHelp = DateDiff("s", tdStart, tdEnd)
    sHelp = ActiveModelReference.DesignFile.FullName + ", " + "Processed " + CStr(count) + " elements [" + sHelp + " seconds]"
    ShowMessage "Smart2Solid Information", sHelp, msdMessageCenterPriorityInfo

End Sub

'----------------------------------------------------------------------+
'
'   Name        main
'
'   Author      BSI - Developer Support                 11/2003
'
'   Description - Main entry point for the example.
'
'----------------------------------------------------------------------+
Sub Main()

    ' Convert Type 19 Solid elements to Type 2 Smart Solid
    ScanFile

End Sub


