Option Explicit
'----------------------------------------------------------------------+
' Module:   SolidCentroid.bas
'
' Status: WORKING
'
' Date:  11/14/03
'
' This example illustrates how get the centroid of a Smart Solid.
'
' Author: BSI-DEVSPT
'
' Copyright: (c) 2003 Bentley Systems, Incorporated. All rights reserved.
'
'-----------------------------------------------------------------------+

'----------------------------------------------------------------------+
'
'   Declarations
'
'----------------------------------------------------------------------+
Private Declare Function mdlKISolid_elementToBody Lib "stdkisolid.dll" (ByRef bodyPP As Long, ByVal edP As Long, ByVal modelRef As Long) As Long
Private Declare Function mdlKISolid_bodyMassProperties Lib "stdkisolid.dll" (ByRef pVolumeOut As Double, ByRef pPeripheryOut As Double, ByRef pCentroidOut As Point3d, ByRef inertiaOut As Long, ByVal pBodyIn As Long, ByRef transformP As Transform3d, ByVal requiredToleranceIn As Double) As Long
Private Declare Sub mdlKISolid_freeBody Lib "stdkisolid.dll" (ByVal bodyP As Long)
Private Declare Sub mdlKISolid_beginCurrTrans Lib "stdkisolid.dll" (ByVal modelRef As Long)
Private Declare Sub mdlKISolid_endCurrTrans Lib "stdkisolid.dll" ()
Private Declare Sub mdlCurrTrans_invScaleDoubleArray Lib "stdmdlbltin.dll" (ByRef out As Double, ByRef in_ As Double, ByVal numValues As Long)
Private Declare Sub mdlCurrTrans_scaleDoubleArray Lib "stdmdlbltin.dll" (ByRef out As Double, ByRef in_ As Double, ByVal numValues As Long)
Private Declare Function mdlCurrTrans_begin Lib "stdmdlbltin.dll" () As Long
Private Declare Function mdlCurrTrans_masterUnitsIdentity Lib "stdmdlbltin.dll" (ByVal translateToGlobalOrigin As Long) As Long
Private Declare Sub mdlVec_scale Lib "stdmdlbltin.dll" (ByRef outVectorP As Point3d, ByRef inVectorP As Point3d, ByVal inScale As Double)
Private Declare Function mdlCnv_UORToMaster Lib "stdmdlbltin.dll" (ByRef masterUnitsP As Double, ByVal UORs As Double, ByVal modelRef As Long) As Long
Private Declare Function mdlCurrTrans_end Lib "stdmdlbltin.dll" () As Long

Const SUCCESS  = 0

'----------------------------------------------------------------------+
'
'   Name        SolidBodyGetScale
'
'   Author      BSI - Developer Support                 11/2003
'
'----------------------------------------------------------------------+
'
'   Description:    Returns the Parasolid scale
'
'   Version:        1.0
'
'   Dependencies:
'
'   Returns:        The active solid scale.
'
'   Remarks:
'
'   Example:
'
'   pScale = SolidBodyGetScale ()
'
'----------------------------------------------------------------------+
Private Function SolidBodyGetScale() As Double

        Dim bodyScale As Double

        bodyScale = 1#

        mdlKISolid_beginCurrTrans ActiveModelReference.MdlModelRefP
        mdlCurrTrans_scaleDoubleArray bodyScale, bodyScale, 1
        mdlKISolid_endCurrTrans

       SolidBodyGetScale = bodyScale

End Function

'----------------------------------------------------------------------+
'
'   Name        SolidPointToMaterUnits
'
'   Author      BSI - Developer Support                 11/2003
'
'----------------------------------------------------------------------+
'
'   Description:    Returns a Point3d in Master File Untis from the given
'                   Parasolid point.
'
'   Version:        1.0
'
'   Dependencies:
'
'   Returns:        Point3d in Master units
'
'   Remarks:
'
'   Example:
'
'   ustnPoint = SolidPointToMaterUnits(pPoint)
'
'----------------------------------------------------------------------+
Function SolidPointToMaterUnits(SolidPoint As Point3d) As Point3d

    Dim pSolidScale As Double
    Dim uorP As Point3d
    Dim numMembers As Long

    numMembers = 3

'   Get the body's scale
    pSolidScale = SolidBodyGetScale

'   Convert Parasolid units to UORs
    mdlVec_scale uorP, SolidPoint, pSolidScale

'   Convert to Master units
    mdlCnv_UORToMaster uorP.X, uorP.X, ActiveModelReference.MdlModelRefP
    mdlCnv_UORToMaster uorP.Y, uorP.Y, ActiveModelReference.MdlModelRefP
    mdlCnv_UORToMaster uorP.Z, uorP.Z, ActiveModelReference.MdlModelRefP

    SolidPointToMaterUnits = uorP

End Function

'----------------------------------------------------------------------+
'
'   Name        main
'
'   Author      BSI - Developer Support                 11/2003
'
'   Description - Main entry point for the example.
'
'----------------------------------------------------------------------+
Sub main()

    Dim oScanCriteria As ElementScanCriteria
    Dim oScanEnumerator As ElementEnumerator
    Dim oElement As Element
    Dim solidCentroid As Point3d
    Dim masterCentroid As Point3d
    Dim solidVolume As Double
    Dim pPeripheryOut As Double
    Dim tmatrix As Transform3d
    Dim edP As Long
    Dim bodyP As Long
    Dim inertiaOut As Long
    Dim stat As Long

'   Set the scan criteria to look for cells only.
'   Note Smart Solids are cell elements that contains
'   the Parasolid information
    Set oScanCriteria = New ElementScanCriteria
    oScanCriteria.IncludeOnlyCell "Smart Solid"

'   Scan the model
    Set oScanEnumerator = ActiveModelReference.Scan(oScanCriteria)

    Do While oScanEnumerator.MoveNext
        Set oElement = oScanEnumerator.Current

        If oElement.AsCellElement.Name = "Smart Solid" Then

'           Get the pointer to the MSElementDescr
            edP = oElement.MdlElementDescrP

'            Get the identity matrix
            tmatrix = Transform3dIdentity

'           Convert the element to a body
            stat = mdlKISolid_elementToBody(bodyP, edP, ActiveModelReference.MdlModelRefP)

'           Extact the centroid and the volume information
            stat = mdlKISolid_bodyMassProperties(solidVolume, pPeripheryOut, solidCentroid, inertiaOut, bodyP, tmatrix, 0)

'           Display the Centroid point in Parasolid untis
            MsgBox "Parasolid Units = " + vbNewLine + _
                   "x: " + CStr(solidCentroid.X) + vbNewLine + _
                   "y: " + CStr(solidCentroid.Y) + vbNewLine + _
                   "z: " + CStr(solidCentroid.Z) + vbNewLine

            If stat = SUCCESS Then
               Dim CentroidElm As LineElement

'                Convert the Parasolid units to Master units
                 masterCentroid = SolidPointToMaterUnits(solidCentroid)

'               Display the Centroid in Master units
                MsgBox "Master Units = " + vbNewLine + _
                       "x: " + CStr(masterCentroid.X) + vbNewLine + _
                       "y: " + CStr(masterCentroid.Y) + vbNewLine + _
                       "z: " + CStr(masterCentroid.Z) + vbNewLine

'               Create a Point at the Centroid of the solid
                Set CentroidElm = CreateLineElement2(Nothing, masterCentroid, masterCentroid)
                CentroidElm.LineWeight = 10
                ActiveModelReference.AddElement CentroidElm
                CentroidElm.Redraw
            End If
        End If
    Loop
End Sub
