/*----------------------------------------------------------------------+
| 
| Copyright (1993) Bentley Systems, Inc., All rights reserved.
| 
| "MicroStation", "MDL", and "MicroCSL" are trademarks of Bentley
|  Systems, Inc. and/or Intergraph Corporation.
|         
| Limited permission is hereby granted to reproduce and modify this 
| copyrighted material provided that the resulting code is used only in
| conjunction with Bentley Systems products under the terms of the 
| license agreement provided therein, and that this notice is retained 
| in its entirety in any such reproduction or modification.  
|  
+----------------------------------------------------------------------*/
/*----------------------------------------------------------------------+
| 
|   solid.mc - solid source code.
|  
|   This example illustrates how to create a MSL library
|  
+----------------------------------------------------------------------*/

/*----------------------------------------------------------------------+
|                                                                       |
|   Include Files                                                       |
|                                                                       |
+----------------------------------------------------------------------*/
#include <mdl.h>
#include <mskisolid.h>

#include <mssystem.fdf>
#include <mselemen.fdf>
#include <mselmdsc.fdf>
#include <mssystem.fdf>
#include <msoutput.fdf>
#include <mskisolid.fdf>
#include <msmisc.fdf>
#include <mscnv.fdf>
#include <mscurrtr.fdf>
#include <mscexpr.fdf>
#include <mstmatrx.fdf>
#include <msrmatrx.fdf>

/*----------------------------------------------------------------------+
|
|    createPTick 
|
|    author             BSI                             7/2002
|
+----------------------------------------------------------------------*/
Public int    createPTick
(
MSElementDescr  **edP,  // <=> Tick Mark element
double          xSize,  // => x distance
double          ySize,  // => y distance
double          zSize   // => z distance
)

    {
    MSElementDescr  *cubeP = NULL;
    KIBODY          *bodyP = NULL;
    double          xyz[3];
  
    // Make a copy
    xyz[0]= xSize;
    xyz[1]= ySize;
    xyz[2]= zSize;



    /* Convert master units to UORs */
    mdlCnv_masterToUOR (&xyz[0], xyz[0], MASTERFILE);
    mdlCnv_masterToUOR (&xyz[1], xyz[1], MASTERFILE);
    mdlCnv_masterToUOR (&xyz[2], xyz[2], MASTERFILE);

    /* Begin current translation */
    mdlKISolid_beginCurrTrans (mdlModelRef_getActive());
    /* Convert current units to Parasolid units */
    mdlCurrTrans_invScaleDoubleArray(xyz,xyz,3);
    /* End current translation */
    mdlKISolid_endCurrTrans ();

    /* Make our tick mark */
    mdlKISolid_makeCuboid (&bodyP, xyz[0],xyz[1],xyz[2]);
    mdlKISolid_makeCuboid (&cubeP, xyz[1],xyz[0],xyz[2]);
    mdlKISolid_unite (bodyP, cubeP);
    

    /* Convert the body to an element */
    mdlKISolid_bodyToElement (edP, bodyP, TRUE, -1, NULL, MASTERFILE);

    /* Free memory */
    mdlKISolid_freeBody (bodyP);
    mdlKISolid_freeBody (cubeP);

    return SUCCESS;
    }

/*----------------------------------------------------------------------+
|
|    createPTick 
|
|    author             BSI                             7/2002
|
+----------------------------------------------------------------------*/
Public int    createPTick2
(
MSElementDescr  **edP,  // <=> Tick Mark element
double          xSize,  // => x distance
double          ySize,  // => y distance
double          zSize   // => z distance
)

    {
    MSElementDescr  *cubeP = NULL;
    KIBODY          *bodyP = NULL;
    double          xyz[3];
  
    // Make a copy
    xyz[0]= xSize;
    xyz[1]= ySize;
    xyz[2]= zSize;


    /* Begin current translation */
    mdlKISolid_beginCurrTrans (mdlModelRef_getActive());

    /* Convert master units to UORs */
    mdlCnv_masterToUOR (&xyz[0], xyz[0], MASTERFILE);
    mdlCnv_masterToUOR (&xyz[1], xyz[1], MASTERFILE);
    mdlCnv_masterToUOR (&xyz[2], xyz[2], MASTERFILE);

    /* Convert current units to Parasolid units */
    mdlCurrTrans_invScaleDoubleArray(xyz,xyz,3);

    /* Make our tick mark */
    mdlKISolid_makeCuboid (&bodyP, xyz[0],xyz[1],xyz[2]);
    mdlKISolid_makeCuboid (&cubeP, xyz[1],xyz[0],xyz[2]);
    mdlKISolid_unite (bodyP, cubeP);
    
    /* End current translation */
    mdlKISolid_endCurrTrans ();

    /* Convert the body to an element */
    mdlKISolid_bodyToElement (edP, bodyP, TRUE, -1, NULL, MASTERFILE);

    /* Free memory */
    mdlKISolid_freeBody (bodyP);
    mdlKISolid_freeBody (cubeP);

    return SUCCESS;
    }

/*----------------------------------------------------------------------+
|                                   
| name      initialize              
|                                   
| author    BSI                                         04/2001
|                                   
+----------------------------------------------------------------------*/
Public  void initialize
(
void
)
    {
    char        *setP;      /* a ptr to a "C expression symbol set" */
    
    // Publish the function tagDisplay.
    setP = mdlCExpression_initializeSet (VISIBILITY_DIALOG_BOX|VISIBILITY_DEBUGGER|VISIBILITY_CALCULATOR|VISIBILITY_LOCAL, 0, TRUE);
    mdlCExpression_symbolPublish  (setP, "createPTick", SYMBOL_CLASS_FUNCTION, (void *)&longType,
                                  createPTick);   


    }

/*----------------------------------------------------------------------+
|                                   
|   name        main                                
|
|   author      BSI                                         12/1998
|                                   
+----------------------------------------------------------------------*/
Private int main 
(
int     argc,   /* => Number of arguments passed in argv */
char    *argv[] /* => Array of pointers to arguments */
)   
    {
    // Initialize
    initialize();

    return SUCCESS;
    }
      