/*-----------------------------------------------------------------------+
|                                                                       |
| Copyright 1993-2001, Bentley Systems, Incorporated                    |
| All rights reserved.                                                  |
|                                                                       |
| Limited permission is hereby granted to reproduce and modify this     |
| copyrighted material provided that the resulting code is used only in |
| conjunction with Bentley Systems products under the terms of the      |
| license agreement provided therein, and that this notice is retained  |
| in its entirety in any such reproduction or modification.             |
|                                                                       |
|----------------------------------------------------------------------*/
/*----------------------------------------------------------------------+
| 
|   myAPI.mc - myAPI source code.
|  
|   This example illustrates how to create a MSL library
|  
+----------------------------------------------------------------------*/
/*----------------------------------------------------------------------+
|                                                                       |
|   Include Files                                                       |
|                                                                       |
+----------------------------------------------------------------------*/
#include <mdl.h>
#include <tcb.h>
#include <stdio.h>
#include <stdlib.h>

#include <mssystem.fdf>
#include <mselemen.fdf>
#include <mselmdsc.fdf>
#include <mssystem.fdf>
#include <msoutput.fdf>
#include <mstagdat.fdf> /* Tag data function prototypes           */
#include <mscexpr.fdf>
#include <ditemlib.fdf>

#include "myAPI.fdf"

/*----------------------------------------------------------------------+
|              
| name          tagDisplay
|                                  
| author    BSI                                         07/2002
|                                
+----------------------------------------------------------------------*/
Public int tagDisplay   
(
MSElementDescr *tagElmDescrP,   // <=> Tag element to change display.
int             state           // => On OR Off, TRUE = On, FLASE = Off, -1 = Current State
)
    {
    UShort      tagProps = 0;
    int         status = SUCCESS;
    TagValue    value;
    boolean     displayable, createTag;

    createTag = FALSE;


    if (!tagElmDescrP)
        return ERROR;

    /*
    //  Extract the tag specification, tag value, and displayable flag. 
    //  I have extracted the tag value because I want to check the data
    //  type of the tag to determine whether or not it should be 
    //  displayed.  For example, I don't want to display binary data.
    //
    //  If I did not extract the value here, I could have also called
    //  mdlTag_getTagDef() to get the tag's type from the tag definition.
    */
    if (SUCCESS != mdlTag_extract (
                                   NULL,                /* <= origin of associated element */
                                   NULL,                /* <= snap point */
                                   NULL,                /* <= tag specification */
                                   &displayable,        /* <= TRUE for displayable tag */
                                   &value,              /* <= value for the tag */
                                   NULL,                /* <= target element's assocation id */
                                   NULL,                /* <= offset */
                                   NULL,                /* <= size for the text */
                                   NULL,                /* <= rotation matrix */
                                   NULL,                /* <= text parameters */
                                   &tagElmDescrP->el,   /* => tag element */
                                   MASTERFILE))         /* => file number for the tag */
        {
        return ERROR;
        }

    if (state == -1)
        status = displayable;
    else
        {
        /*  
        //  Note:
        //  The only tag property that is stored with the tag element is
        //  the display flag.  Therefore, there is no need to retrieve the
        //  existing property mask for the tag definition using
        //  mdlTag_getTagDef() before modifying the tag properties.
        //  When I recreate the tag using mdlTag_create(), I can simply
        //  pass the display flag for the property parameter.
        */

        //  If the tag is NOT currently displayed and the user
        //  wants to turn ON the display then (display toggle is ON) 
        if (state == TRUE)
            /* Turn ON the display of the flag */
            tagProps = ~TAG_PROP_DISPOFF;

        //  Else if the tag is currently displayed and the user wants to
        //  turn the display OFF then (display toggle is OFF)
        else
            /* Turn OFF the display of this tag */
            tagProps = TAG_PROP_DISPOFF;
            
        createTag = TRUE;

        /*******************************************************************
        //
        //  When the value parameter is returned from mdlTag_extract(),
        //  MicroStation has allocated memory for the stringVal and binaryVal
        //  members of the TagValue structure.  You are responsible for
        //  freeing this memory.
        //
        ********************************************************************/
        switch (value.type)
            {
            case MS_TAGTYPE_CHAR:
                {
                /* Free the memory allocated by mdlTag_extract() */
                free(value.val.stringVal);

                break;
                }
                case MS_TAGTYPE_BINARY:
                {

                /* Free the memory allocated by mdlTag_extract() */
                free(value.val.binaryVal);

                /*
                //  Do not change the display flag for the binary type since
                //  the data is not readable anyway 
                */
                createTag = FALSE;

                break;
                }

            default:
                break;
            }


        if (TRUE == createTag)
            {
            status = mdlTag_create(
                                    &tagElmDescrP->el,     /* output area for new tag element */
                                    &tagElmDescrP->el,  /* template element */
                                    NULL,           /* specification for tag definition */
                                    &tagProps,      /* display flag */
                                    NULL,           /* value extracted from text element */
                                    NULL,           /* tag id from mdlAssoc_tagElement */
                                    NULL,           /* computed location offset */
                                    NULL,           /* rest are special text properties */
                                    NULL,           /* ...                  */
                                    NULL,
                                    NULL
                                );
            if (SUCCESS != status)
                {
                mdlOutput_error ("Error Creating Tag");
                }
            else
                {

                /*******************************************************************
                //  The file position of the tag element is contained in the
                //  userData1 member in the element descriptor header.
                //
                //  For example:
                //
                //  MSElementDescr *tagElmDescrP;
                //
                //  tagElmDescrP->h.userData1  contains the file position of the tag
                //
                //  This file position is needed since I intend to modify the tag 
                //  element and rewrite the element to the design file at the file
                //  position contained in tagElmDescrP->h.userData1
                //
                //********************************************************************/
                mdlElement_display(&tagElmDescrP->el, ERASE);
                mdlElement_display(&tagElmDescrP->el, NORMALDRAW);
                }

            } /* if (TRUE == createTag) */
         }
    
    return status;
    }

/*----------------------------------------------------------------------+
|                                   
| name      initialize              
|                                   
| author    BSI                                         04/2001
|                                   
+----------------------------------------------------------------------*/
Public  void initialize
(
void
)
    {
    char        *setP;      /* a ptr to a "C expression symbol set" */
    
    // Publish the function tagDisplay.
    setP = mdlCExpression_initializeSet (VISIBILITY_DIALOG_BOX|VISIBILITY_DEBUGGER|VISIBILITY_CALCULATOR|VISIBILITY_LOCAL, 0, TRUE);
    mdlCExpression_symbolPublish  (setP, "tagDisplay", SYMBOL_CLASS_FUNCTION, (void *)&longType,
                                  tagDisplay);   


    ;
    }

/*----------------------------------------------------------------------+
|                                   
|   name        main                                
|
|   author      BSI                                         04/2001
|                                   
+----------------------------------------------------------------------*/
Private int main 
(
int     argc,   /* => Number of arguments passed in argv */
char    *argv[] /* => Array of pointers to arguments */
)   
        {
        // Initialize
        initialize();

        /* Open for use with the Resource Manager */
        return SUCCESS;
        }
