Attribute VB_Name = "FindNReplace"
Option Explicit
'----------------------------------------------------------------------+
' replaceTagText
'
' Status: WORKING
'
' Date: 05/22/03
'
' This example illustrates how to scan all tags for a particular
' string value and replace with the new value
'
' Usage:
' Keyin: vba run [TagFindNReplace]main Working,1H,1 1/2
'
' Author: BSI-DEVSPT
'
'-----------------------------------------------------------------------+

'-----------------------------------------------------------------------+
'
'   Public variables
'
'-----------------------------------------------------------------------+
Public oTagEl() As TagElement
Public count As Long

'-----------------------------------------------------------------------+
'
'   name        VersionGetNumber
'
'   author      BSI-DEVSPT                                       02/2003
'
'-----------------------------------------------------------------------+
Function VersionGetNumber() As Long
    
    Dim partialString   As String
    Dim wholeString     As String
    Dim strVersion      As String
    Dim length          As Long
    Dim inStringLoc     As Long
    
    ' Version property return a string that contains the active V8s version
    ' number. For example, "Version 08.00.04.01 Windows x86"
    wholeString = MicroStationDGN.Version
    
    ' Gets the number of characters in the string
    length = Len(wholeString)
    
    ' Determines how far in from the left of the string where the version
    ' value is
    inStringLoc = InStr(wholeString, "0")
    
    ' Only process if inStringLoc at least contains a 1
    If inStringLoc > 0 Then
        strVersion = Mid(wholeString, inStringLoc, 11)
        
        ' Use the Mid function to only extract the numbers of the version, not
        ' the periods ("."s)
        partialString = Mid(strVersion, 1, 2)
        partialString = partialString + Mid(strVersion, 4, 2)
        partialString = partialString + Mid(strVersion, 7, 2)
        partialString = partialString + Mid(strVersion, 10, 2)
        
        ' Converts the String value to a Long
        VersionGetNumber = Val(partialString)
    Else
        ShowError "Invalid version number"
    End If

End Function

'-----------------------------------------------------------------------+
'
'   name        locateTagText
'
'   author      BSI-DEVSPT                                       10/2002
'
'-----------------------------------------------------------------------+
Sub locateTagText(oTagEl() As TagElement, count As Long, fontName As String, _
                 searchTxt As String, newString As String)
    
    ' Compare strings
    If UCase(oTagEl(count).Value) = UCase(searchTxt) Then
                 
        'Set the active textstyle's fractions on
        ActiveSettings.TextStyle.IsFractions = True
        oTagEl(count).Font = ActiveDesignFile.Fonts(fontName)
        oTagEl(count).Value = newString
        oTagEl(count).Rewrite
        
    End If
     
End Sub

'-----------------------------------------------------------------------+
'
'   name        ScanForTags
'
'   author      BSI                                             10/2002
'
'-----------------------------------------------------------------------+
Sub ScanForTags(fontName As String, searchTxt As String, newString As String)

    Dim oScanCriteria As New ElementScanCriteria
    Dim oElEnum As ElementEnumerator
    Dim oEl As Element
    
    'Scan for visible elements only
    oScanCriteria.IncludeOnlyVisible
    Set oElEnum = ActiveModelReference.Scan(oScanCriteria)
    
    While oElEnum.MoveNext = True
        Set oEl = oElEnum.Current

        'Check to see if the element has a tags
        If oEl.HasAnyTags = True Then
            oTagEl() = oEl.GetTags
            For count = 0 To UBound(oTagEl)
                'Check to see if the tag value matches the search
                'string, then replace if necessary
                locateTagText oTagEl(), count, fontName, searchTxt, newString
            Next
        End If

    Wend
    
    RedrawAllViews
    
End Sub

'-----------------------------------------------------------------------+
'
'   name        main
'
'   author      BSI-DEVSPT                                       05/2003
'
'-----------------------------------------------------------------------+
Sub main()

    Dim keyinPos As Variant
    Dim args() As String
    Dim fontName As String
    Dim searchTxt As String
    Dim newString As String
    
    ' Ensure that we have a compatible version
    If VersionGetNumber <= 8000220 Then
        MsgBox "You must use a newer version of MicroStation than:" & _
        vbCr & MicroStationDGN.Version
        Exit Sub
    End If
    
    'Check for command line arguments
    If KeyinArguments = "" Then
        MsgBox "No command Line arguments!" + vbCr + _
                "Usage: Font,SearchString,NewString"
        End
    End If
    
   'Parse the command line
    args() = Split(KeyinArguments, ",", -1, 1)
    fontName = args(0)
    searchTxt = args(1)
    newString = args(2)
   
    'Search for text
    ScanForTags fontName, searchTxt, newString

End Sub
