Attribute VB_Name = "Module1"
Option Explicit

'-----------------------------------------------------------------------+
'                                                                       |
' Copyright 1993-2003, Bentley Systems, Incorporated                    |
' All rights reserved.                                                  |
'                                                                       |
' Limited permission is hereby granted to reproduce and modify this     |
' copyrighted material provided that the resulting code is used only in |
' conjunction with Bentley Systems products under the terms of the      |
' license agreement provided therein, and that this notice is retained  |
' in its entirety in any such reproduction or modification.             |
'                                                                       |
'----------------------------------------------------------------------*/

' This example illustrates the use of the BeforeRedraw event and
' OnDesignFileOpened events. When the BeforeRedraw event occurs, all text
' that have their Background attribute enabled will be cloned so that they
' display on top of the other elements. The OnDesignFileOpened event along
' with the OnProjectLoad() will keep the app loaded at all times as long as
' it's set to 'Auto-Load'. This example will also show how to create a new
' text style and set it.

'-----------------------------------------------------------------------+

' Public variable that defines an object for the OpenClose class module
Dim oOpenClose As OpenClose

' Public variable that defines an object for the Redraw class module
Dim oRedraw As Redraw

'-----------------------------------------------------------------------+
'
'   name        textStyleExists
'
'   author      BSI                                             04/2002
'
'-----------------------------------------------------------------------+
Function textStyleExists(sStyleName As String) As Boolean
    
    Dim oStyle As TextStyle

' If the text style does not exist textStyleExists will return False
On Error GoTo Failed
    Set oStyle = ActiveDesignFile.UserTextStyles(sStyleName)
    textStyleExists = True
    Exit Function
        
Failed:
End Function

'-----------------------------------------------------------------------+
'
'   name        TextStyleCreate
'
'   author      BSI                                             08/2003
'
'-----------------------------------------------------------------------+
Sub TextStyleCreate(TextStyleName As String)

    Dim oTxtStyles As TextStyles
    Dim oTxtStyle As TextStyle
    Dim BorderMargin As Point2d
    
    ' If the text stle exists 'Set' it, if it doesn't exist create it
    If textStyleExists(TextStyleName) = False Then
        Set oTxtStyle = ActiveDesignFile.TextStyles.Add(Nothing, TextStyleName)
    Else
        Set oTxtStyle = ActiveDesignFile.TextStyles(TextStyleName)
    End If
    
    ' Set the text style's font
    Set oTxtStyle.Font = ActiveDesignFile.Fonts("Engineering")
    
    ' Set the text style's text height and width
    oTxtStyle.Height = 0.1
    oTxtStyle.Width = 0.1
    
    ' Set the text style's line spacing
    oTxtStyle.NodeLineSpacing = 0.05
    
    ' Set the text style's color as ByLevel
    oTxtStyle.Color = ByLevelColor
    
    ' Set the text style's Background Border Color
    oTxtStyle.BorderColor = 255
    
    ' Set the text style's Background Fill Color
    oTxtStyle.BackgroundFillColor = 255
    
    ' Set the text style's Background Border X and Y size
    BorderMargin.X = 0.05
    BorderMargin.Y = 0.05
    oTxtStyle.BorderMargins = BorderMargin
    
    ' Set the text style's Background as enabled
    oTxtStyle.BorderAndBackgroundVisible = True
    
    ' Set active text style.
    ActiveSettings.TextStyle = oTxtStyle
    
    ' Unload the Text Style dialog box since the above settings will not
    ' be shown until it's reopened anyway
    CadInputQueue.SendCommand "mdl unload textstyle"

End Sub

'-----------------------------------------------------------------------+
'
'   name        reCreateElement
'
'   author      BSI                                             08/2003
'
'-----------------------------------------------------------------------+
Sub reCreateElement(oElement As Element)

    Dim CloneElem As Element

    ' Copies the current element
    Set CloneElem = oElement.Clone

    ' Creates the new element
    ActiveModelReference.AddElement CloneElem

    ' Removes the selected element
    ActiveModelReference.RemoveElement oElement

    ' Redraw the element
    CloneElem.Redraw msdDrawingModeNormal
    
End Sub

'-----------------------------------------------------------------------+
'
'   name        ModelScanForText
'
'   author      BSI                                             08/2003
'
'-----------------------------------------------------------------------+
Sub ModelScanForText()

    Dim oScanCriteria   As New ElementScanCriteria
    Dim oEnumerator     As ElementEnumerator
    Dim oElement        As Element

    ' Only search for text and text node elements
    oScanCriteria.ExcludeAllTypes
    oScanCriteria.IncludeType msdElementTypeText
    oScanCriteria.IncludeType msdElementTypeTextNode
    
    ' Setting an Undo Mark
    CadInputQueue.SendCommand "Mark"
    
    'Set enumerator
    Set oEnumerator = ActiveModelReference.Scan(oScanCriteria)
    
    'Step through the elements in the active model
    Do While oEnumerator.MoveNext
        Set oElement = oEnumerator.Current
      
        If oElement.Type = msdElementTypeText Then
        
            ' Do not modify this text element if it's text style's Background toggle
            ' is disabled
            If oElement.AsTextElement.TextStyle.BorderAndBackgroundVisible = True Then
            
                ' Process the text elements
                Call reCreateElement(oElement)
            End If
        End If
        
        If oElement.Type = msdElementTypeTextNode Then
        
            ' Process the text node elements
            Call reCreateElement(oElement)
        End If
    Loop

End Sub

'-----------------------------------------------------------------------+
'
'   name        ElementsProcess
'
'   author      BSI                                             08/2003
'
'-----------------------------------------------------------------------+
Sub ElementsProcess()

    Dim oElement As Element
    Dim ElementID As DLong
        
    ' Get the element id from the last created element
    ElementID = ActiveDesignFile.GetLargestElementID
    
' This example VBA app will crash in this sub if it runs after an undo
' occurs. This 'On Error' will bypass the process
On Error GoTo errorHandler

    ' Get the last element placed
    Set oElement = ActiveModelReference.GetElementByID(ElementID)
    
    ' Do not modify the elements if a text, text node or non-graphic element
    ' has been created last
    If oElement.Type <> msdElementTypeText Then
        If oElement.Type <> msdElementTypeTextNode Then
        
            ' Scan for all text and text node elements
            ModelScanForText
        End If
    End If
    
errorHandler:

End Sub

'-----------------------------------------------------------------------+
'
'   name        StartApp
'
'   author      BSI                                             08/2003
'
'-----------------------------------------------------------------------+
Sub StartApp()

    ' Enables the Redraw class module
    Set oRedraw = New Redraw
    
    ' Creates or modifies the specified text style
    TextStyleCreate "NewStyle"
End Sub

'-----------------------------------------------------------------------+
'
'   name        ViewAttributesChange
'
'   author      BSI                                             08/2003
'
'-----------------------------------------------------------------------+
Sub ViewAttributesChange()

    Dim oView As View
    Dim i As Integer
    
    For i = 1 To 8
    
        Set oView = ActiveDesignFile.Views(i)
        
        If oView.IsOpen = True Then
        
            ' Turn on Fill
            oView.DisplaysFill = True
            
            ' Turn off grid
            oView.DisplaysGrid = False
            
            ' Redraw the view window
            oView.Redraw
        End If
    Next i

End Sub

'-----------------------------------------------------------------------+
'
'   name        OnProjectLoad
'
'   author      BSI                                             08/2003
'
' This subroutine automatically runs this VBA app when it is loaded.
'
'-----------------------------------------------------------------------+
Sub OnProjectLoad()
    Set oOpenClose = New OpenClose
End Sub
