Attribute VB_Name = "Module1"
Option Explicit

'-----------------------------------------------------------------------+
'                                                                       |
' Copyright 1993-2003, Bentley Systems, Incorporated                    |
' All rights reserved.                                                  |
'                                                                       |
' Limited permission is hereby granted to reproduce and modify this     |
' copyrighted material provided that the resulting code is used only in |
' conjunction with Bentley Systems products under the terms of the      |
' license agreement provided therein, and that this notice is retained  |
' in its entirety in any such reproduction or modification.             |
'                                                                       |
'----------------------------------------------------------------------*/
'-----------------------------------------------------------------------+
' formPosition
'
' Status: WORKING
'
' This example illustrates how to scan the active model for only text and
' text nodes, iterate through a text node and display the following attributes
' from the type 17 Text elements in VBAs Immediate window:

' Element ID
' Text value
' Style name
' Font name
' Font type
'
' Author: BSI                                                   05/2003
'
'-----------------------------------------------------------------------+
'-----------------------------------------------------------------------+
'
'   name        TextGetAttributes
'
'   author      BSI                                             05/2003
'
'-----------------------------------------------------------------------+
Sub TextGetAttributes(oText As TextElement)
    
    Dim textID          As Long
    Dim textValue       As String
    Dim textStyleName   As String
    Dim textFont        As String
    Dim textType        As Long
    Dim strTextType     As String
          
    ' Extract the element ID number
    textID = oText.ID.Low
          
    ' Extract the text value
    textValue = oText.Text
    
    ' Extract the text style name
    textStyleName = oText.TextStyle.Name
    
    ' If the text does not have a style, print "None"
    If textStyleName = "" Then
        textStyleName = "None"
    End If
    
    ' Extract the font
    textFont = oText.TextStyle.Font.Name
    
    ' Extract the text type
    textType = oText.TextStyle.Font.Type
    
    ' This gets the text type description from the type value
    Select Case oText.TextStyle.Font.Type
        Case 0
            strTextType = "Native MicroStation font"
        Case 1
            strTextType = "AutoCAD SHX font"
        Case 2
            strTextType = "Windows TrueType font"
        Case 3
            strTextType = "Unknown font type"
    End Select
    
    ' Print the info in the Immediate window
    Debug.Print "Text ID: " & CStr(textID) & ", Value: '" & textValue & "'" _
                & ", Style: '" & textStyleName & "'" & ", Font: '" & textFont _
                & "'" & ", Type: '" & strTextType & "'"
            
End Sub

'-----------------------------------------------------------------------+
'
'   name        ElementScanComplex
'
'   author      BSI                                             05/2003
'
'-----------------------------------------------------------------------+
Sub ElementScanComplex(oElement As ComplexElement)
    
    Dim oEnumerator As ElementEnumerator
    Dim oSubElement As Element

    Set oEnumerator = oElement.GetSubElements

    Do While oEnumerator.MoveNext
    
        Set oSubElement = oEnumerator.Current
        If oSubElement.IsTextElement = True Then
            TextGetAttributes oSubElement
        Else
            ShowError "Sub unit found is not a text element!"
            Exit Sub
        End If
              
        If oSubElement.IsComplexElement Then
            ElementScanComplex oSubElement
        End If
    Loop
    
End Sub

'-----------------------------------------------------------------------+
'
'   name        ScanModel
'
'   author      BSI                                             05/2003
'
'-----------------------------------------------------------------------+
Sub ScanModel()

    Dim oEnumerator As ElementEnumerator
    Dim oScanCriteria As New ElementScanCriteria
    Dim oText As TextElement
    Dim oTextNode As TextNodeElement
    
    ' Scan only for text and text nodes
    oScanCriteria.ExcludeAllTypes
    oScanCriteria.IncludeType msdElementTypeText
    oScanCriteria.IncludeType msdElementTypeTextNode
    
    Set oEnumerator = ActiveModelReference.Scan(oScanCriteria)
    
    While oEnumerator.MoveNext
        If oEnumerator.Current.Type = msdElementTypeText Then
            Set oText = oEnumerator.Current
            TextGetAttributes oText
        ElseIf oEnumerator.Current.Type = msdElementTypeTextNode Then
            Set oTextNode = oEnumerator.Current
            ElementScanComplex oTextNode
        End If
    Wend
End Sub
