Attribute VB_Name = "TextPlace"
Option Explicit

'-----------------------------------------------------------------------+
'                                                                       |
' Copyright 1993-2003, Bentley Systems, Incorporated                    |
' All rights reserved.                                                  |
'                                                                       |
' Limited permission is hereby granted to reproduce and modify this     |
' copyrighted material provided that the resulting code is used only in |
' conjunction with Bentley Systems products under the terms of the      |
' license agreement provided therein, and that this notice is retained  |
' in its entirety in any such reproduction or modification.             |
'                                                                       |
'----------------------------------------------------------------------*/
' This example illustrates how to place a text element at the same angle
' as an existing line element

'-----------------------------------------------------------------------+
' Public Variables
'-----------------------------------------------------------------------+
Public TextValue            As String
Public LineAngle            As Double
Public DataPointEnteredIs   As Integer
'-----------------------------------------------------------------------+
'-----------------------------------------------------------------------+
'
'   name        LineAngleGet
'
'   author      BSI                                             09/2003
'
'-----------------------------------------------------------------------+
Function LineAngleGet(oLine As LineElement) As Double

    Dim ZAngle As Double
    
    'Calculate the angle
    ZAngle = Atn2((oLine.EndPoint.Y - oLine.StartPoint.Y), oLine.EndPoint.X - oLine.StartPoint.X)
        
    LineAngleGet = ZAngle

End Function

'-----------------------------------------------------------------------+
'
'   name        IsLineFound
'
'   author      BSI                                             09/2003
'
'-----------------------------------------------------------------------+
Function IsLineFound(Point As Point3d, View As View) As Boolean

    Dim oLine           As LineElement
    Dim oElementGeneric As Element

     'Try to get a line if it was near the datapoint
    Set oElementGeneric = CommandState.LocateElement(Point, View, True)
    
    ' Only do this if an element was found
    If Not oElementGeneric Is Nothing Then
    
        'Get the component element only
        Set oElementGeneric = CommandState.GetLocatedElement(False)
           
        ' If the element selected is not a line don't rotate the cell
        If oElementGeneric.Type = msdElementTypeLine Then
            Set oLine = oElementGeneric
           
            ' This function will return the line element's rotation
            LineAngle = LineAngleGet(oLine)
            
            IsLineFound = True
            Exit Function
        End If
    End If
    IsLineFound = False
End Function

'-----------------------------------------------------------------------+
'
'   name        StartApp
'
'   author      BSI                                             10/2003
'
'-----------------------------------------------------------------------+
Sub StartApp()

    ' Initialize the form
    UserForm1.TextBox1.Text = ""
    
    DataPointEnteredIs = 0
    
    ' Start the IPrimitiveCommandEvents object
    CommandState.StartPrimitive New TextPlaceCommand
End Sub
