Attribute VB_Name = "EmailFile"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
'-----------------------------------------------------------------------+
'                                                                       |
' Copyright 1993-2001, Bentley Systems, Incorporated                    |
' All rights reserved.                                                  |
'                                                                       |
' Limited permission is hereby granted to reproduce and modify this     |
' copyrighted material provided that the resulting code is used only in |
' conjunction with Bentley Systems products under the terms of the      |
' license agreement provided therein, and that this notice is retained  |
' in its entirety in any such reproduction or modification.             |
'                                                                       |
'----------------------------------------------------------------------*/
'-----------------------------------------------------------------------+
' EmailFile
'
' Status: WORKING
'
' Date: 10/25/01
'
' This example illustrates how to send an email with an attachment in VBA
'
' Author: BSI                                                 10/25/01
'
'-----------------------------------------------------------------------+

'-----------------------------------------------------------------------+
'
'   Globals
'
'-----------------------------------------------------------------------+
Dim fileName As String
Dim vars     As String
Dim flag     As Integer
Const MESSAGE_SHOWADBOOK = 11
Const SESSION_SIGNON = 1

Private Declare Function GetEnvironmentVariable Lib "kernel32" Alias "GetEnvironmentVariableA" (ByVal lpName As String, ByVal lpBuffer As String, ByVal nSize As Long) As Long

'-----------------------------------------------------------------------+
'
'   name        elemNameFromNumber
'
'   author      BSI                                           10/2001
'
'-----------------------------------------------------------------------+
Public Function elemNameFromNumber(ByVal eType As Integer) As String
    
     
    Select Case eType
    Case 1: elemNameFromNumber = "Cell Library Header         "
    Case 2: elemNameFromNumber = "Cell Header                 "
    Case 3: elemNameFromNumber = "Line                        "
    Case 4: elemNameFromNumber = "Line String                 "
    Case 5: elemNameFromNumber = "Group Data                  "
    Case 6: elemNameFromNumber = "Shape                       "
    Case 7: elemNameFromNumber = "Text Node                   "
    Case 8: elemNameFromNumber = "Digitizer Setup Data        "
    Case 9: elemNameFromNumber = "Design File Header          "
    Case 10: elemNameFromNumber = "Level Symbology            "
    Case 11: elemNameFromNumber = "Curve                      "
    Case 12: elemNameFromNumber = "Complex String             "
    Case 13: elemNameFromNumber = "Conic                      "
    Case 14: elemNameFromNumber = "Complex Shape              "
    Case 15: elemNameFromNumber = "Ellipse                    "
    Case 16: elemNameFromNumber = "Arc                        "
    Case 17: elemNameFromNumber = "Text                       "
    Case 18: elemNameFromNumber = "Surface                    "
    Case 19: elemNameFromNumber = "Solid                      "
    Case 21: elemNameFromNumber = "BSpline Pole               "
    Case 22: elemNameFromNumber = "Point String               "
    Case 23: elemNameFromNumber = "Cone                       "
    Case 24: elemNameFromNumber = "BSpline Surface            "
    Case 25: elemNameFromNumber = "BSpline Boundary           "
    Case 26: elemNameFromNumber = "BSpline Knot               "
    Case 27: elemNameFromNumber = "BSplinr Curve              "
    Case 28: elemNameFromNumber = "BSpline Weight             "
    Case 33: elemNameFromNumber = "Dimension                  "
    Case 34: elemNameFromNumber = "Shared Cell Definition     "
    Case 35: elemNameFromNumber = "Shared Cell                "
    Case 36: elemNameFromNumber = "MultiLine                  "
    Case 37: elemNameFromNumber = "Tag                        "
    Case 38: elemNameFromNumber = "DgnStore Component         "
    Case 39: elemNameFromNumber = "DgnStore Header            "
    Case 51: elemNameFromNumber = " ARCH (Intergraph)         "
    Case 54: elemNameFromNumber = "SDS (Intergraph)           "
    Case 56: elemNameFromNumber = "LEDS, WMS                  "
    Case 60: elemNameFromNumber = "SIS                        "
    Case 61: elemNameFromNumber = "EWS SCREMANC               "
    Case 62: elemNameFromNumber = "EWS PANEL                  "
    Case 63: elemNameFromNumber = "EWS PDS                    "
    Case 66: elemNameFromNumber = "MicroStation Application   "
    Case 87: elemNameFromNumber = "Raster Header              "
    Case 88: elemNameFromNumber = "Raster Component           "
    Case 89: elemNameFromNumber = "Raster Data                "
    Case 90: elemNameFromNumber = "Raster Reference Attachment"
    Case 91: elemNameFromNumber = "Raster Reference Component "
    Case 96: elemNameFromNumber = "Level Table                "
    Case 97: elemNameFromNumber = "View Group Element         "
    Case 98: elemNameFromNumber = "View Element               "
    Case 99: elemNameFromNumber = "Level Mask Element         "
    Case 100: elemNameFromNumber = "Reference Attach Element  "
    Case 101: elemNameFromNumber = "Matrix Header             "
    Case 102: elemNameFromNumber = "Matrix Int Data           "
    Case 103: elemNameFromNumber = "Matrix Double Data        "
    Case 105: elemNameFromNumber = "Mesh Header               "
    Case Else: elemNameFromNumber = "Type                  " + CStr(eType)
    
    
    End Select
End Function

'-----------------------------------------------------------------------+
'
'   name        GetVariable
'
'   author      BSI                                           10/2001
'
'-----------------------------------------------------------------------+
Function GetVariable(Variable As String) As String

    Dim results As Long
    Dim tmp As String
    
    ' Size tmp
    tmp = Space$(1024)

    ' Make a call into kernel32.dll for GetEnvironmentVariable
    results = GetEnvironmentVariable(Variable, tmp, Len(tmp))
    
    If results = 0 Then
        'Failed returned nothing
    Else
        GetVariable = Left$(tmp, results)
    End If

End Function

'-----------------------------------------------------------------------+
'
'   name        scanFile
'
'   author      BSI                                           10/2001
'
'-----------------------------------------------------------------------+
Private Function scanFile() As String
    
    Dim oScanCriteria          As New ElementScanCriteria
    Dim oEnumerator            As ElementEnumerator
    Dim count                  As Integer
    Dim message                As String
    Dim elementCount(1 To 127) As Integer
    Dim i                      As Integer
       
    'Counter
    count = 0
        
    'clear out any previous scans
    oScanCriteria.Reset
                    
    'Include type 66 elements
    oScanCriteria.IncludeType 66
     
    'Set enumerator
    Set oEnumerator = ActiveModelReference.Scan(oScanCriteria)

    'Title
    message = message + "Elements Type          " + Chr(9) + Chr(9) + "Element ID" + Chr(9) + Chr(9) + "Level" + Chr(9) + Chr(9) + "Color" + Chr(9) + "Weight"
    message = message + Chr(10) + "========================================================================= " + Chr(10)
    
    'Step through every element in the file
    Do While oEnumerator.MoveNext
        With oEnumerator.Current
            
            count = count + 1
            If (.Type <> 0) Then
                
                'Count element types
                elementCount(.Type) = elementCount(.Type) + 1
                 
                'Create element data
                message = message + elemNameFromNumber(.Type) + Chr(9) + Chr(9)
                message = message + "  " + DLongToString(.ID) + Chr(9)
                
                ' Only extact these values if the elements are graphical
                If .IsGraphical = True Then
                
                    message = message + "  " + Chr(9) + CStr(.Level.Name) + Chr(9)
                    message = message + " " + CStr(.Color) + Chr(9)
                    message = message + " " + CStr(.LineWeight) + Chr(9)
                End If
                
                message = message + Chr(10)
            End If
              
        End With
    Loop
       
    message = message + "========================================================================== " + Chr(10)
    
    'Report element count
    For i = 1 To 127
        
        If (elementCount(i) <> 0) Then
            message = message + elemNameFromNumber(i) + Chr(9) + CStr(elementCount(i)) + Chr(10)
        End If
        
    Next i
    
    message = message + "========================================================================== " + Chr(10)
    message = message + "Total Elements = " + CStr(count)
 
    'Return report data
    scanFile = Left$(message, Len(message))
    
End Function

'-----------------------------------------------------------------------+
'
'   name        addressBtn_Click
'
'   author      BSI                                           10/2001
'
'-----------------------------------------------------------------------+
Private Sub addressBtn_Click()

    'Set to current message
    MAPIMessages1.MsgIndex = -1
    MAPIMessages1.AddressEditFieldCount = 1
    
    On Error GoTo errorFlag
        
    'Open Addrress Book
    MAPIMessages1.Action = MessagesActionConstants.mapShowAddressBook
     
    For i = 0 To MAPIMessages1.RecipCount - 1
       MAPIMessages1.RecipIndex = i
       
        If (TextBox1.Text = "" And i <> 1) Then
            TextBox1.Text = MAPIMessages1.RecipDisplayName
        Else
        
        'Display names in text box
        TextBox1.Text = TextBox1.Text & ";" & MAPIMessages1.RecipDisplayName
             
       End If
       
    Next i
    
    'Set index back to 0
    MAPIMessages1.RecipIndex = 0
   
   flag = 1

    Exit Sub

errorFlag:
    
    'Null out GoTo
    On Error GoTo 0

End Sub
'End Sub

'-----------------------------------------------------------------------+
'
'   name        EmailFileBTN_Click
'
'   author      BSI                                           10/2001
'
'-----------------------------------------------------------------------+
Private Sub EmailFileBTN_Click()
    
    Dim status As String
    Dim userName As String
    
    'Get user name from system
    userName = UCase(GetVariable("username"))
    
    On Error GoTo errorFlag
    
    'Set RecipAddress to TO field
    MAPIMessages1.RecipType = mapToList
   
    If (TextBox1.Text <> "" And flag <> 1) Then
    
        'Set email address
        MAPIMessages1.RecipAddress = TextBox1.Text

        'Resolve email address
        MAPIMessages1.AddressResolveUI = True
        MAPIMessages1.ResolveName
                
    End If
         
    'Set body of text
    If (userName <> "") Then
    
        'Add message to email
        MAPIMessages1.MsgNoteText = IIf(TextBox2.Text <> "", " " + Chr(10) + TextBox2.Text + Chr(10) + Chr(10), "  ") + Chr(10) + "From:" + _
                                        UCase(GetVariable("username")) + Chr(10) + Chr(10)
    Else
        If (TextBox2.Text <> "") Then
        
            'Add message to email
            MAPIMessages1.MsgNoteText = " " + Chr(10) + TextBox2.Text + Chr(10) + Chr(10)
        
        End If
    End If
    
    'Scan file and create a report
    MAPIMessages1.MsgNoteText = MAPIMessages1.MsgNoteText + scanFile() + Chr(10)
    
    'If we have an attachment
    If (fileName <> "") Then
       
       'Set Subject
        MAPIMessages1.MsgSubject = "File " + fileName
        
        'Set attachment
        MAPIMessages1.AttachmentPathName = fileName
        
    Else
        
        'Set Subject
        MAPIMessages1.MsgSubject = "Element Information for " + MicroStationDGN.ActiveDesignFile.Name
              
    End If

    'Send email
    MAPIMessages1.Send (False)
    
    'Log off email
    MAPISession1.SignOff
    
    'End program
    End
    
    Exit Sub
    
errorFlag:

    'Display error number in text field
    TextBox2.Text = Err
    
    'Null out GoTo
    On Error GoTo 0
    
    'log off email
    MAPISession1.SignOff

End Sub

'-----------------------------------------------------------------------+
'
'   name        OpenFileButton1_Click
'
'   author      BSI                                           10/2001
'
'-----------------------------------------------------------------------+
Private Sub OpenFileButton1_Click()
    
    'Open file dialog
    CommonDialog1.ShowOpen
    fileName = CommonDialog1.fileName
 
End Sub

'-----------------------------------------------------------------------+
'
'   name        TextBox1_KeyPress
'
'   author      BSI                                           10/2001
'
'-----------------------------------------------------------------------+
Private Sub TextBox1_KeyPress(ByVal KeyAscii As MSForms.ReturnInteger)

    'IF Ctrl k
    If KeyAscii = 11 Then
    
    'Set RecipAddress to TO field
    MAPIMessages1.RecipType = mapToList
    
    'Resolve recip address
    MAPIMessages1.RecipAddress = TextBox1.Text
    MAPIMessages1.AddressResolveUI = True
    MAPIMessages1.ResolveName
    
    'Update dialog box with recip name
    TextBox1.Text = MAPIMessages1.RecipDisplayName
    
    End If
    
End Sub

'-----------------------------------------------------------------------+
'
'   name        UserForm_Initialize
'
'   author      BSI                                           10/2001
'
'-----------------------------------------------------------------------+
Private Sub UserForm_Initialize()

    'Sign on to email
    MAPISession1.Action = SESSION_SIGNON
    MAPIMessages1.SessionID = MAPISession1.SessionID

    ' Compose a new email
    MAPIMessages1.Compose
    
End Sub
