Attribute VB_Name = "Main"
'-----------------------------------------------------------------------+
'                                                                       |
' Copyright 1993-2001, Bentley Systems, Incorporated                    |
' All rights reserved.                                                  |
'                                                                       |
' Limited permission is hereby granted to reproduce and modify this     |
' copyrighted material provided that the resulting code is used only in |
' conjunction with Bentley Systems products under the terms of the      |
' license agreement provided therein, and that this notice is retained  |
' in its entirety in any such reproduction or modification.             |
'                                                                       |
'----------------------------------------------------------------------*/
'-----------------------------------------------------------------------+
' ElementAccess
'
' Status: WORKING
'
' Date: 05/01/01
'
' This example will scan the current design file for all elements
' and write element information to an ODBC Access DataBase.
'
' If an ODBC Data Source does not exist, you are prompted to enter a
' an ODBC Data Source Name and a Access DataBase name to link to the
' ODBC Data Source.  The ODBC Data Source Name and Access DataBase are
' then created dynamically and element information is written to the
' database.
'
' Tools Required:
'   MicroSoft Common Dialog Control 6.0 (SP3)
'   MicroSoft ActiveX Data Objects 2.x Library
'   Microsoft ADO Ext 2.x for DLL and Security
'   Microsoft Access 8.0 Object Library
'
' Author: Scott Gabel                                     05/01/01
'
'-----------------------------------------------------------------------+

'-----------------------------------------------------------------------+
'
'   Globals
'
'-----------------------------------------------------------------------+
Public dbConnection     As New ADODB.Connection         'Current DataBase Connection
Public appAccess        As New Access.Application       'Used to open up MicroSoft Access

Sub Main()
    
    'Variable Declarations
    '-------------------------------------------
    Dim dlgSaveAs       As New CommonDialog             'Common dialog box control used to save new database
    Dim status          As Integer                      'Used for message boxes
    Dim DSname          As String                       'Name of ODBC data source to create, if any
    '-------------------------------------------
    
    'Ask for ODBC Data Source Name
    DSname = InputBox("Please enter an ODBC Data Source: ", "ODBC Data Source")
    If DSname = "" Then Exit Sub

    'If ODBC Data Source does not exists, give option to create one
    If Not ODBCdataSourceExist(DSname) Then
    
        'Ask whether or not to create ODBC Data Source
        status = MsgBox("There is an error while trying to open the ODBC Data Source: " & _
                         DSname & vbCrLf & "Would you like to create a new ODBC Data Source and link an Access DataBase to it?", _
                         vbYesNo + vbQuestion, "Open ODBC Data Source Error")
        If status = vbNo Then Exit Sub
                
        'Ask for New ODBC Data Source name, if cancel exit program
        DSname = InputBox("Enter the name of ODBC Data Source to create:", "Create ODBC Data Source")
        If DSname = "" Then Exit Sub
        
        'Ask for New Database name to associate to Data Source, use Common Dialog for a "Save As" dialog box
        dlgSaveAs.DialogTitle = "Link MicroSoft Access DataBase to " & DSname & " ODBC Data Source"
        dlgSaveAs.Filter = "Microsoft Access DataBase (*.mdb)"
        dlgSaveAs.DefaultExt = "mdb"
        dlgSaveAs.ShowSave
        If dlgSaveAs.FileName = "" Then Exit Sub
            
        
            '--------------------------------------------------
            'Create Database with pre-defined tables and fields
            '--------------------------------------------------
            Dim oCat As ADOX.Catalog
            Dim colName(1 To 7) As String
            Dim colType(1 To 7) As ADOX.DataTypeEnum
            Set oCat = New ADOX.Catalog
            
            'Create DataBase
            Call dbADOX_DataBaseCreate(dlgSaveAs.FileName)
            oCat.ActiveConnection = "Provider=Microsoft.Jet.OLEDB.4.0;Data Source=" & dlgSaveAs.FileName & ";Jet OLEDB:Engine Type=4;"
    
            'Define Field Names / Data Types
            colName(1) = "Element_Type"
            colName(2) = "Element_ID"
            colName(3) = "Element_Name"
            colName(4) = "Color"
            colName(5) = "Style"
            colName(6) = "Weight"
            colName(7) = "Level_Name"

            colType(1) = adInteger
            colType(2) = adInteger
            colType(3) = adVarWChar
            colType(4) = adInteger
            colType(5) = adInteger
            colType(6) = adInteger
            colType(7) = adVarWChar
    
            'Create Table
            Call dbADOX_TableCreate(oCat, "ElementInfo", colName(), colType())
            
            'Release Connection to Catalog
            Set oCat = Nothing
                        
            '--------------------------------------------------
        
        'Create New ODBC Data Source
        Call ODBCdataSourceCreate(DSname, dlgSaveAs.FileName)
    
    End If

    'Connect / open database
    dbConnection.ConnectionString = "Data Source=" + DSname
    dbConnection.Open
       
    'Scan current design file
    Call ScanWriteToDataBase
    
    'Ask Whether or not to Open MicroSoft Access
    status = MsgBox("Scan Complete.  Would you like to open the database for viewing?", _
                    vbYesNo, "Open MicroSoft Access")
    If status = vbYes Then
        'Open Access
        AccessID = Shell("C:\Program Files\Microsoft Office\Office\MSACCESS.EXE " _
                          & FileGetShortPath(dbConnection.DefaultDatabase + ".mdb"), 1)
        AppActivate AccessID
        
        'appAccess.OpenCurrentDatabase FileGetShortPath(dbConnection.DefaultDatabase + ".mdb")
    End If

    'Close ADO connection to database
    dbConnection.Close

End Sub

