Attribute VB_Name = "ScanForPipingData"
Option Explicit

'-----------------------------------------------------------------------+
'
'   Public variables
'
'-----------------------------------------------------------------------+
Public pipeFound()      As String 'Array of pipe sizes.
Public pipeCountTotal   As Integer 'Total count of all pipes found.
Public pipeUse()        As String ' Array of pipe uses (i.e. Cold Water Supply)
Public pipeLength()     As Double ' Array that contains each pipes length

'-----------------------------------------------------------------------+
'
'   name        GetPipeLength
'
'   author      BSI                                             10/2002
'
'-----------------------------------------------------------------------+
Function GetPipeLength(oElement) As Double

    Dim oLine       As LineElement
    Dim getRounded  As Double
    
    Set oLine = oElement
    
    ' Use VBs Round function to limit the decimal places to 2
    getRounded = (Round(oLine.length, 2))
    GetPipeLength = getRounded
End Function

'-----------------------------------------------------------------------+
'
'   name        IsLStyleObject
'
'   author      BSI                                             12/2002
'
'-----------------------------------------------------------------------+
Function IsLStyleObject() As Boolean
    Dim obj As Object
    Dim oSettings As Object
    
    Set oSettings = ActiveSettings
    
    On Error GoTo IsLong
    
    Set obj = oSettings.lineStyle
    IsLStyleObject = True
    Exit Function
    
IsLong:
    Exit Function
End Function

'-----------------------------------------------------------------------+
'
'   name        GetLStyle
'
'   author      BSI                                             12/2002
'
'-----------------------------------------------------------------------+
Function GetLStyle(oElement As Element) As Long
    Dim oEl As Object
    Dim oLS As Object
    
    Set oEl = oElement
    
    If IsLStyleObject Then
        GetLStyle = oEl.Level.ElementLineStyle.Number
    Else
        GetLStyle = oEl.Level.ElementLineStyle
    End If
End Function

'-----------------------------------------------------------------------+
'
'   name        GetLineStyle
'
'   author      BSI                                             10/2002
'
'-----------------------------------------------------------------------+
Function GetLineStyle(oElement As Element) As Integer

    Dim bLSOAvailable As Boolean
    Dim oLineStyle As Object
    Dim oElemHasLS As Object
    bLSOAvailable = True
    Dim test As Long
 
    Select Case oElement.LineStyleSource

        ' ByLevel
        Case msdValueSourceLevel
            GetLineStyle = GetLStyle(oElement)
            Exit Function

        ' ByElement
        Case msdValueSourceLocal
            GetLineStyle = oElement.LineStyleSource
            Exit Function

        Case Else
            MsgBox "The linestyle for an element could not be found"
            Exit Function
    End Select
    
End Function

'-----------------------------------------------------------------------+
'
'   name        ScanForLineStylesInRef
'
'   author      BSI                                             10/2002
'
'-----------------------------------------------------------------------+
Sub ScanForLineStylesInRef(oAttachment As Attachment)
    
    Dim oScanCriteria       As New ElementScanCriteria
    Dim oEnumerator         As ElementEnumerator
    Dim coldWaterSupply     As Long
    Dim hotWaterSupply      As Long
    Dim foundLineStyle      As Boolean
    Dim currentPipeLgth     As Double
    Dim oElement            As Element
    Dim oTag()              As TagElement
    Dim count               As Integer
    Dim pipeSize            As String
    Dim lineStyle           As String
    Dim currentPipeFound    As String
    
    ' Initialize the variable
    foundLineStyle = False
    
    ' Initializes the public variables
    ReDim pipeFound(0)
    ReDim pipeUse(0)
    ReDim pipeLength(0)
    pipeCountTotal = 0
    
    coldWaterSupply = 4
    hotWaterSupply = 6
    
    ' Exclude all element types
    oScanCriteria.ExcludeAllTypes
    
    ' Include only lines and linestrings
    oScanCriteria.IncludeType msdElementTypeLine
    
    Set oEnumerator = oAttachment.Scan(oScanCriteria)

    Do While oEnumerator.MoveNext
        
        Set oElement = oEnumerator.Current
        
        ' Determines if the linestyle is one were looking for
        Select Case (GetLineStyle(oElement))
        Case 4
            lineStyle = "Cold Water Supply"
            foundLineStyle = True
        Case 6
            lineStyle = "Hot Water Supply"
            foundLineStyle = True
        Case Else
            foundLineStyle = False
        End Select
       
        If foundLineStyle = True Then
        
            ' Determines if the line has a tag attached were looking for
            If oElement.HasAnyTags = True Then
                oTag() = oElement.GetTags
                
                ' Increment number of pipes found
                pipeCountTotal = pipeCountTotal + 1
                
                For count = 0 To UBound(oTag)
                    If oTag(count).TagDefinitionName = "Size" Then
                        
                        ' Gets the size of the pipe
                        currentPipeFound = oTag(count).Value
                        
                        ' Gets the length of the pipe
                        currentPipeLgth = GetPipeLength(oElement)
                         
                        ' ---- Sets the array with the size of the pipe ----
                        ' Redimension pipeFound array to and add this pipe
                        ReDim Preserve pipeFound(pipeCountTotal)
                         
                        ' Populate currentPipeFound to pipeFound array
                        pipeFound(pipeCountTotal) = currentPipeFound
                        
                        ' ---- Sets the correct array with the use of the pipe ----
                        ' Redimension pipeUse array to and add this pipe
                        ReDim Preserve pipeUse(pipeCountTotal)
                        
                        ' Populate lineStyle to pipeUse array
                        pipeUse(pipeCountTotal) = lineStyle
                        
                        ' ---- Sets the array with the length of the pipe ----
                        ' Redimension pipeLength array to and add this pipe
                        ReDim Preserve pipeLength(pipeCountTotal)
                        
                        ' Populate currentPipeLgth to pipeLength array
                        pipeLength(pipeCountTotal) = currentPipeLgth
                    End If
                Next
            End If
        End If
    Loop
    
    ' Displays a message if piping is found or not
    If pipeCountTotal <> 0 Then
        
        ' Update prompt
        ShowStatus "Scan Complete"
    Else
        ' Update prompt
        ShowStatus "No piping found"
    End If

End Sub
