Attribute VB_Name = "WdModule"
'-----------------------------------------------------------------------+
'                                                                       |
' Copyright 1993-2001, Bentley Systems, Incorporated                    |
' All rights reserved.                                                  |
'                                                                       |
' Limited permission is hereby granted to reproduce and modify this     |
' copyrighted material provided that the resulting code is used only in |
' conjunction with Bentley Systems products under the terms of the      |
' license agreement provided therein, and that this notice is retained  |
' in its entirety in any such reproduction or modification.             |
'                                                                       |
'----------------------------------------------------------------------*/
'-----------------------------------------------------------------------+
' WordExample
'
' Status: WORKING
'
' Date: 04/24/01
'
' This example scans design file and lists element name, unique ID, color,
' weight, level, and total number of elements found to a Word document.
'
' Author: 1214                                            04/24/01
'
'-----------------------------------------------------------------------+
'-----------------------------------------------------------------------+
'
'   name        FileGetShortPath
'
'   author      BSI                              04/2001
'
'-----------------------------------------------------------------------+
Function FileGetShortPath(filespec) As String
   
    Dim fs As Object
    Dim f  As Object
    
    ' File object
    Set fs = CreateObject("Scripting.FileSystemObject")
    Set f = fs.GetFile(filespec)
    ' ShortPath property
    FileGetShortPath = f.ShortPath
    
End Function
'-----------------------------------------------------------------------+
'
'   name        elemNameFromNumber
'
'   author      BSI                                             03/2001
'
'-----------------------------------------------------------------------+
Public Function GetElementTypeName(ByVal eType As Integer) As String
    
     
    Select Case eType
    Case 1: GetElementTypeName = "Cell Library Header         "
    Case 2: GetElementTypeName = "Cell Header                 "
    Case 3: GetElementTypeName = "Line                        "
    Case 4: GetElementTypeName = "Line String                 "
    Case 5: GetElementTypeName = "Group Data                  "
    Case 6: GetElementTypeName = "Shape                       "
    Case 7: GetElementTypeName = "Text Node                   "
    Case 8: GetElementTypeName = "Digitizer Setup Data        "
    Case 9: GetElementTypeName = "Design File Header          "
    Case 10: GetElementTypeName = "Level Symbology            "
    Case 11: GetElementTypeName = "Curve                      "
    Case 12: GetElementTypeName = "Complex String             "
    Case 13: GetElementTypeName = "Conic                      "
    Case 14: GetElementTypeName = "Complex Shape              "
    Case 15: GetElementTypeName = "Ellipse                    "
    Case 16: GetElementTypeName = "Arc                        "
    Case 17: GetElementTypeName = "Text                       "
    Case 18: GetElementTypeName = "Surface                    "
    Case 19: GetElementTypeName = "Solid                      "
    Case 21: GetElementTypeName = "BSpline Pole               "
    Case 22: GetElementTypeName = "Point String               "
    Case 23: GetElementTypeName = "Cone                       "
    Case 24: GetElementTypeName = "BSpline Surface            "
    Case 25: GetElementTypeName = "BSpline Boundary           "
    Case 26: GetElementTypeName = "BSpline Knot               "
    Case 27: GetElementTypeName = "BSplinr Curve              "
    Case 28: GetElementTypeName = "BSpline Weight             "
    Case 33: GetElementTypeName = "Dimension                  "
    Case 34: GetElementTypeName = "Shared Cell Definition     "
    Case 35: GetElementTypeName = "Shared Cell                "
    Case 36: GetElementTypeName = "MultiLine                  "
    Case 37: GetElementTypeName = "Tag                        "
    Case 38: GetElementTypeName = "DgnStore Component         "
    Case 39: GetElementTypeName = "DgnStore Header            "
    Case 51: GetElementTypeName = " ARCH (Intergraph)         "
    Case 54: GetElementTypeName = "SDS (Intergraph)           "
    Case 56: GetElementTypeName = "LEDS, WMS                  "
    Case 60: GetElementTypeName = "SIS                        "
    Case 61: GetElementTypeName = "EWS SCREMANC               "
    Case 62: GetElementTypeName = "EWS PANEL                  "
    Case 63: GetElementTypeName = "EWS PDS                    "
    Case 66: GetElementTypeName = "MicroStation Application"
    Case 87: GetElementTypeName = "Raster Header              "
    Case 88: GetElementTypeName = "Raster Component           "
    Case 89: GetElementTypeName = "Raster Data                "
    Case 90: GetElementTypeName = "Raster Reference Attachment"
    Case 91: GetElementTypeName = "Raster Reference Component "
    Case 96: GetElementTypeName = "Level Table                "
    Case 97: GetElementTypeName = "View Group Element"
    Case 98: GetElementTypeName = "View Element               "
    Case 99: GetElementTypeName = "Level Mask Element         "
    Case 100: GetElementTypeName = "Reference Attach Element"
    Case 101: GetElementTypeName = "Matrix Header             "
    Case 102: GetElementTypeName = "Matrix Int Data           "
    Case 103: GetElementTypeName = "Matrix Double Data        "
    Case 105: GetElementTypeName = "Mesh Header               "
    Case Else: GetElementTypeName = "NONE"
    
    
    End Select
End Function
Sub Main()

    Const totElemTypes As Integer = 127
    
    Dim WordSht As New Word.Application
    Dim elem As Element
    Dim elemName As String
    Dim name As String
    Dim colValue As Long
    Dim wtValue As Long
    Dim styValue As Long
    Dim reply As Integer
    Dim Count As Integer
    Dim elemCount(1 To 127) As Integer
    Dim totCount As Integer
    Dim ScanCriteria As New ElementScanCriteria
    Dim saveas As New CommonDialog
    Dim Enumer As ElementEnumerator
    
    'Creates a Word document.
    WordSht.Documents.Add
    'Initialize counter.
    totCount = 0

    ScanCriteria.Reset
    
    Set Enumer = Application.ActiveModelReference.Scan(ScanCriteria)

    'Create column headings.
    WordSht.selection.TypeText "Element Type" + VBA.Chr(9) + VBA.Chr(9) + "Element ID" + VBA.Chr(9) _
                                + "Color" + VBA.Chr(9) + "Weight" + VBA.Chr(9) + "  " + "LineStyle" _
                                + "  Level" + vbCrLf
    WordSht.selection.TypeText "------------" + VBA.Chr(9) + VBA.Chr(9) + "----------" + VBA.Chr(9) _
                                + "-----" + VBA.Chr(9) + "------" + VBA.Chr(9) + "  ---------" _
                                + "      " + "-----" + vbCrLf
                            
    'Loop continues until no more elements are found.
    Do While Enumer.MoveNext
        
        'Grab current element
        Set elem = Enumer.Current
        
        'Function call to determine element name.
        elemName = GetElementTypeName(elem.Type)
    
        'Determines color, weight, level, and unique ID as long as element is found.
        If elemName <> "NONE" Then
            If elem.IsGraphical Then
                'Checks for ByLevel value, else use element value.
                If elem.ColorSource = msdValueSourceLevel Then
                    colValue = elem.Level.ElementColor
                Else
                    colValue = elem.Color
                End If
                If elem.LineWeightSource = msdValueSourceLevel Then
                    wtValue = elem.Level.ElementLineWeight
                Else
                    wtValue = elem.LineWeight
                End If
                                
                If elem.LineStyleSource = msdValueSourceLevel Then
                    styValue = elem.Level.ElementLineStyle
                Else
                    styValue = elem.LineStyle.Number
                End If
    
                WordSht.selection.TypeText elemName + VBA.Chr(9) + DLongToString(elem.ID) + VBA.Chr(9) + VBA.Chr(9) + _
                                        CStr(colValue) + VBA.Chr(9) + CStr(wtValue) + VBA.Chr(9) + "  " + _
                                        CStr(styValue) + VBA.Chr(9) + "  " + "     " + CStr(elem.Level.name) + vbCrLf
                    
                totCount = totCount + 1
            
                elemCount(elem.Type) = elemCount(elem.Type) + 1
            End If
        End If
            
    Loop

    WordSht.selection.TypeText vbCrLf + vbCrLf

    Count = 1

    Do While Count <= totElemTypes
    
        If elemCount(Count) > 0 Then
        
            name = GetElementTypeName(Count)
            WordSht.selection.TypeText name + VBA.Chr(9) + CStr(elemCount(Count)) + vbCrLf
        
        End If
        Count = Count + 1
    Loop
    WordSht.selection.TypeText "===================" + vbCrLf
    WordSht.selection.TypeText "Total Elements - " + CStr(totCount)
    
    
    
    saveas.DialogTitle = "Save As a Word Document "
    saveas.Filter = "Microsoft Word Document(*.doc);*.doc"
    saveas.DefaultExt = "doc"
    saveas.ShowSave

    If saveas.FileName <> "" Then
 
        'Either create new sheet or overwrite existing one
        WordSht.ActiveDocument.saveas saveas.FileName
                
        ' prompt user to view excel spreadsheet
        reply = MsgBox("Would you like to open " & saveas.FileName & " for viewing?", _
                vbInformation + vbYesNo + vbDefaultButton1, "open Word")
            
    Else
    
        reply = vbCancel
        WordSht.Application.Quit False
        Set WordSht = Nothing
    
    End If
    
    If reply = vbYes Then
        
        ' handles spaces and long filenames
        saveas.FileName = FileGetShortPath(saveas.FileName)
            
        WordSht.Visible = True
        WordSht.Application.Activate
        
    Else
        
        ' Close Word with the Quit method on the Application object.
        WordSht.Application.Quit
                

    End If
                
    ' Release the object variable.
    Set WordSht = Nothing
   

End Sub




