Attribute VB_Name = "modLine2xls"
Option Explicit
'-----------------------------------------------------------------------+
'                                                                       |
' Copyright 1993-2001, Bentley Systems, Incorporated                    |
' All rights reserved.                                                  |
'                                                                       |
' Limited permission is hereby granted to reproduce and modify this     |
' copyrighted material provided that the resulting code is used only in |
' conjunction with Bentley Systems products under the terms of the      |
' license agreement provided therein, and that this notice is retained  |
' in its entirety in any such reproduction or modification.             |
'                                                                       |
'----------------------------------------------------------------------*/
'-----------------------------------------------------------------------+
' line2xls
'
' Status: WORKING
'
' Date: 03/02/01
'
' The example processes all line elements.
' It extracts level, color, & weight values into a new excel spreadsheet.
' It also provides an option to view spreadsheet.
'
' Tools Required:
'   MicroSoft Excel 8.0 Object Library
'   MicroSoft Common Dialog Control
'   MicroSoft Forms
'
' Author: Mary Hamric                                            03/03/01
'
'-----------------------------------------------------------------------+
'
'  keyins to execute:
'  vba load line2xls    'loads project if not already loaded
'  vba run [line2xls]main
'
'---------------------------------------------------------------------


'-----------------------------------------------------------------------+
'
'   name        main
'
'   author      BSI                                             03/2001
'
'-----------------------------------------------------------------------+
Sub main()
    Dim xlsheet       As New Excel.Application
    Dim wbkWorkbook   As Excel.Workbook
    Dim wksWorksheet  As Excel.Worksheet
    Dim rngHeader     As Excel.Range
    Dim response      As Integer
    Dim oScanCriteria As New ElementScanCriteria
    Dim oEnumerator   As ElementEnumerator
    Dim count         As Integer
    Dim row           As Integer
    Dim saveas        As New CommonDialog
    Dim startpnt      As Point3d
    Dim endpnt        As Point3d
    
    Set wbkWorkbook = xlsheet.Workbooks.Add

  ' Return reference to first worksheet.
    Set wksWorksheet = wbkWorkbook.Worksheets(1)
    ' Return reference to ranges in this workbook.
    Set rngHeader = wksWorksheet.Rows(1)

    ' prompt user to scan for line elements in master file
    response = MsgBox("Put Symbology Values" + Chr$(10) + "into excel spreadsheet?", vbInformation + vbOKCancel + vbDefaultButton1, "Scan For Lines")
    
    If response = vbOK Then
    
        ' Start Microsoft Excel and create a new Worksheet object.
        'Set xlsheet = CreateObject("Excel.Sheet")
        
        rngHeader.Cells(1, 1).Value = "line count"
        rngHeader.Cells(1, 2).Value = "Element ID"
        rngHeader.Cells(1, 3).Value = "Level"
        rngHeader.Cells(1, 4).Value = "Color"
        rngHeader.Cells(1, 5).Value = "Weight"
        rngHeader.Cells(1, 6).Value = "Start X"
        rngHeader.Cells(1, 7).Value = "Start Y"
        rngHeader.Cells(1, 8).Value = "Start Z"
        rngHeader.Cells(1, 9).Value = "End X"
        rngHeader.Cells(1, 10).Value = "End Y"
        rngHeader.Cells(1, 11).Value = "End Z"
        
        'Initializations
        '-------------------------------------------------
        count = 0
        
        '  Initially, the scan criteria is set up to handle all types.
        '  First, exclude all and then just include the ones we want
        oScanCriteria.ExcludeAllTypes
        
        '  scanning for type 3 line elements only
        oScanCriteria.IncludeType msdElementTypeLine
        
        Set oEnumerator = ActiveModelReference.Scan(oScanCriteria)

        ' step through every line element in the file
        Do While oEnumerator.MoveNext
            With oEnumerator.Current
                count = count + 1
                row = count + 1
                    
                ' add element symbology values into next row of excel spreadsheet
                rngHeader.Cells(row, 1).Value = count
                rngHeader.Cells(row, 2).Value = DLongToString(.ID)
                rngHeader.Cells(row, 3).Value = .Level.ID
                rngHeader.Cells(row, 4).Value = .Color
                rngHeader.Cells(row, 5).Value = .LineWeight

                startpnt = .AsLineElement.StartPoint
                endpnt = .AsLineElement.EndPoint

                rngHeader.Cells(row, 6).Value = Round(startpnt.X, 4)
                rngHeader.Cells(row, 7).Value = Round(startpnt.Y, 4)
                rngHeader.Cells(row, 8).Value = Round(startpnt.Z, 4)
                rngHeader.Cells(row, 9).Value = Round(endpnt.X, 4)
                rngHeader.Cells(row, 10).Value = Round(endpnt.Y, 4)
                rngHeader.Cells(row, 11).Value = Round(endpnt.Z, 4)

            End With
        Loop
        ' display status message with the total number of lines scanned
        ShowStatus "line2xls: number of line elements :" & count
                
        ' use Common Dialog for a "Save As" dialog box
        saveas.DialogTitle = "Save As an Excel Spreadsheet"
        saveas.Filter = "Microsoft Excel Workbook(*.xls);*.xls"
        saveas.DefaultExt = "xls"
        saveas.ShowSave
        
        If saveas.FileName <> "" Then
 
           ' xlsheet.saveas saveas.FileName
           'wbkWorkbook.saveas saveas.FileName
           xlsheet.ActiveWorkbook.saveas saveas.FileName
           ' prompt user to view excel spreadsheet
           response = MsgBox("Would you like to open " & saveas.FileName & " for viewing?", vbInformation + vbOKCancel + vbDefaultButton1, "open excel")
            
        Else
        
            response = vbCancel
            
        End If
    
    
        If response = vbOK Then
            
            ' view the file saveas.FileName
            xlsheet.Visible = True
            
        Else
        
            ' Close Excel with the Quit method on the Application object.
            xlsheet.Quit
        End If
       
    End If
    ' Release the object variable.
    Set xlsheet = Nothing
    Set wbkWorkbook = Nothing
    Set wksWorksheet = Nothing
    Set rngHeader = Nothing
 
End Sub
