VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "DevsptClass"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit
Option Compare Text

Dim WithEvents hooks As Application
Attribute hooks.VB_VarHelpID = -1
Public sScriptFiles As String
Public sHelp As String
Public gbNotify As Boolean

Private Sub Class_Initialize()
    
    ' Register project load and application event handlers here.
    Set hooks = Application
    
    ' ==> Additional Global messaging
    gbNotify = True
    
    ' Check setup
    If CfgVarExists("MS_VBA_FILEOPEN") = False And CfgVarExists("MS_VBA_FILECLOSE") = False Then
        Dim sHelp As String
        sHelp = "To process MicroStation Keyin Script files on design file open and close events " & _
                "add one or more script files to the MicroStation environment variable MS_VBA_FILEOPEN " & _
                "and/or MS_VBA_FILECLOSE"
        ShowMessage "Additional Setup Information for RunVBA", sHelp, msdMessageCenterPriorityInfo
        Exit Sub
    End If
    
    
End Sub

'----------------------------------------------------------------------+
'
'   Name        ArgFind
'
'   Author      BSI - Developer Support                 10/2003
'
'----------------------------------------------------------------------+
'
'   Description:    Return the position of a matching argument, or 0.
'
'----------------------------------------------------------------------+
Public Function ArgFind(sMatch As String) As Integer
    
    Dim arArgs() As String
    Dim iCount As Integer
    Dim sArg
    
    ArgFind = False
    
    ' Get command line arguments
    arArgs = Split(KeyinArguments)
    
    ' Try to match sMatch in command line argument array
    For Each sArg In arArgs
        iCount = iCount + 1
        If sArg = sMatch Then
            ArgFind = iCount
            Exit For
        End If
    Next

End Function


'-----------------------------------------------------------------------+
'
'   Name        CfgVarExists
'
'   Author      BSI - Developer Support                 10/2003
'
'-----------------------------------------------------------------------+
Public Function CfgVarExists(sCfgVar As String) As String
    '("$(" & sCfgVar & ")")
    CfgVarExists = ActiveWorkspace.IsConfigurationVariableDefined(sCfgVar)
    
End Function

'-----------------------------------------------------------------------+
'
'   Name        CfgVarExpand
'
'   Author      BSI - Developer Support                 10/2003
'
'-----------------------------------------------------------------------+
Public Function CfgVarExpand(sCfgVar As String) As String
        
    CfgVarExpand = ActiveWorkspace.ExpandConfigurationVariable("$(" & sCfgVar & ")")
    
End Function

'-----------------------------------------------------------------------+
'
'   Name        FileExists
'
'   author      BSI - Developer Support                 08/2001
'
'-----------------------------------------------------------------------+
Public Function FileExists(sFileName) As String

    Dim oFS, oFile
    Dim sFileSpec As String
    
    Set oFS = CreateObject("Scripting.FileSystemObject")
    Set oFile = oFS.GetFile(sFileName)
    sFileSpec = oFile.ShortPath
    If Dir(sFileSpec) <> "" Then
        FileExists = sFileSpec
    End If
    Set oFS = Nothing
    Set oFile = Nothing

End Function

'-----------------------------------------------------------------------+
'
'   Name        ScriptFilesProcess
'
'   Author      BSI - Developer Support                 10/2003
'
'-----------------------------------------------------------------------+
Public Function ScriptFilesProcess(sFileList As String) As Integer
    
    Dim arFiles() As String
    Dim sFile, sFileSpec As String
    Dim iCount As Integer
            
    ' Validate input
    If sFileList = "" Then
        Exit Function
    End If
    
    ' Get command line arguments
    arFiles = Split(sFileList, ";")
    
    ' Try to match sMatch in command line argument array
    For Each sFile In arFiles
        ' Validate file
        sFileSpec = FileExists(sFile)
        If sFileSpec = "" Then
            ShowMessage "File not found", "File Not Found: " & sFile, msdMessageCenterPriorityError
            Exit Function
        End If
        
        ' Have MicroStation process script file
        iCount = iCount + 1
        CadInputQueue.SendCommand "@" & sFileSpec
    Next
    
    ScriptFilesProcess = iCount

End Function

'-----------------------------------------------------------------------+
'
'   Name        OnDesignFileClosed
'
'   Author      BSI - Developer Support                 10/2003
'
'-----------------------------------------------------------------------+
Private Sub hooks_OnDesignFileClosed(ByVal sDesignFileName As String)
    
    ' Request script files to process
    sScriptFiles = CfgVarExpand("MS_VBA_FILECLOSE")
    
    ' Process script files
    If sScriptFiles = "" Then
        If gbNotify = True Then
            ShowMessage "No MicroStation Keyin Script files to process", msdMessageCenterPriorityError
        End If
        Exit Sub
    End If
    
    If ScriptFilesProcess(sScriptFiles) = 0 Then
        If gbNotify = True Then
            ShowMessage "No design file close scripts were processed", "Script files not processed: " & _
            sScriptFiles, msdMessageCenterPriorityError
        End If
    End If

End Sub

'-----------------------------------------------------------------------+
'
'   Name        OnDesignFileOpened
'
'   Author      BSI - Developer Support                 10/2003
'
'-----------------------------------------------------------------------+
Private Sub hooks_OnDesignFileOpened(ByVal sDesignFileName As String)
    
    ' Request script files to process
    If CfgVarExists("MS_VBA_FILEOPEN") = False Then
        ShowMessage "Additional Setup Information for RunVBA", sHelp, msdMessageCenterPriorityInfo
        Exit Sub
    End If
    
    sScriptFiles = CfgVarExpand("MS_VBA_FILEOPEN")
    
    ' Process script files
    If sScriptFiles = "" Then
        If gbNotify = True Then
            ShowMessage "No MicroStation Keyin Script files to process", msdMessageCenterPriorityError
        End If
        Exit Sub
    End If
    
    If ScriptFilesProcess(sScriptFiles) = 0 Then
        If gbNotify = True Then
            ShowMessage "No design file open scripts were processed", "Script files not processed: " & _
            sScriptFiles, msdMessageCenterPriorityError
        End If
    End If
    
End Sub

